@@ -127,7 +127,7 @@ public final class RandomReads3 {
 				//do nothing
 			}else if(a.equals("quantize") || a.equals("quantizesticky")){
 				quantizeQuality=Quantizer.parse(arg, a, b);
-			}else if(a.equals("coverage")){
+			}else if(a.equals("coverage") || a.equals("depth")){
 				targetCov=Float.parseFloat(b);
 			}else if(a.equals("simplenames") || a.equals("simple") || a.equals("tagsimple")){
 				FASTQ.TAG_CUSTOM_SIMPLE=Parse.parseBoolean(b);
@@ -348,6 +348,8 @@ public final class RandomReads3 {
 				FASTQ.SPACE_SLASH=spaceslash;
 			}else if(a.equals("in")){
 				in1=(b==null || b.equalsIgnoreCase("null") ? null : b);
+			}else if(Parser.parseCommonStatic(arg, a, b)){
+				//Flag was captured by the parser; do nothing
 			}else{throw new RuntimeException("Unknown parameter "+args[i]);}
 
 		}
