@@ -4,9 +4,11 @@ import java.io.File;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.LinkedHashSet;
 import java.util.Map.Entry;
 
 import align2.BandedAlignerConcrete;
+import dna.AminoAcid;
 import fileIO.ByteFile;
 import fileIO.ByteStreamWriter;
 import fileIO.FileFormat;
@@ -14,6 +16,7 @@ import fileIO.TextFile;
 import shared.LineParser2;
 import shared.Tools;
 import structures.ByteBuilder;
+import structures.LongList;
 
 public class BarcodeStats {
 	
@@ -25,7 +28,7 @@ public class BarcodeStats {
 		String expected=(args.length>3 ? args[3] : null);
 		
 		BarcodeStats bs=new BarcodeStats(delimiter, codesPerRead, null);
-		if(expected!=null) {bs.loadBarcodeList(expected);}
+		if(expected!=null) {bs.loadBarcodeList(expected, delimiter, false, false);}
 		bs.loadBarcodeCounts(fname);
 		if(codesPerRead>1) {
 			bs.leftStats=bs.makeLeft();
@@ -59,7 +62,54 @@ public class BarcodeStats {
 	/*----------------    Initialization Methods    ----------------*/
 	/*--------------------------------------------------------------*/
 	
-	public int loadBarcodeList(String fname){
+
+	
+	/**
+	 * Loads a set of barcodes from a set of barcodes or filenames.
+	 * They can have a tab after them with more stuff, which will be trimmed.
+	 * Modifies the input set.
+	 */
+	public static LinkedHashSet<String> loadBarcodeSet(LinkedHashSet<String> barcodeSet,
+			byte barcodeDelimiter, boolean rcIndex1, boolean rcIndex2){
+		if(barcodeSet==null || barcodeSet.isEmpty()) {return null;}
+		int barcodeLength=0;
+		
+		//Handle names
+		@SuppressWarnings("unchecked")
+		LinkedHashSet<String> tempSet=new LinkedHashSet<String>(barcodeSet);
+		barcodeSet.clear();
+		for(String s : tempSet) {
+			File f=new File(s);
+			if(f.exists() && f.isFile()){//Read the file
+				TextFile tf=new TextFile(s);
+				String[] lines=tf.toStringLines();
+				for(String line : lines){
+//					assert(false) : "'"+Character.toString(barcodeDelimiter)+"'="+((int)(barcodeDelimiter));
+					if(barcodeDelimiter>0 || rcIndex1 || rcIndex2) {
+						line=BarcodeStats.fixBarcode(line, barcodeDelimiter, rcIndex1, rcIndex2);
+						assert(barcodeDelimiter<1 || line.indexOf(barcodeDelimiter)>=0) : 
+							line+", "+barcodeDelimiter;
+					}
+					assert(barcodeLength==0 || barcodeLength==line.length()) : barcodeLength+", "+line.length()+
+						", "+"'"+Character.toString(barcodeDelimiter<0 ? '0' : barcodeDelimiter)+"'="+((int)(barcodeDelimiter))+"\n"+line;
+					barcodeLength=line.length();
+					barcodeSet.add(line);
+				}
+			}else{
+				if(barcodeDelimiter>0 || rcIndex1 || rcIndex2) {
+					s=BarcodeStats.fixBarcode(s, barcodeDelimiter, rcIndex1, rcIndex2);
+					assert(barcodeDelimiter<1 || s.indexOf(barcodeDelimiter)>=0) : 
+						s+", "+barcodeDelimiter;
+				}
+				assert(barcodeLength==0 || barcodeLength==s.length());
+				barcodeLength=s.length();
+				barcodeSet.add(s);
+			}//Re-add each key as a literal name
+		}
+		return barcodeSet;
+	}
+	
+	public int loadBarcodeList(String fname, int forceDelimiter, boolean rcIndex1, boolean rcIndex2){
 		if(fname==null){return 0;}
 		String[] codes;
 		if(new File(fname).exists()) {
@@ -67,11 +117,14 @@ public class BarcodeStats {
 		}else {
 			codes=fname.split(",");
 		}
-		for(String s : codes){
+		for(int i=0; i<codes.length; i++){
+			String s=codes[i];
 			linesProcessed++;
 			bytesProcessed+=s.length();
 			if(!Tools.startsWith(s, '#')){
-				assert(s.indexOf('\t')<0) : "Barcodes should not contain a tab: '"+s+"'";//Although it's fine if they do; just use -da
+//				s=removeTab(s);
+				//assert(s.indexOf('\t')<0) : "Barcodes should not contain a tab: '"+s+"'";//Although it's fine if they do; just use -da
+				s=fixBarcode(s, forceDelimiter, rcIndex1, rcIndex2);
 				Barcode b=new Barcode(s);
 				expectedCodeList.add(b);
 				expectedCodeMap.put(b.name, b);
@@ -80,6 +133,53 @@ public class BarcodeStats {
 		return expectedCodeList.size();
 	}
 	
+//	//For the special case of barcode <tab> count, like CountBarcodes2's output
+//	//Now handled by fixBarcode
+//	private static String removeTab(String s) {
+//		int tab=s.indexOf('\t');
+//		if(tab<0) {return s;}
+//		for(int i=tab+1; i<s.length(); i++) {
+//			char c=s.charAt(i);
+//			if(!Tools.isDigit(c) && c!='\t') {return s;}
+//		}
+//		return s.substring(0, tab);
+//	}
+	
+	public static String fixBarcode(String s, int delimiter, boolean rcIndex1, boolean rcIndex2) {
+		if(s==null) {return null;}
+		final int tab=s.indexOf('\t');
+		if(delimiter!='\t' && tab>0) {s=s.substring(0, tab);}
+		if(delimiter>0) {s=fixDelimiter(s, delimiter);}
+		s=rcompBarcode(s, delimiter, rcIndex1, rcIndex2);
+		return s;
+	}
+	
+	public static String rcompBarcode(String s, int delimiter, boolean rcIndex1, boolean rcIndex2) {
+		if(s==null || (!rcIndex1 && !rcIndex2)) {return s;}
+		int idx=s.indexOf((char)delimiter);
+		if(idx<0) {
+			return rcIndex1 ? AminoAcid.reverseComplementBases(s) : s;
+		}else {
+			String a=s.substring(0, idx);
+			String b=s.substring(idx+1);
+			a=(rcIndex1 ? AminoAcid.reverseComplementBases(a) : a);
+			b=(rcIndex2 ? AminoAcid.reverseComplementBases(b) : b);
+			return a+((char)delimiter)+b;
+		}
+	}
+	
+	public static String fixDelimiter(String s, int forceDelimiter) {
+		if(s==null || forceDelimiter<1 || s.indexOf(forceDelimiter)>=0) {return s;}
+		for(int i=0; i<s.length(); i++) {
+			char c=s.charAt(i);
+			if(!Character.isLetter(c)) {
+				return s.replace(c, (char)forceDelimiter);
+			}
+		}
+		assert(false) : "Can't find delimiter in '"+s+"'";
+		return s;
+	}
+	
 	public long loadBarcodeCounts(String fname){
 		LineParser2 lp=new LineParser2('\t');
 		if(fname==null){
@@ -110,6 +210,7 @@ public class BarcodeStats {
 	public BarcodeStats split(int pairnum) {
 		assert(barcodesPerRead==2) : barcodesPerRead;
 		BarcodeStats split=new BarcodeStats((char)0, 1, pairnum==0 ? "Left" : "Right");
+		split.length1=(pairnum==0 ? length1 : length2);
 		for(Barcode b : expectedCodeList) {
 			split.addExpectedCode(b.getStringForPairnum(pairnum, delimiter));
 		}
@@ -139,7 +240,7 @@ public class BarcodeStats {
 	public static BarcodeStats loadStatic(String fname, String expectedBarcodeFile, int expectedBarcodeCount) {
 		BarcodeStats bs=loadStatic(fname);
 		if(expectedBarcodeFile!=null){
-			bs.loadBarcodeList(expectedBarcodeFile);
+			bs.loadBarcodeList(expectedBarcodeFile, bs.delimiter, false, false);
 		}else if(expectedBarcodeCount>0) {
 			ArrayList<Barcode> list=bs.toList();
 			for(int i=0, max=Tools.min(expectedBarcodeCount, list.size()); i<max; i++) {
@@ -191,6 +292,19 @@ public class BarcodeStats {
 		b.increment(amt);
 	}
 	
+	public void increment(String key, long amt, int tile) {
+		final String key2=(tile==0 ? key : key+tile);
+		Barcode b=codeMap.get(key2);
+		if(b==null){
+			b=new Barcode(key);
+			b.tile=tile;
+			codeMap.put(key2, b);
+		}else {
+			assert(b.tile==tile);
+		}
+		b.increment(amt);
+	}
+	
 	public void incrementBad(String key, long amt) {
 		Barcode b=badPairMap.get(key);
 		if(b==null){
@@ -209,6 +323,48 @@ public class BarcodeStats {
 		b.increment(amt);
 	}
 	
+	public void incrementMismatches(final String s, final long count, LongList transitions) {
+		final Barcode b=findClosest(s, 6, 1);
+		if(b==null) {return;}
+		final String s2=b.name;
+		for(int i=0; i<s.length(); i++) {
+			final char q=s.charAt(i), r=s2.charAt(i);
+			if(Tools.isLetter(q) && r!='N') {
+				if(q!=r) {
+					incrementMismatch(i, r, q, count, transitions);
+				}
+			}
+		}
+//		assert(false) : transitions;
+	}
+	
+	public void incrementMismatch(int pos, int ref, int query, long incr, LongList transitions) {
+		int idx=Transition.encode(pos, ref, query);
+		transitions.increment(idx, incr);
+	}
+	
+	public ByteBuilder printTransitions() {return printTransitions(transitions, null);}
+	
+	public static ByteBuilder printTransitions(LongList transitions, ByteBuilder bb) {
+		if(bb==null) {bb=new ByteBuilder();}
+		ArrayList<Transition> list=new ArrayList<Transition>();
+		for(int i=0; i<transitions.size(); i++) {
+			long count=transitions.get(i);
+			if(count>0) {
+				Transition t=Transition.decode(i);
+				t.count=count;
+				list.add(t);
+			}
+		}
+		Collections.sort(list);
+		bb.append("#pos\tref\tquery\tcount\n");
+		for(Transition t : list) {
+			t.appendTo(bb).nl();
+		}
+//		assert(false) : transitions+"\n"+bb;
+		return bb;
+	}
+	
 	public void calcStats() {//Should only be called after merging, only on master copy
 //		System.err.println(codeMap);
 //		assert(false) : codeMap.size();
@@ -223,20 +379,25 @@ public class BarcodeStats {
 				eb.increment(b.count());
 				expectedCodes+=b.count();
 				expectedCodesU++;
-//				validArray[2]+=b.count();
-			}else if(b.countUndefined()>0){
-				nCodes+=b.count();
-				nCodesU++;
-			}else if((mer=b.checkHomopolymer())>=0){
-				polymerArray[mer]+=b.count();
-				polymerArrayU[mer]++;
-			}else if(!expectedCodeList.isEmpty() && barcodesPerRead<2){
-				int hdist=calcHdist(b);
-				int edist=(hdist>1 && calcEdist ? calcEdist(b) : hdist);
-				hdistArray[hdist]+=b.count();
-				edistArray[edist]+=b.count();
-				hdistArrayU[hdist]++;
-				edistArrayU[edist]++;
+				//validArray[2]+=b.count();
+			}else{
+				if(!expectedCodeList.isEmpty()) {
+					incrementMismatches(b.name, b.count(), transitions);
+				}
+				if(b.countUndefined()>0){
+					nCodes+=b.count();
+					nCodesU++;
+				}else if((mer=b.checkHomopolymer())>=0){
+					polymerArray[mer]+=b.count();
+					polymerArrayU[mer]++;
+				}else if(!expectedCodeList.isEmpty() && barcodesPerRead<2){
+					int hdist=calcHdist(b);
+					int edist=(hdist>1 && calcEdist ? calcEdist(b) : hdist);
+					hdistArray[hdist]+=b.count();
+					edistArray[edist]+=b.count();
+					hdistArrayU[hdist]++;
+					edistArrayU[edist]++;
+				}
 			}
 			
 			if(leftStats!=null && !leftStats.expectedCodeList.isEmpty()) {
@@ -281,6 +442,65 @@ public class BarcodeStats {
 		}
 	}
 	
+	public Barcode findClosest(String s, int maxHDist) {
+		assert(!expectedCodeList.isEmpty());
+		Barcode best=expectedCodeMap.get(s);
+		if(best!=null || maxHDist<1) {return best;}
+		int min=s.length();
+		for(int i=0; i<expectedCodeList.size(); i++) {
+			Barcode b=expectedCodeList.get(i);
+			int dist=b.hdist(s);
+			if(dist<min) {
+				best=b;
+				min=dist;
+			}
+		}
+		assert(min>0);
+		return min<=maxHDist ? best : null;
+	}
+	
+	public Barcode findClosest(String s, int maxHDist, int clearzone) {
+		assert(!expectedCodeList.isEmpty());
+		Barcode best=expectedCodeMap.get(s);
+//		Barcode best2=null;
+		
+		//Note:  This ignores cases where you have a perfect match to the wrong barcode,
+		//or two barcodes are within clearzone of each other.
+		if(best!=null || maxHDist<1) {return best;}
+		int hdist=s.length();
+		int hdist2=hdist;
+		assert(best==null); //Otherwise hdist should be 0.
+		
+		for(int i=0; i<expectedCodeList.size(); i++) {
+			Barcode b=expectedCodeList.get(i);
+			int d=b.hdist(s);
+			if(d<hdist2) {
+//				best2=b;
+				hdist2=d;
+				if(d<hdist) {
+//					best2=best;
+					hdist2=hdist;
+					best=b;
+					hdist=d;
+				}
+			}
+		}
+		if(hdist<=maxHDist && hdist+clearzone<=hdist2) {return best;}
+		return null;
+	}
+	
+	//Slightly different than the below method; it uses a String and has no preconditions
+	public int calcHdist(String s) {
+		assert(!expectedCodeList.isEmpty());
+		if(expectedCodeMap.containsKey(s)) {return 0;}
+		int min=s.length();
+		for(int i=0; i<expectedCodeList.size(); i++) {
+			Barcode b=expectedCodeList.get(i);
+			min=Tools.min(min, b.hdist(s));
+		}
+		return min;
+	}
+	
 	public int calcHdist(Barcode b) {
 		assert(!expectedCodeList.isEmpty());
 		assert(!expectedCodeMap.containsKey(b.name)) : "Check this first.";
@@ -403,6 +623,10 @@ public class BarcodeStats {
 		if(barcodesPerRead>1) {
 			sb.append("\n-codesPerRead\t"+pad(barcodesPerRead));
 			sb.append("\n-delimiter\t"+(delimiter>0 ? Character.toString(delimiter) : ""));
+			if(length1>0) {sb.append("\n-bcLength1\t"+pad(length1));}
+			if(length2>0) {sb.append("\n-bcLength2\t"+pad(length2));}
+		}else {
+			if(length1>0) {sb.append("\n-bcLength\t"+pad(length1));}
 		}
 		if(!expectedCodeList.isEmpty()) {
 			sb.append("\n-expected\t"+pad(expectedCodes)+"\t("+expectedCodesU+" unique)");
@@ -437,7 +661,9 @@ public class BarcodeStats {
 		printToFile(ff);
 	}
 	
-	public void printToFile(FileFormat ff){
+	public void printToFile(FileFormat ff){printToFile(ff, 0);}
+	
+	public void printToFile(FileFormat ff, long minCount){
 		if(ff==null) {return;}
 		ArrayList<Barcode> list=toList();
 		long sum=0;
@@ -450,9 +676,11 @@ public class BarcodeStats {
 			bsw.println("#Unique\t"+list.size());
 			ByteBuilder bb=new ByteBuilder(128);
 			for(Barcode bc : list){
-				bc.appendTo(bb).nl();
-				bsw.print(bb);
-				bb.clear();
+				if(bc.count()>=minCount) {
+					bc.appendTo(bb).nl();
+					bsw.print(bb);
+					bb.clear();
+				}
 			}
 			errorState|=bsw.poisonAndWait();
 		} catch (Throwable e) {
@@ -473,6 +701,8 @@ public class BarcodeStats {
 
 	public final char delimiter;
 	public final int barcodesPerRead;
+	public int length1;
+	public int length2;
 	
 	public long linesProcessed=0, bytesProcessed=0;
 	
@@ -494,6 +724,8 @@ public class BarcodeStats {
 	public long[] polymerArrayU=new long[4];
 	public long[] hdistArrayU=new long[30];
 	public long[] edistArrayU=new long[30];
+	
+	public LongList transitions=new LongList();
 
 	public BarcodeStats leftStats=null;
 	public BarcodeStats rightStats=null;
