@@ -69,7 +69,7 @@ public class BloomFilter implements Serializable {
 		
 		//Set shared static variables
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		KmerCountAbstract.CANONICAL=true;
 		
 		//Create a parser object
@@ -173,6 +173,11 @@ public class BloomFilter implements Serializable {
 		filter=load();
 	}
 	
+	public BloomFilter(int k_, int kbig_, int bits_, int hashes_,
+			int minConsecutiveMatches_, boolean rcomp_, boolean ecco_, boolean merge_, float memFraction){
+		this(null, null, null, k_, kbig_, bits_, hashes_, minConsecutiveMatches_, rcomp_, ecco_, merge_, memFraction);
+	}
+	
 	public BloomFilter(String in1_, String in2_, ArrayList<String> extra_, int k_, int kbig_, int bits_, int hashes_,
 			int minConsecutiveMatches_, boolean rcomp_, boolean ecco_, boolean merge_, float memFraction){
 		if(extra_!=null){
@@ -219,7 +224,7 @@ public class BloomFilter implements Serializable {
 	}
 	
 	private static long setMemory(double mult){
-		Shared.printMemory();
+		if(printMem) {Shared.printMemory();}
 		
 		Runtime rt=Runtime.getRuntime();
 		final long mmemory=rt.maxMemory();
@@ -245,11 +250,16 @@ public class BloomFilter implements Serializable {
 		final int cbits=bits;
 		final long totalBits=8*filterMemory;
 		final long cells=(OVERRIDE_CELLS>0 ? OVERRIDE_CELLS : totalBits/cbits);
-//		System.err.println("filterMemory="+filterMemory+", cells="+cells);
-		ReadCounter rc=new ReadCounter(k, rcomp, ecco, merge, Shared.AMINO_IN);
-		KCountArray7MTA kca=(KCountArray7MTA)rc.makeKca(in1, in2, extra==null || extra.isEmpty() ? null : extra,
-				cbits, cells, hashes, minq,
-				maxReads, 1, 1, 1, 1, null, 0);
+		//		System.err.println("filterMemory="+filterMemory+", cells="+cells);
+		KCountArray7MTA kca;
+		if(in1!=null || (extra!=null && extra.size()>0)) {
+			ReadCounter rc=new ReadCounter(k, rcomp, ecco, merge, Shared.AMINO_IN);
+			kca=(KCountArray7MTA)rc.makeKca(in1, in2, extra==null || extra.isEmpty() ? null : extra,
+					cbits, cells, hashes, minq,
+					maxReads, 1, 1, 1, null, 0);
+		}else {
+			kca=(KCountArray7MTA) KCountArray.makeNew(cells, cbits, hashes, null, 0);
+		}
 		return kca;
 	}
 
@@ -632,6 +642,57 @@ public class BloomFilter implements Serializable {
 		return (rcomp ? Tools.max(kmer, rkmer) : kmer);
 	}
 	
+	public static final void toKmers(Read r, final LongList list, int k, final int minQuality, final float minProb, final boolean rcomp){
+		assert(k<=32);
+		assert(list!=null);
+		
+		final byte[] bases=r.bases;
+		final byte[] quals=r.quality;
+		
+		if(bases==null || bases.length<k){return;}
+		
+		final int shift=bitsPerBase*k;
+		final int shift2=shift-bitsPerBase;
+		final long mask=(shift>63 ? -1L : ~((-1L)<<shift));
+		
+		long kmer=0;
+		long rkmer=0;
+		int len=0;
+		float prob=1;
+		
+		for(int i=0; i<bases.length; i++){
+			final byte b=bases[i];
+			long x=AminoAcid.baseToNumber[b];
+			long x2=AminoAcid.baseToComplementNumber[b];
+			kmer=((kmer<<2)|x)&mask;
+			rkmer=((rkmer>>>2)|(x2<<shift2))&mask;
+
+			final byte q;
+			if(quals==null){
+				q=50;
+			}else{
+				q=quals[i];
+				prob=prob*align2.QualityTools.PROB_CORRECT[q];
+				if(len>k){
+					byte oldq=quals[i-k];
+					prob=prob*align2.QualityTools.PROB_CORRECT_INVERSE[oldq];
+				}
+			}
+
+			if(x<0 || q<minQuality){
+				len=0;
+				kmer=rkmer=0;
+				prob=1;
+			}else{
+				len++;
+				if(len>=k && prob>=minProb){
+					long key=(rcomp ? Tools.max(kmer, rkmer) : kmer);
+					list.add(key);
+				}
+			}
+		}
+	}
+	
 	/*--------------------------------------------------------------*/
 	/*----------------         Inner Classes        ----------------*/
 	/*--------------------------------------------------------------*/
@@ -678,6 +739,7 @@ public class BloomFilter implements Serializable {
 	
 	public static long OVERRIDE_CELLS=-1;
 	static final int bitsPerBase=2;
+	public static boolean printMem=true;
 	
 	/*--------------------------------------------------------------*/
 	/*----------------        Common Fields         ----------------*/
