@@ -9,6 +9,7 @@ import dna.AminoAcid;
 import shared.Shared;
 import shared.Tools;
 import structures.ByteBuilder;
+import structures.LongList;
 
 /**
  * @author Brian Bushnell
@@ -171,7 +172,9 @@ public abstract class KCountArray implements Serializable {
 	 * @return Array of counts of all 4 possible right-adjacent kmers
 	 */
 	public int[] readAllRight(final long key, final int k, boolean makeCanonical, int[] rvec){throw new RuntimeException("Unsupported.");}
-	
+
+	//Appears to never be used?  Should be a LongList now anyway
+	@Deprecated
 	public void increment(long[] keys){
 		synchronized(this){
 			for(long key : keys){
@@ -180,6 +183,16 @@ public abstract class KCountArray implements Serializable {
 		}
 	}
 	
+	//TODO: Optionally, add flag to eliminate duplicates or bulk-add them here
+	public void increment(LongList keys){
+		synchronized(this){
+			final long[] array=keys.array;
+			for(int i=0; i<keys.size; i++){
+				increment(array[i]);
+			}
+		}
+	}
+	
 	public abstract long[] transformToFrequency();
 	public final long[] transformToFrequency(int[][] matrix){
 		long[] freq=new long[100000];
