@@ -6,6 +6,8 @@ import java.util.LinkedHashMap;
 import java.util.Map.Entry;
 import java.util.concurrent.atomic.AtomicIntegerArray;
 import java.util.concurrent.atomic.AtomicLongArray;
+import java.util.concurrent.locks.ReadWriteLock;
+import java.util.concurrent.locks.ReentrantReadWriteLock;
 
 import fileIO.ByteFile;
 import fileIO.FileFormat;
@@ -76,7 +78,7 @@ public class FixScaffoldGaps implements Accumulator<FixScaffoldGaps.ProcessThrea
 		
 		//Set shared static variables prior to parsing
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		
 		samFilter.includeUnmapped=false;
 		samFilter.includeSupplimentary=false;
@@ -360,7 +362,7 @@ public class FixScaffoldGaps implements Accumulator<FixScaffoldGaps.ProcessThrea
 		}
 		
 		//Wait for threads to finish
-		boolean success=ThreadWaiter.waitForThreads(alpt, this);
+		boolean success=ThreadWaiter.waitForThreadsToFinish(alpt, this);
 		errorState&=!success;
 		
 		//Do anything necessary after processing
@@ -752,6 +754,10 @@ public class FixScaffoldGaps implements Accumulator<FixScaffoldGaps.ProcessThrea
 	/** Primary output file */
 	private final FileFormat ffout;
 	
+	@Override
+	public final ReadWriteLock rwlock() {return rwlock;}
+	private final ReadWriteLock rwlock=new ReentrantReadWriteLock();
+	
 	/*--------------------------------------------------------------*/
 	/*----------------        Common Fields         ----------------*/
 	/*--------------------------------------------------------------*/
