@@ -6,6 +6,8 @@ import java.util.LinkedHashMap;
 import java.util.Map.Entry;
 import java.util.concurrent.atomic.AtomicIntegerArray;
 import java.util.concurrent.atomic.AtomicLongArray;
+import java.util.concurrent.locks.ReadWriteLock;
+import java.util.concurrent.locks.ReentrantReadWriteLock;
 
 import dna.AminoAcid;
 import fileIO.ByteFile;
@@ -76,7 +78,7 @@ public class Lilypad implements Accumulator<Lilypad.ProcessThread> {
 		
 		//Set shared static variables prior to parsing
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		SamLine.RNAME_AS_BYTES=false;
 		
 		samFilter.includeUnmapped=false;
@@ -366,7 +368,7 @@ public class Lilypad implements Accumulator<Lilypad.ProcessThread> {
 		}
 		
 		//Wait for threads to finish
-		boolean success=ThreadWaiter.waitForThreads(alpt, this);
+		boolean success=ThreadWaiter.waitForThreadsToFinish(alpt, this);
 		errorState&=!success;
 		
 		//Do anything necessary after processing
@@ -973,6 +975,10 @@ public class Lilypad implements Accumulator<Lilypad.ProcessThread> {
 	/** Primary output file */
 	private final FileFormat ffout;
 	
+	@Override
+	public final ReadWriteLock rwlock() {return rwlock;}
+	private final ReadWriteLock rwlock=new ReentrantReadWriteLock();
+	
 	/*--------------------------------------------------------------*/
 	/*----------------        Common Fields         ----------------*/
 	/*--------------------------------------------------------------*/
