@@ -128,7 +128,7 @@ public final class FileFormat {
 	
 	public static FileFormat testInput(String fname, int defaultFormat, String overrideExtension, boolean allowSubprocess, boolean allowFileRead){
 		if(verbose){System.err.println("testInputB("+fname+", "+defaultFormat+", "+overrideExtension+", "+allowSubprocess+", "+allowFileRead+")");}
-		return testInput(fname, defaultFormat, overrideExtension, allowSubprocess, allowFileRead, false);
+		return testInput(fname, defaultFormat, overrideExtension, allowSubprocess, allowFileRead, allowFileRead && !isStdin(fname));
 	}
 	
 	public static FileFormat testInput(String fname, int defaultFormat, String overrideExtension, boolean allowSubprocess, boolean allowFileRead, boolean forceFileRead){
@@ -225,8 +225,9 @@ public final class FileFormat {
 		
 		if(a[0]==UNKNOWN && overrideFormat<1){
 			a[0]=defaultFormat;
-			if(defaultFormat!=TEXT && PRINT_WARNING){
-				System.err.println("Unspecified format for "+(mode_==READ ? "input" : "output")+" "+(fname==null ? "stream" : fname)+"; defaulting to "+FORMAT_ARRAY[a[0]]+".");
+			if(defaultFormat!=TEXT && PRINT_WARNING && a[2]!=DEVNULL){
+				System.err.println("Unspecified format for "+(mode_==READ ? "input" : "output")+
+						" "+(fname==null ? "stream" : fname)+"; defaulting to "+FORMAT_ARRAY[a[0]]+".");
 			}
 		}
 		if(verbose){System.err.println(Arrays.toString(a));}
@@ -248,6 +249,8 @@ public final class FileFormat {
 		length=a[5];
 		numBarcodes=a[6];
 		barcodeDelimiter=a[7];
+		barcodeLength1=a[8];
+		barcodeLength2=a[9];
 		mode=mode_;
 		input=input_;
 //		assert(!allowFileRead || barcodeDelimiter>=0) : barcodeDelimiter;
@@ -319,10 +322,10 @@ public final class FileFormat {
 	/*--------------------------------------------------------------*/
 	
 	/** Returns an int array: {format, compression, type, interleaved, 
-	 * quality, length, numBarcodes, barcodeDelimiter} */
+	 * quality, length, numBarcodes, barcodeDelimiter, bcLength1, bcLength2} */
 	public static final int[] testFormat(String fname, boolean allowFileRead, boolean forceFileRead){
 		if(verbose){System.err.println("testFormat("+fname+", "+allowFileRead+", "+forceFileRead+")");}
-		final int[] r=new int[] {UNKNOWN, RAW, FILE, UNKNOWN, -1, -1, -1, -1};
+		final int[] r=new int[] {UNKNOWN, RAW, FILE, UNKNOWN, -1, -1, -1, -1, -1, -1};
 		if(fname==null || fname.length()<1){
 			r[2]=STDIO;
 			return r;
@@ -374,7 +377,7 @@ public final class FileFormat {
 		if(slc.length()>2 && slc.charAt(0)=='s' && slc.charAt(1)=='t'){
 			if(slc.equals("stdin") || slc.startsWith("stdin.") || slc.equals("standardin")){r[2]=STDIO;}
 			else if(slc.equals("stdout") || slc.startsWith("stdout.") || slc.equals("standardout")){r[2]=STDIO;}
-		}else if("/dev/null".equalsIgnoreCase(slc)){
+		}else if("/dev/null".equalsIgnoreCase(slc) || "devnull".equalsIgnoreCase(slc)){
 			r[2]=DEVNULL;
 		}
 
@@ -386,7 +389,8 @@ public final class FileFormat {
 				+","+stream.FASTQ.TEST_INTERLEAVED);//false,false,true,true,true,true,false
 		}
 		if(r[0]==UNKNOWN || (r[0]!=BAM && forceFileRead) || 
-				((r[0]==FASTQ || r[0]==FASTA) && r[3]==UNKNOWN && allowFileRead && !stream.FASTQ.FORCE_INTERLEAVED && stream.FASTQ.TEST_INTERLEAVED)){
+				((r[0]==FASTQ || r[0]==FASTA) && r[3]==UNKNOWN && allowFileRead 
+				&& !stream.FASTQ.FORCE_INTERLEAVED && stream.FASTQ.TEST_INTERLEAVED)){
 			File f=(allowFileRead && r[2]==FILE ? new File(fname) : null);
 			if(f!=null && f.exists() && !f.isDirectory()){
 //				//a: {quality, interleaved, length, format}
@@ -401,6 +405,8 @@ public final class FileFormat {
 						if(al>1 && r[5]==-1){r[5]=al;}
 						r[6]=abc;
 						r[7]=abd;
+						r[8]=a[6];
+						r[9]=a[7];
 					}
 				} catch (Exception e) {
 					// TODO Auto-generated catch block
@@ -455,7 +461,8 @@ public final class FileFormat {
 	
 	/**
 	 * @param fname File to read
-	 * @return {quality, interleaved, length, format, numBarcodes, barcodeDelimiter}
+	 * @return {quality, interleaved, length, format, numBarcodes, barcodeDelimiter,
+	 * bcLen1, bcLen2}
 	 */
 	public static int[] testInterleavedAndQuality(String fname, boolean forceFastq){
 		final ArrayList<String> oct=getFirstOctet(fname);
@@ -494,9 +501,9 @@ public final class FileFormat {
 	public static int[] testInterleavedAndQuality(final ArrayList<String> oct, String fname, boolean forceFastq){
 		int len=-1, format=UNKNOWN;
 		byte q=-1, i=UNKNOWN;
-		int numBarcodes=0, barcodeDelimiter=0;
+		int numBarcodes=0, barcodeDelimiter=0, bcLen1=-1, bcLen2=-1;
 		if(oct==null || oct.size()<1){
-			return new int[] {q, i, len, format, numBarcodes, barcodeDelimiter};
+			return new int[] {q, i, len, format, numBarcodes, barcodeDelimiter, bcLen1, bcLen2};
 		}
 		{
 			String s1=oct.size()>0 ? oct.get(0) : "";
@@ -557,9 +564,19 @@ public final class FileFormat {
 				if(len<2){len=-1;}
 				if(h!=null){
 					String barcode=Read.headerToBarcode(h, false);
-					if(barcode!=null && barcode.length()>2){
-						barcodeDelimiter=barcodeDelimiter(barcode);
-						numBarcodes=(countLetters(barcode))>0 ? (barcodeDelimiter>0 ? 2 : 1) : 0;
+					if(barcode!=null && barcode.length()>0) {
+						numBarcodes=1;
+						if(barcode.length()>2){
+							barcodeDelimiter=barcodeDelimiter(barcode);
+							numBarcodes=(countLetters(barcode))>0 ? (barcodeDelimiter>0 ? 2 : 1) : 0;
+						}
+						if(numBarcodes==1) {
+							bcLen1=barcode.length();
+							bcLen2=0;
+						}else {
+							bcLen1=barcode.indexOf(barcodeDelimiter);
+							bcLen2=barcode.length()-bcLen1-1;
+						}
 					}
 				}
 			}
@@ -567,7 +584,7 @@ public final class FileFormat {
 			stream.FASTQ.ASCII_OFFSET=oldQin;
 			stream.FASTQ.ASCII_OFFSET_OUT=oldQout;
 		}
-		int[] r=new int[] {q, i, len, format, numBarcodes, barcodeDelimiter};
+		int[] r=new int[] {q, i, len, format, numBarcodes, barcodeDelimiter, bcLen1, bcLen2};
 		if(verbose){System.err.println(Arrays.toString(r));}
 		return r;
 	}
@@ -625,13 +642,44 @@ public final class FileFormat {
 		return ext.equals("faa"); //TODO: Investigate whether other extensions imply AA.
 	}
 	
-	public static boolean isStdio(String s){
+	public static boolean isStdin(String s) {
 		if(s==null){return false;}
-		if(new File(s).exists()){return false;}
-		if(s.contains(".")){s=s.substring(0, s.indexOf('.'));
-		}
-		return (s.equalsIgnoreCase("stdin") || s.equalsIgnoreCase("stdout") || s.equalsIgnoreCase("stderr"));
+//		if(new File(s).exists()){return false;}
+		int idx=s.indexOf('.');
+		if(idx>=0){s=s.substring(0, idx);}
+		return s.equalsIgnoreCase("stdin") || s.equalsIgnoreCase("standardin");
 	}
+	public static boolean isStdout(String s) {
+		if(s==null){return false;}
+//		if(new File(s).exists()){return false;}
+		int idx=s.indexOf('.');
+		if(idx>=0){s=s.substring(0, idx);}
+		return s.equalsIgnoreCase("stdout") || s.equalsIgnoreCase("standardout");
+	}
+	public static boolean isStderr(String s) {
+		if(s==null){return false;}
+//		if(new File(s).exists()){return false;}
+		int idx=s.indexOf('.');
+		if(idx>=0){s=s.substring(0, idx);}
+		return s.equalsIgnoreCase("stderr") || s.equalsIgnoreCase("standarderror");
+	}
+	public static boolean isStdio(String s) {
+		if(s==null){return false;}
+//		if(new File(s).exists()){return false;}
+		int idx=s.indexOf('.');
+		if(idx>=0){s=s.substring(0, idx);}
+		return s.equalsIgnoreCase("stdin") || s.equalsIgnoreCase("standardin") ||
+				s.equalsIgnoreCase("stdout") || s.equalsIgnoreCase("standardout") ||
+				s.equalsIgnoreCase("stderr") || s.equalsIgnoreCase("standarderror");
+	}
+	
+//	public static boolean isStdio(String s){
+//		if(s==null){return false;}
+//		if(new File(s).exists()){return false;}
+//		if(s.contains(".")){s=s.substring(0, s.indexOf('.'));
+//		}
+//		return (s.equalsIgnoreCase("stdin") || s.equalsIgnoreCase("stdout") || s.equalsIgnoreCase("stderr"));
+//	}
 	
 	public static boolean isFastq(String ext){
 		if(ext==null){return false;}
@@ -678,6 +726,9 @@ public final class FileFormat {
 
 	public int barcodeDelimiter() {return barcodeDelimiter;}
 	public int barcodesPerRead() {return numBarcodes;}
+	public int barcodeLength(int barcodeNum) {
+		return barcodeNum==1 ? barcodeLength1 : barcodeNum==2 ? barcodeLength2 : 0;
+	}
 	
 	/*--------------------------------------------------------------*/
 	/*----------------            Getters           ----------------*/
@@ -850,6 +901,8 @@ public final class FileFormat {
 	private final boolean amino;
 	private final int numBarcodes;
 	private final int barcodeDelimiter;
+	private final int barcodeLength1;
+	private final int barcodeLength2;
 
 	private final boolean overwrite;
 	private final boolean append;
