@@ -123,7 +123,12 @@ public class ReadWrite {
 
 	public static void writeString(CharSequence x, String fname){writeString(x, fname, false);}
 	public static void writeString(CharSequence x, String fname, boolean append){
+		writeString(x, fname, !append, append);
+	}
+	public static void writeString(CharSequence x, String fname, boolean overwrite, boolean append){
 		if(verbose){System.err.println("writeString(x, "+fname+", "+append+")");}
+		File f=new File(fname);
+		assert(overwrite || append || !f.exists()) : "File "+fname+" exists and overwrite=f";
 		OutputStream os=getOutputStream(fname, append, true, false);
 		
 		try {
@@ -524,18 +529,15 @@ public class ReadWrite {
 	}
 	
 	public static OutputStream getGZipOutputStream(String fname, boolean append, boolean allowSubprocess){
-		if(verbose){System.err.println("getGZipOutputStream("+fname+", "+append+", "+allowSubprocess+"); "+USE_BGZIP+", "+USE_PIGZ+", "+USE_GZIP+", "+RAWMODE);}
+		if(verbose){System.err.println("getGZipOutputStream("+fname+", "+append+", "+allowSubprocess+"); "+USE_BGZIP+", "+Data.BGZIP()+", "+USE_PIGZ+", "+USE_GZIP+", "+RAWMODE);}
 //		assert(false) : ReadWrite.ZIPLEVEL+", "+Shared.threads()+", "+MAX_ZIP_THREADS+", "+ZIP_THREAD_MULT+", "+allowSubprocess+", "+USE_PIGZ+", "+Data.PIGZ();
-		
 		if(FORCE_BGZIP && USE_BGZIP && Data.BGZIP()){return getBgzipStream(fname, append);}
-		
 		if(FORCE_PIGZ || (allowSubprocess && Shared.threads()>=2)){
 			if((fname.endsWith(".vcf.gz") || fname.endsWith(".sam.gz") || (PREFER_BGZIP && ZIPLEVEL<10)) && USE_BGZIP && Data.BGZIP()){return getBgzipStream(fname, append);}
 			if(USE_PIGZ && Data.PIGZ()){return getPigzStream(fname, append);}
 			if(USE_BGZIP && Data.BGZIP()){return getBgzipStream(fname, append);}
 			if(USE_GZIP && Data.GZIP()/* && (Data.SH() /*|| fname.equals("stdout") || fname.startsWith("stdout."))*/){return getGzipStream(fname, append);}
 		}
-		
 		final OutputStream raw=getRawOutputStream(fname, append, false);
 		if(RAWMODE){return raw;}
 		try {
@@ -1919,15 +1921,25 @@ public class ReadWrite {
 	public static int ZIPLEVEL=4;
 	/** Bzip2 compression level */
 	public static int BZIPLEVEL=9;
-	public static int MAX_ZIP_THREADS=96;
+	private static int MAX_ZIP_THREADS=96;
 	public static int MAX_SAMTOOLS_THREADS=64;
 	public static int PIGZ_BLOCKSIZE=128;
 	public static int PIGZ_ITERATIONS=-1;
+
+	public static boolean SET_ZIP_THREAD_MULT=false;
+	public static boolean SET_ZIP_THREADS=false;
 	
-	public static void setZipThreadMult(float x){
-		ZIP_THREAD_MULT=Tools.min(1, Tools.max(0.125f, x));
+	public static int MAX_ZIP_THREADS() {return MAX_ZIP_THREADS;}
+	public static void setZipThreads(int x){
+		MAX_ZIP_THREADS=Tools.mid(1, x, 96);
+		SET_ZIP_THREADS=true;
 	}
-	public static float ZIP_THREAD_MULT=1f;
+	public static float ZIP_THREAD_MULT() {return ZIP_THREAD_MULT;}
+	public static void setZipThreadMult(float x){
+		ZIP_THREAD_MULT=Tools.mid(0.125f, x, 1f);
+		SET_ZIP_THREAD_MULT=true;
+	}
+	private static float ZIP_THREAD_MULT=1f;
 	public static boolean ALLOW_ZIPLEVEL_CHANGE=true;
 	
 	public static final String FILESEP=System.getProperty("file.separator");
