@@ -54,7 +54,7 @@ public class DiskBench {
 		}
 		
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		
 		Parser parser=new Parser();
 		parser.overwrite=true;
