@@ -6,10 +6,11 @@ import java.util.ArrayList;
 import java.util.Locale;
 import java.util.Random;
 
+import bloom.BloomFilter;
+import dna.AminoAcid;
 import fileIO.ByteFile;
 import fileIO.FileFormat;
 import fileIO.ReadWrite;
-import fileIO.TextStreamWriter;
 import jgi.Dedupe;
 import kmer.AbstractKmerTable;
 import kmer.HashArray1D;
@@ -27,6 +28,7 @@ import stream.FASTQ;
 import stream.FastaReadInputStream;
 import stream.Read;
 import structures.ListNum;
+import structures.LongList;
 
 /**
  * Analyzes a flow cell for low-quality areas.
@@ -69,7 +71,7 @@ public class AnalyzeFlowCell {
 		
 		//Set shared static variables
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		
 		Parser parser=parse(args);
 		
@@ -86,13 +88,9 @@ public class AnalyzeFlowCell {
 			
 			in1=parser.in1;
 			in2=parser.in2;
-			qfin1=parser.qfin1;
-			qfin2=parser.qfin2;
 
 			out1=parser.out1;
 			out2=parser.out2;
-			qfout1=parser.qfout1;
-			qfout2=parser.qfout2;
 			
 			extin=parser.extin;
 			extout=parser.extout;
@@ -142,13 +140,13 @@ public class AnalyzeFlowCell {
 			}else if(a.equals("seed")){
 				seed=Long.parseLong(b);
 			}else if(a.equals("divisor") || a.equals("size")){
-				Tile.xSize=Tile.ySize=Integer.parseInt(b);
+				Tile.xSize=Tile.ySize=Parse.parseIntKMG(b);
 			}else if(a.equals("xdivisor") || a.equals("xsize")){
-				Tile.xSize=Integer.parseInt(b);
+				Tile.xSize=Parse.parseIntKMG(b);
 			}else if(a.equals("ydivisor") || a.equals("ysize")){
-				Tile.ySize=Integer.parseInt(b);
+				Tile.ySize=Parse.parseIntKMG(b);
 			}else if(a.equals("target")){
-				targetAverageReads=Integer.parseInt(b);
+				targetAverageReads=Parse.parseIntKMG(b);
 			}else if(a.equals("dump")){
 				dump=b;
 			}else if(a.equals("indump") || a.equals("ind") || a.equals("dumpin")){
@@ -157,62 +155,34 @@ public class AnalyzeFlowCell {
 				pound=Parse.parseBoolean(b);
 			}else if(a.equals("loadkmers") || a.equals("usekmers")){
 				loadKmers=Parse.parseBoolean(b);
-			}else if(a.equals("lqo") || a.equals("lowqualityonly")){
-				discardOnlyLowQuality=Parse.parseBoolean(b);
-			}else if(a.equals("dl") || a.equals("discardlevel")){
-				discardLevel=Integer.parseInt(b);
-			}else if(a.equals("outbad") || a.equals("outb") || a.equals("outtoss") || a.equals("outt") || a.equals("outunwanted") || a.equals("outu")){
-				outbad=b;
-			}
-			
-			else if(a.equals("deviations") || a.equals("d")){
-				qDeviations=uDeviations=eDeviations=gDeviations=Float.parseFloat(b);
-			}else if(a.equals("qdeviations") || a.equals("qd") || a.equals("dq")){
-				qDeviations=Float.parseFloat(b);
-			}else if(a.equals("udeviations") || a.equals("ud") || a.equals("du")){
-				uDeviations=Float.parseFloat(b);
-			}else if(a.equals("edeviations") || a.equals("ed") || a.equals("de")){
-				eDeviations=Float.parseFloat(b);
-			}else if(a.equals("gdeviations") || a.equals("gd") || a.equals("dg")){
-				gDeviations=Float.parseFloat(b);
-			}
-			
-			else if(a.equals("qfraction") || a.equals("qf")){
-				qualFraction=Float.parseFloat(b);
-			}else if(a.equals("ufraction") || a.equals("uf")){
-				uniqueFraction=Float.parseFloat(b);
-			}else if(a.equals("efraction") || a.equals("ef")){
-				errorFreeFraction=Float.parseFloat(b);
-			}else if(a.equals("gfraction") || a.equals("gf")){
-				gFraction=Float.parseFloat(b);
-			}
-			
-			else if(a.equals("qabsolute") || a.equals("qa")){
-				qualAbs=Float.parseFloat(b);
-			}else if(a.equals("uabsolute") || a.equals("ua")){
-				uniqueAbs=Float.parseFloat(b);
-			}else if(a.equals("eabsolute") || a.equals("ea")){
-				errorFreeAbs=Float.parseFloat(b);
-			}else if(a.equals("gabsolute") || a.equals("ga")){
-				gAbs=Float.parseFloat(b);
-			}
-			
-			else if(a.equals("gton")){
-				gToN=Parse.parseBoolean(b);
-			}else if(a.equals("discardg")){
-				discardG=Parse.parseBoolean(b);
+			}else if(a.equals("allkmers")){
+				allKmers=Parse.parseBoolean(b);
+			}else if(a.equals("multithreaded")){
+				multithreaded=Parse.parseBoolean(b);
+			}else if(a.equals("bloom")){
+				bloomMode=Parse.parseBoolean(b);
+			}else if(a.equals("bits") || a.equals("cbits")){
+				cbits=Integer.parseInt(b);
 			}
 			
 			else if(a.equals("minpolyg")){
 				MicroTile.MIN_POLY_G=Integer.parseInt(b);
 			}else if(a.equals("trackcycles")){
 				MicroTile.TRACK_CYCLES=Parse.parseBoolean(b);
+			}else if(a.equals("extra")){
+				if(b!=null) {
+					for(String s : b.split(",")) {
+						extra.add(s);
+					}
+				}
 			}
 			
 			else if(a.equals("parse_flag_goes_here")){
 				//Set a variable here
 			}else if(parser.parse(arg, a, b)){//Parse standard flags in the parser
 				//do nothing
+			}else if(b==null && new File(arg).exists()){
+				extra.add(b);
 			}else{
 				outstream.println("Unknown parameter "+args[i]);
 				assert(false) : "Unknown parameter "+args[i];
@@ -322,16 +292,6 @@ public class AnalyzeFlowCell {
 			if(flowcell.avgReads<targetAverageReads){
 				flowcell=flowcell.widen(targetAverageReads);
 			}
-			
-			avgQuality=flowcell.avgQuality;
-			avgUnique=flowcell.avgUnique;
-			avgErrorFree=flowcell.avgErrorFree;
-			avgG=flowcell.avgG;
-			stdQuality=flowcell.stdQuality;
-			stdUnique=flowcell.stdUnique;
-			stdErrorFree=flowcell.stdErrorFree;
-			stdG=flowcell.stdG;
-
 			long readsToDiscard=markTiles(flowcell.toList(), flowcell.avgReads);
 		}
 		processReads(t);
@@ -345,7 +305,7 @@ public class AnalyzeFlowCell {
 		//Create a read input stream
 		final ConcurrentReadInputStream cris;
 		{
-			cris=ConcurrentReadInputStream.getReadInputStream(maxReads, true, ffin1, ffin2, qfin1, qfin2);
+			cris=ConcurrentReadInputStream.getReadInputStream(maxReads, true, ffin1, ffin2);
 			cris.start(); //Start the stream
 			if(verbose){outstream.println("Started cris");}
 		}
@@ -370,7 +330,7 @@ public class AnalyzeFlowCell {
 		//Create a read input stream
 		final ConcurrentReadInputStream cris;
 		{
-			cris=ConcurrentReadInputStream.getReadInputStream(maxReads, true, ffin1, ffin2, qfin1, qfin2);
+			cris=ConcurrentReadInputStream.getReadInputStream(maxReads, true, ffin1, ffin2);
 			cris.start(); //Start the stream
 			if(verbose){outstream.println("Started cris");}
 		}
@@ -396,7 +356,7 @@ public class AnalyzeFlowCell {
 			//Create a read input stream
 			final ConcurrentReadInputStream cris;
 			{
-				cris=ConcurrentReadInputStream.getReadInputStream(maxReads, true, ffin1, ffin2, qfin1, qfin2);
+				cris=ConcurrentReadInputStream.getReadInputStream(maxReads, true, ffin1, ffin2);
 				cris.start(); //Start the stream
 				if(verbose){outstream.println("Started cris");}
 			}
@@ -407,7 +367,7 @@ public class AnalyzeFlowCell {
 			final ConcurrentReadOutputStream ros, rosb;
 			final int buff=4;
 			if(ffout1!=null){
-				ros=ConcurrentReadOutputStream.getStream(ffout1, ffout2, qfout1, qfout2, buff, null, false);
+				ros=ConcurrentReadOutputStream.getStream(ffout1, ffout2, buff, null, false);
 				ros.start(); //Start the stream
 			}else{ros=null;}
 			
@@ -554,34 +514,42 @@ public class AnalyzeFlowCell {
 				assert((ffin1==null || ffin1.samOrBam()) || (r.mate!=null)==cris.paired());
 			}
 			
+			LongList kmers=new LongList(300);
 			//As long as there is a nonempty read list...
 			while(ln!=null && reads!=null && reads.size()>0){//ln!=null prevents a compiler potential null access warning
 				if(verbose){outstream.println("Fetched "+reads.size()+" reads.");}
 				
 				//Loop through each read in the list
+//				for(int idx=0; idx<reads.size(); idx++){
+//					final Read r1=reads.get(idx);
+//					final Read r2=r1.mate;
+//					
+//					//Track the initial length for statistics
+//					final int initialLength1=r1.length();
+//					final int initialLength2=(r1.mateLength());
+//					
+//					if(initialLength1>=k && randy.nextBoolean()){
+//						final long kmer=toKmer(r1.bases, r1.quality, randy.nextInt(initialLength1-k2), k);
+//						if(kmer>=0){
+//							AbstractKmerTable table=keySets[(int)(kmer%WAYS)];
+//							table.increment(kmer, 1);
+//						}
+//					}
+//					
+//					if(initialLength2>=k && randy.nextBoolean()){
+//						final long kmer=toKmer(r2.bases, r2.quality, randy.nextInt(initialLength2-k2), k);
+//						if(kmer>=0){
+//							AbstractKmerTable table=keySets[(int)(kmer%WAYS)];
+//							table.increment(kmer, 1);
+//						}
+//					}
+//				}
 				for(int idx=0; idx<reads.size(); idx++){
 					final Read r1=reads.get(idx);
 					final Read r2=r1.mate;
-					
-					//Track the initial length for statistics
-					final int initialLength1=r1.length();
-					final int initialLength2=(r1.mateLength());
-					
-					if(initialLength1>=k && randy.nextBoolean()){
-						final long kmer=toKmer(r1.bases, r1.quality, randy.nextInt(initialLength1-k2), k);
-						if(kmer>=0){
-							AbstractKmerTable table=keySets[(int)(kmer%WAYS)];
-							table.increment(kmer, 1);
-						}
-					}
-					
-					if(initialLength2>=k && randy.nextBoolean()){
-						final long kmer=toKmer(r2.bases, r2.quality, randy.nextInt(initialLength2-k2), k);
-						if(kmer>=0){
-							AbstractKmerTable table=keySets[(int)(kmer%WAYS)];
-							table.increment(kmer, 1);
-						}
-					}
+
+					loadKmers(r1, kmers);
+					loadKmers(r2, kmers);
 				}
 				
 				//Notify the input stream that the list was used
@@ -602,10 +570,39 @@ public class AnalyzeFlowCell {
 		//Do anything necessary after processing
 	}
 	
+	private void loadKmers(Read r, LongList kmers) {
+		if(r==null || r.length()<k) {return;}
+		if(!randy.nextBoolean()) {return;}//Speed optimization, I guess
+		if(allKmers) {
+			kmers.clear();
+			BloomFilter.toKmers(r, kmers, k, 0, 0, true);
+			if(keySets!=null) {
+				for(int i=0; i<kmers.size; i++) {
+					long kmer=kmers.array[i];
+					AbstractKmerTable table=keySets[(int)(kmer%WAYS)];
+					table.increment(kmer, 1);
+				}
+			}else {
+				for(int i=0; i<kmers.size; i++) {
+					long kmer=kmers.array[i];
+					bloomFilter.filter.increment(kmer);
+				}
+			}
+		}else {
+			final long kmer=toKmer(r.bases, randy.nextInt(r.length()-k2), k);
+			if(kmer>=0){
+				if(keySets!=null) {
+					AbstractKmerTable table=keySets[(int)(kmer%WAYS)];
+					table.increment(kmer, 1);
+				}else {
+					bloomFilter.filter.increment(kmer);
+				}
+			}
+		}
+	}
+	
 	/** Iterate through the reads */
 	public void fillTilesInner(final ConcurrentReadInputStream cris){
-
-
 		Timer t2=new Timer();
 		outstream.print("Filling tiles:  \t");
 		
@@ -642,23 +639,8 @@ public class AnalyzeFlowCell {
 					final MicroTile mt=flowcell.getMicroTile(r1.id);
 					
 					if(loadKmers){
-						if(initialLength1>=k){
-							final long kmer=toKmer(r1.bases, r1.quality, randy.nextInt(initialLength1-k2), k);
-							if(kmer>=0){
-								AbstractKmerTable table=keySets[(int)(kmer%WAYS)];
-								if(table.getValue(kmer)>0){mt.hits++;}
-								else{mt.misses++;}
-							}else{mt.misses++;}
-						}
-						
-						if(initialLength1>=k){
-							final long kmer=toKmer(r1.bases, r1.quality, randy.nextInt(initialLength1-k2), k);
-							if(kmer>=0){
-								AbstractKmerTable table=keySets[(int)(kmer%WAYS)];
-								if(table.getValue(kmer)>0){mt.hits++;}
-								else{mt.misses++;}
-							}else{mt.misses++;}
-						}
+						processTileKmers(r1, mt);
+						processTileKmers(r2, mt);
 					}
 
 					mt.add(r1);
@@ -688,52 +670,29 @@ public class AnalyzeFlowCell {
 			flowcell=flowcell.widen(targetAverageReads);
 			mtList=flowcell.toList();
 		}
-		avgQuality=flowcell.avgQuality;
-		avgUnique=flowcell.avgUnique;
-		avgErrorFree=flowcell.avgErrorFree;
-		avgG=flowcell.avgG;
-		
-		stdQuality=flowcell.stdQuality;
-		stdUnique=flowcell.stdUnique;
-		stdErrorFree=flowcell.stdErrorFree;
-		stdG=flowcell.stdG;
 		
 		long readsToDiscard=markTiles(mtList, flowcell.avgReads);
 		
 		if(dump!=null){
-			TextStreamWriter tsw=new TextStreamWriter(dump, overwrite, append, false);
-			tsw.start();
-
-			tsw.println("#xSize\t"+Tile.xSize);
-			tsw.println("#ySize\t"+Tile.ySize);
-			tsw.println("#reads\t"+String.format(Locale.ROOT, "%d", flowcell.readsProcessed));
-			tsw.println("#avgReads\t"+String.format(Locale.ROOT, "%.1f", flowcell.avgReads));
-			
-			tsw.println("#avgQuality\t"+String.format(Locale.ROOT, "%.3f", avgQuality));
-			tsw.println("#avgUnique\t"+String.format(Locale.ROOT, "%.3f", avgUnique));
-			tsw.println("#avgErrorFree\t"+String.format(Locale.ROOT, "%.3f", avgErrorFree));
-			tsw.println("#avgG\t"+String.format(Locale.ROOT, "%.3f", avgG));
-			
-			tsw.println("#stdQuality\t"+String.format(Locale.ROOT, "%.5f", stdQuality));
-			tsw.println("#stdUnique\t"+String.format(Locale.ROOT, "%.5f", stdUnique));
-			tsw.println("#stdErrorFree\t"+String.format(Locale.ROOT, "%.5f", stdErrorFree));
-			tsw.println("#stdG\t"+String.format(Locale.ROOT, "%.5f", stdG));
-			
-			tsw.println((pound ? "#" : "") + "lane\ttile\tx1\tx2\ty1\ty2\treads\tunique\tquality\tprobErrorFree\tdiscard");
-			
-			for(Lane lane : flowcell.lanes){
-				if(lane!=null){
-					for(Tile tile : lane.tiles){
-						if(tile!=null){
-							tsw.print(tile.toString());
-						}
-					}
-				}
-			}
-			tsw.poisonAndWait();
+			flowcell.dump(dump, overwrite);
 		}
 	}
 	
+	void processTileKmers(Read r, MicroTile mt) {
+		if(r==null || r.length()<k) {return;}
+		final long kmer=toKmer(r.bases, randy.nextInt(r.length()-k2), k);
+		if(kmer<0) {
+			mt.misses++;
+			return;
+		}
+		final long key=(allKmers ? Tools.max(kmer, AminoAcid.reverseComplementBinaryFast(kmer, k)) : kmer);
+		final int value=(keySets==null ? bloomFilter.getCount(key) : keySets[(int)(key%WAYS)].getValue(key));
+		mt.depthSum+=value;
+		final int cutoff=(allKmers ? 2 : 1);
+		if(value>=cutoff) {mt.hits++;}
+		else {mt.misses++;}
+	}
+	
 	/*--------------------------------------------------------------*/
 	/*----------------         Inner Methods        ----------------*/
 	/*--------------------------------------------------------------*/
@@ -772,16 +731,16 @@ public class AnalyzeFlowCell {
 		if(len1>0){
 			double qual=r1.avgQualityByProbabilityDouble(true, len1);
 			double prob=100*r1.probabilityErrorFree(true, len1);
-			if(qual<avgQuality-qDeviations*stdQuality){return false;}
-			if(prob<avgErrorFree-eDeviations*stdErrorFree){return false;}
+			if(qual<flowcell.avgQuality-qDeviations*flowcell.stdQuality){return false;}
+			if(prob<flowcell.avgErrorFree-eDeviations*flowcell.stdErrorFree){return false;}
 			if(discardG && shouldDiscardG(r1, mt)){return false;}
 			if(gToN){gsTransformedToN+=doGToN(r1, mt);}
 		}
 		if(len2>0){
 			double qual=r2.avgQualityByProbabilityDouble(true, len2);
 			double prob=100*r2.probabilityErrorFree(true, len2);
-			if(qual<avgQuality-qDeviations*stdQuality){return false;}
-			if(prob<avgErrorFree-eDeviations*stdErrorFree){return false;}
+			if(qual<flowcell.avgQuality-qDeviations*flowcell.stdQuality){return false;}
+			if(prob<flowcell.avgErrorFree-eDeviations*flowcell.stdErrorFree){return false;}
 			if(discardG && shouldDiscardG(r2, mt)){return false;}
 			if(gToN){gsTransformedToN+=doGToN(r2, mt);}
 		}
@@ -792,7 +751,8 @@ public class AnalyzeFlowCell {
 		final byte[] bases=r.bases;
 		final float[] gArray=mt.tracker.cycleAverages[2];
 		
-		final float thresh=(float)(avgG+Tools.max(gDeviations*stdG, avgG*gFraction, gAbs));
+		final float thresh=(float)(flowcell.avgG+Tools.max(gDeviations*flowcell.stdG, 
+				flowcell.avgG*gFraction, gAbs));
 		for(int i=0; i<bases.length; i++){
 			byte b=bases[i];
 			if(b=='G' && gArray[i]>thresh){
@@ -807,7 +767,8 @@ public class AnalyzeFlowCell {
 		final byte[] quals=r.quality;
 		final float[] gArray=mt.tracker.cycleAverages[2];
 		
-		final float thresh=(float)(avgG+Tools.max(gDeviations*stdG, avgG*gFraction, gAbs));
+		final float thresh=(float)(flowcell.avgG+Tools.max(gDeviations*flowcell.stdG, 
+				flowcell.avgG*gFraction, gAbs));
 		int changes=0;
 		for(int i=0; i<bases.length; i++){
 			byte b=bases[i];
@@ -831,11 +792,7 @@ public class AnalyzeFlowCell {
 	 * @param klen kmer length
 	 * @return kmer
 	 */
-	private static final long toKmer(final byte[] bases, byte[] quals, final int start, final int klen){
-//		if(minprob>0 && quals!=null){
-//			float prob=toProb(quals, start, klen);
-//			if(prob<minprob){return -1;}
-//		}
+	private static final long toKmer(final byte[] bases, final int start, final int klen){
 		final int stop=start+klen;
 		assert(stop<=bases.length) : klen+", "+bases.length;
 		long kmer=0;
@@ -862,26 +819,27 @@ public class AnalyzeFlowCell {
 			double u=mt.uniquePercent();
 			double g=mt.maxG();
 
-			double dq=avgQuality-q;
-			double de=avgErrorFree-e;
-			double du=u-avgUnique;
-			double dg=g-avgG;
+			double dq=flowcell.avgQuality-q;
+			double de=flowcell.avgErrorFree-e;
+			double du=u-flowcell.avgUnique;
+			double dg=g-flowcell.avgG;
 			
 			if(mt.readCount<10 && mt.readCount<0.02f*avgReads){
 				mt.discard++;
 				cDiscards++;
 			}
 			
-			if(dq>qDeviations*stdQuality && dq>avgQuality*qualFraction && dq>qualAbs){
+			if(dq>qDeviations*flowcell.stdQuality && dq>flowcell.avgQuality*qualFraction && dq>qualAbs){
 				mt.discard++;
 				qDiscards++;
 			}
-			if(de>eDeviations*stdErrorFree && de>avgErrorFree*errorFreeFraction && de>errorFreeAbs){
+			if(de>eDeviations*flowcell.stdErrorFree && 
+					de>flowcell.avgErrorFree*errorFreeFraction && de>errorFreeAbs){
 				mt.discard++;
 				eDiscards++;
 			}
-			if(avgUnique>2 && avgUnique<98){
-				if(du>uDeviations*stdUnique && du>avgUnique*uniqueFraction && du>uniqueAbs){
+			if(flowcell.avgUnique>2 && flowcell.avgUnique<98){
+				if(du>uDeviations*flowcell.stdUnique && du>flowcell.avgUnique*uniqueFraction && du>uniqueAbs){
 					mt.discard++;
 					uDiscards++;
 				}
@@ -903,7 +861,7 @@ public class AnalyzeFlowCell {
 //				}
 //			}
 			
-			if((discardG || gToN) && (dg>gDeviations*stdG && dg>avgG*gFraction && dg>gAbs)){
+			if((discardG || gToN) && (dg>gDeviations*flowcell.stdG && dg>flowcell.avgG*gFraction && dg>gAbs)){
 				mt.discard++;
 				gDiscards++;
 			}
@@ -935,8 +893,7 @@ public class AnalyzeFlowCell {
 	/** Secondary input file path */
 	private String in2=null;
 	
-	private String qfin1=null;
-	private String qfin2=null;
+	private ArrayList<String> extra=new ArrayList<String>();
 
 	/** Primary output file path */
 	private String out1=null;
@@ -945,9 +902,6 @@ public class AnalyzeFlowCell {
 
 	/** Discard output file path */
 	private String outbad=null;
-
-	private String qfout1=null;
-	private String qfout2=null;
 	
 	/** Override input file extension */
 	private String extin=null;
@@ -989,6 +943,15 @@ public class AnalyzeFlowCell {
 	/** Hold kmers.  A kmer X such that X%WAYS=Y will be stored in keySets[Y] */
 	private AbstractKmerTable[] keySets;
 	
+	private BloomFilter bloomFilter;
+	
+	private boolean bloomMode=false;
+	private boolean multithreaded=false;
+	
+	private boolean loadKmers=true;
+	private boolean allKmers=false;
+	private int cbits=2;
+	
 	private int targetAverageReads=800;
 
 	private float minprob=0;
@@ -1019,17 +982,16 @@ public class AnalyzeFlowCell {
 	private float errorFreeAbs=1f;
 	private float gAbs=0.05f;
 	
-	private double avgQuality;
-	private double avgUnique;
-	private double avgErrorFree;
-	private double avgG;
-	
-	private double stdQuality;
-	private double stdUnique;
-	private double stdErrorFree;
-	private double stdG;
+//	private double avgQuality;
+//	private double avgUnique;
+//	private double avgErrorFree;
+//	private double avgG;
+//	
+//	private double stdQuality;
+//	private double stdUnique;
+//	private double stdErrorFree;
+//	private double stdG;
 
-	private boolean loadKmers=true;
 	private boolean discardOnlyLowQuality=true;
 	private int discardLevel=1;
 	private boolean gToN=false;
