@@ -18,7 +18,6 @@ import json.JsonObject;
 import shared.Parse;
 import shared.Parser;
 import shared.PreParser;
-import shared.ReadStats;
 import shared.Shared;
 import shared.Timer;
 import shared.Tools;
@@ -29,7 +28,8 @@ import stream.FastaReadInputStream;
 import stream.Read;
 import stream.SamLine;
 import structures.ByteBuilder;
-import structures.EntropyTracker;
+import tracker.EntropyTracker;
+import tracker.ReadStats;
 
 /**
  * Detects inverted repeats in PacBio reads.
@@ -81,7 +81,7 @@ public final class IceCreamFinder {
 		//Set shared static variables prior to parsing
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
 		ReadWrite.USE_BGZIP=ReadWrite.USE_UNBGZIP=ReadWrite.PREFER_BGZIP=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		FASTQ.TEST_INTERLEAVED=FASTQ.FORCE_INTERLEAVED=false;
 		SamLine.SET_FROM_OK=true;
 		Shared.setBufferData(1000000);
@@ -553,7 +553,7 @@ public final class IceCreamFinder {
 		bsw.start();
 		final float mult=1.0f/(hist.length-1);
 
-		bsw.print("#Counted\t").println(Tools.sum(hist));
+		bsw.print("#Counted\t").println(shared.Vector.sum(hist));
 		bsw.print("#Mean\t").println(Tools.averageHistogram(hist)*mult, 3);
 		bsw.print("#Median\t").println(Tools.medianHistogram(hist)*mult, 3);
 		bsw.print("#Mode\t").println(Tools.calcModeHistogram(hist)*mult, 3);
