@@ -67,7 +67,7 @@ public class BBMask{
 		}
 		
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=16;
+		ReadWrite.setZipThreads(16);
 		FASTQ.FORCE_INTERLEAVED=FASTQ.TEST_INTERLEAVED=false;
 		
 		boolean setEntropyMode=false;
