@@ -93,6 +93,8 @@ public class CallPeaks {
 				progressiveMult=Float.parseFloat(b);
 			}else if(a.equals("ploidy")){
 				ploidyClaimed=Integer.parseInt(b);
+			}else if(a.equalsIgnoreCase("ploidyLogic")){
+				ploidyLogic=Integer.parseInt(b);
 			}else if(a.equals("column") || a.equals("col") || a.equals("countcolumn")){
 				countColumn=Integer.parseInt(b);
 			}else if(a.equals("k")){
@@ -302,25 +304,35 @@ public class CallPeaks {
 		double gcmult=1.0/k;
 		if(peaks.size()>0){
 			try {
-				final Peak p0=peaks.get(0);
-				final int center0=p0.center;
-				final int ploidyEstimate=calcPloidy(peaks);
+//				final Peak p0=peaks.get(0);
+//				final int center0=p0.center;
+				float minHetRate=0.0003f; //Less than the human rate of ~0.001, but this will miss the first peak for inbred species.
+				final float minVolumeFraction=Tools.min(1, singleCopyKmerFraction(minHetRate, k, 2));
+				final int ploidyEstimate=calcPloidy(peaks, minVolumeFraction);
 				final int ploidy=ploidyClaimed>0 ? ploidyClaimed : ploidyEstimate;
-				final long errorKmers=errorKmers(p0, hist);
-				final long genomeSize=genomeSize(peaks);
-				final long genomeSize2=genomeSize2(peaks, hist);
+				final float haploidPeakCenter=haploidPeakCenter(peaks, ploidy);
+				final long errorKmers=errorKmers(peaks, hist, minVolumeFraction);
+				final long genomeSizeInPeaks=genomeSizeInPeaks(peaks, ploidy, haploidPeakCenter);
+				final long totalGenomeSize=genomeSize2(peaks, ploidy, haploidPeakCenter, hist);
 				final double gcContent=(gcHist==null ? -1 : gcContent(peaks, k));
 				final double gcContent2=(gcHist==null ? -1 : gcContent2(peaks, k, hist, gcHist));
-				final long repeatSize=repeatSize(peaks, ploidy);
-				final long repeatSize2=hist==null ? -1 : repeatSize2(peaks, ploidy, hist);
-				final long haploidSize=genomeSize2/ploidy;
-				final long hetLocs=calcHetLocations(peaks, ploidy, k);
-				final double hetRate=hetLocs/(double)haploidSize;
-				final double repeatRate=repeatSize*1.0/genomeSize;
-				final double repeatRate2=repeatSize2*1.0/genomeSize2;
+				final long repeatSize=repeatSize(peaks, ploidy, haploidPeakCenter);
+				final long repeatSize2=hist==null ? -1 : repeatSize2(peaks, ploidy, haploidPeakCenter, hist);
+				final long haploidSize=totalGenomeSize/ploidy;
+				final long hetLocs=calcHetLocations(peaks, ploidy, haploidPeakCenter, k);
+				final double hetRate=(hetLocs/(double)haploidSize)/2;//Final /2 is because a SNP will make BOTH paired chromosomes single-copy
+				final double repeatRate=repeatSize*1.0/genomeSizeInPeaks;
+				final double repeatRate2=repeatSize2*1.0/totalGenomeSize;
 				
+				if(verbose) {
+					System.err.println("genomeSize="+genomeSizeInPeaks+", \trepeatSize="+repeatSize);
+					System.err.println("genomeSize2="+totalGenomeSize+",\trepeatSize2="+repeatSize2);
+				}
+				
+				Peak p0=peaks.get(0);
 				Peak ploidyPeak=p0, mainPeak=p0;
-				int target=center0*ploidy, haploidCov;
+				float target=haploidPeakCenter*ploidy;
+				int haploidCov;
 				for(Peak p : peaks){
 					if(p.volume>mainPeak.volume){
 						mainPeak=p;
@@ -332,7 +344,7 @@ public class CallPeaks {
 				if(Tools.max(target,ploidyPeak.center)/(float)Tools.min(target,ploidyPeak.center)<1.3f){
 					haploidCov=ploidyPeak.center;
 				}else{
-					haploidCov=target;
+					haploidCov=(int)target;
 				}
 				
 //				System.err.println("ploidyEstimate="+ploidyEstimate);
@@ -353,8 +365,8 @@ public class CallPeaks {
 				bsw.println("#error_kmers\t"+errorKmers);
 				bsw.println("#genomic_kmers\t"+(uniqueKmers-errorKmers));
 				bsw.println("#main_peak\t"+mainPeak.center);
-				bsw.println("#genome_size_in_peaks\t"+genomeSize);
-				bsw.println("#genome_size\t"+genomeSize2);
+				bsw.println("#genome_size_in_peaks\t"+genomeSizeInPeaks);
+				bsw.println("#genome_size\t"+totalGenomeSize);
 				if(mainPeak.gc>0){
 					bsw.println("#main_peak_gc\t"+String.format(Locale.ROOT, "%.3f", Tools.mid(0, 1, mainPeak.gc*gcmult/mainPeak.volume)));
 				}
@@ -363,7 +375,7 @@ public class CallPeaks {
 					bsw.println("#gc_content\t"+String.format(Locale.ROOT, "%.3f", Tools.mid(0, 1, gcContent2)));
 				}
 				if(ploidy>1 || true){bsw.println("#haploid_genome_size\t"+haploidSize);}
-				bsw.println("#fold_coverage\t"+center0);
+				bsw.println("#fold_coverage\t"+Math.round(haploidPeakCenter));
 				if(ploidy>1 || true){bsw.println("#haploid_fold_coverage\t"+haploidCov);}
 				bsw.println("#ploidy\t"+ploidy);
 				if(ploidy!=ploidyEstimate){bsw.println("#ploidy_detected\t"+ploidyEstimate);}
@@ -389,22 +401,44 @@ public class CallPeaks {
 		errorState|=bsw.poisonAndWait();
 	}
 	
-	private static long errorKmers(Peak p0, long[] hist){
-		if(p0==null || hist==null){return -1;}
+	private static Peak firstGenomicPeak(ArrayList<Peak> peaks, float minFraction) {
+		if(peaks.size()<1) {return null;}
+		assert(minFraction<=1);
+		final Peak biggest=peaks.get(biggestPeak(peaks));
+		final long minVolume=(long)(biggest.volume*minFraction);
+		for(Peak p : peaks) {
+			if(p.volume>=minVolume) {return p;}
+		}
+		assert(false) : peaks.size()+", "+biggest;
+		return null;
+	}
+	
+	//single-copy kmers as a fraction of haploid genome size (result can range from 0 to ploidy).
+	//The math here is all totally wrong and terrible but might be within a factor of 4 of correct
+	private static float singleCopyKmerFraction(float hetRate, int k, int ploidy) {
+		if(ploidy<2) {return 1;}
+		float kmersPerSnp=k;//*(ploidy==2 ? 2 : 1);//Could be 1 or 2 in higher ploidies depending on the structure
+		float singleCopyKmers=hetRate*kmersPerSnp; //Fraction of whole genome size
+		float asymptote=singleCopyKmers/(1+singleCopyKmers);//Prevents the number from going over 1
+		return asymptote*2;
+	}
+	
+	private static long errorKmers(ArrayList<Peak> peaks, long[] hist, float minVolumeFraction){
+		if(peaks.size()<1) {return 0;}
+		final Peak first=firstGenomicPeak(peaks, minVolumeFraction);
+		if(first==null || hist==null){return -1;}
 		long sum=0;
-		for(int i=0; i<p0.start; i++){
+		for(int i=0; i<first.start; i++){
 			sum+=hist[i];
 		}
 		return sum;
 	}
 	
-	private static long genomeSize(ArrayList<Peak> peaks){
+	private static long genomeSizeInPeaks(ArrayList<Peak> peaks, final int ploidy, final float haploidPeakCenter){
 		if(peaks.size()<1){return 0;}
 		
 		long sizeSum=0;
-		final Peak p0=peaks.get(0);
-		final int center0=p0.center;
-		final double mult=1.0/(Tools.max(1, center0));
+		final double mult=1.0/(Tools.max(1, haploidPeakCenter));
 		for(Peak p : peaks){
 			long size=p.volume*(Math.round(p.center*mult));
 			sizeSum+=size;
@@ -412,14 +446,13 @@ public class CallPeaks {
 		return sizeSum;
 	}
 	
-	private static long genomeSize2(ArrayList<Peak> peaks, long[] hist){
+	private static long genomeSize2(ArrayList<Peak> peaks, final int ploidy, final float haploidPeakCenter, long[] hist){
 		if(peaks.size()<1){return 0;}
 		
 		long sizeSum=0;
+		final double mult=1.0/(Tools.max(1, haploidPeakCenter));
+
 		final Peak p0=peaks.get(0);
-		final int center0=p0.center;
-		final double mult=1.0/(Tools.max(1, center0));
-		
 		for(int i=p0.start; i<hist.length; i++){
 			long size=hist[i]*(Tools.max(1, (long)Math.round(i*mult)));
 			sizeSum+=size;
@@ -463,13 +496,11 @@ public class CallPeaks {
 	}
 	
 	/** Based on all peaks */
-	private static long repeatSize(ArrayList<Peak> peaks, int ploidy){
+	private static long repeatSize(ArrayList<Peak> peaks, final int ploidy, final float haploidPeakCenter){
 		if(peaks.size()<2){return 0;}
 		assert(ploidy>0) : ploidy;
-		final int homozygousLoc=homozygousPeak(peaks, ploidy);
-		final Peak p0=peaks.get(0);
-		final int center0=p0.center;
-		final double mult=1.0/(Tools.max(1, center0));
+		final int homozygousLoc=homozygousPeak(peaks, ploidy, haploidPeakCenter);
+		final double mult=1.0/(Tools.max(1, haploidPeakCenter));
 		
 		long sizeSum=0;
 		for(int i=homozygousLoc+1; i<peaks.size(); i++){
@@ -480,16 +511,19 @@ public class CallPeaks {
 		return sizeSum;
 	}
 	
-	/** Based on primary peak only */
-	private static long repeatSize2(ArrayList<Peak> peaks, int ploidy, long[] hist){
+	/** Based on ploidy peak only */
+	private static long repeatSize2(ArrayList<Peak> peaks, final int ploidy, final float haploidPeakCenter, long[] hist){
 		assert(ploidy>0) : ploidy;
-		final int homozygousLoc=homozygousPeak(peaks, ploidy);
-		final Peak p0=peaks.get(0);
-		final int center0=p0.center;
-		final double mult=1.0/(Tools.max(1, center0));
+		final double mult=1.0/(Tools.max(1, haploidPeakCenter));
+		int valley=(int)Math.ceil(haploidPeakCenter*ploidy*(1.2f+1.0f/Tools.max(2,ploidy)));
 		
+		//Optional
+		final int homozygousLoc=homozygousPeak(peaks, ploidy, haploidPeakCenter);
+		if(ploidy>1 && homozygousLoc>=0) {
+			Peak p=peaks.get(homozygousLoc);
+			valley=p.stop+1;
+		}
 		
-		final int valley=(int)Math.ceil(center0*1.7f);
 		long sizeSum=0;
 		for(int i=valley; i<hist.length; i++){
 			long size=hist[i]*(Math.round(i*mult)-1);
@@ -498,6 +532,24 @@ public class CallPeaks {
 		return sizeSum;
 	}
 	
+//	/** Based on primary peak only */
+//	private static long repeatSize2(ArrayList<Peak> peaks, final float haploidPeakCenter, final int ploidy, long[] hist){
+//		assert(ploidy>0) : ploidy;
+////		final int homozygousLoc=homozygousPeak(peaks, haploidPeakCenter, ploidy);
+//		final Peak p0=peaks.get(0);
+//		final int center0=p0.center;
+//		final double mult=1.0/(Tools.max(1, center0));
+//		
+//		
+//		final int valley=(int)Math.ceil(center0*1.7f);
+//		long sizeSum=0;
+//		for(int i=valley; i<hist.length; i++){
+//			long size=hist[i]*(Math.round(i*mult)-1);
+//			sizeSum+=size;
+//		}
+//		return sizeSum;
+//	}
+	
 	private static int biggestPeak(ArrayList<Peak> peaks){
 		if(peaks.size()<2){return peaks.size()-1;}
 
@@ -545,18 +597,17 @@ public class CallPeaks {
 		return sloc;
 	}
 	
-	private static int homozygousPeak(ArrayList<Peak> peaks, final int ploidy){
+	private static int homozygousPeak(ArrayList<Peak> peaks, final int ploidy, final float haploidPeakCenter){
 		if(peaks.size()<2){return peaks.size()-1;}
 		assert(ploidy>0) : ploidy;
 
-		final Peak p0=peaks.get(0);
-		final int target=p0.center*ploidy;
+		final float target=haploidPeakCenter*ploidy;
 //		System.err.println("target="+target);
-		int bestDif=Integer.MAX_VALUE;
+		float bestDif=Integer.MAX_VALUE;
 		int loc=0;
 		for(int i=0; i<peaks.size(); i++){
 			Peak p=peaks.get(i);
-			int dif=Tools.absdif(target, p.center);
+			float dif=Tools.absdif(target, p.center);
 //			System.err.println("dif="+dif+" for peak "+i+", center "+p.center);
 			if(dif<bestDif){
 				bestDif=dif;
@@ -567,30 +618,62 @@ public class CallPeaks {
 		return loc;
 	}
 	
-	private static int calcPloidy(ArrayList<Peak> peaks){
+	private static float haploidPeakCenter(ArrayList<Peak> peaks, int ploidy) {
+		assert(ploidy>0);
+		final Peak biggest=peaks.get(biggestPeak(peaks));
+		final Peak second=peaks.get(secondBiggestPeak(peaks));
+		if(second.volume*4>=biggest.volume) {//Similar volume; lowest is haploid peak
+			return Tools.min(biggest.center, second.center);
+		}
+		//Assume biggest is ploidy peak
+		return biggest.center/(float)ploidy;
+	}
+	
+	private static int calcPloidy(ArrayList<Peak> peaks, float minVolumeFraction){
 		if(peaks.size()<2){return 1;}
 
 		final Peak p0=peaks.get(0);
 		final Peak biggest=peaks.get(biggestPeak(peaks));
+		final Peak second=peaks.get(secondBiggestPeak(peaks));
 		
-		if(biggest!=p0){
-			int ratio=Math.round((biggest.center/(float)p0.center));
-//			System.err.println("A: "+biggest+", "+((biggest.center/(float)p0.center)));
-			return ratio;
-		}else{//p0 is biggest.
-			final Peak second=peaks.get(secondBiggestPeak(peaks));
-//			System.err.println("B: "+biggest+" > "+second);
-			if(second.volume*4<biggest.volume){return 1;} //Probably second is a repeat peak.
-			int ratio=(int)(second.center/(float)biggest.center);
-//			System.err.println("C: "+ratio);
-			return ratio;
+		if(ploidyLogic==1) {//Old code path, assumes no error/contamination peaks
+			if(biggest==p0){//p0 is biggest.
+				//			System.err.println("a: "+biggest+" > "+second);
+				if(second.volume*4<biggest.volume){return 1;} //Probably second is a repeat peak.
+				int ratio=Math.round((second.center/(float)biggest.center));
+				//			System.err.println("b: "+ratio);
+				return Tools.max(1, ratio);
+			}else {//p0 is not biggest.
+				//I wonder if this should be biggest/second...
+				int ratio=Math.round((biggest.center/(float)p0.center));
+				//			System.err.println("c: "+biggest+", "+((biggest.center/(float)p0.center)));
+				return ratio;
+			}
+		}else {
+
+			if(second==biggest) {
+				return 1;
+			}else if(second.center<biggest.center) {//Second volume could be low, and is based on het rate
+				if(second.volume<biggest.volume*minVolumeFraction) {
+					//Second peak is tiny and likely contamination.
+					return 1;
+				}
+			}else {//second volume should be substantial since it is the heterozygous fraction
+				if(second.volume*4<biggest.volume) {
+					//Second peak is small and likely repeat.
+					return 1;
+				}
+			}
+			final int max=Tools.max(biggest.center, second.center);
+			final int min=Tools.min(biggest.center, second.center);
+			return Math.round(max/(float)min);
 		}
 	}
 	
-	private static long calcHetLocations(ArrayList<Peak> peaks, final int ploidy, final int k){
+	private static long calcHetLocations(ArrayList<Peak> peaks, final int ploidy, final float haploidPeakCenter, final int k){
 		if(peaks.size()<2){return 0;}
 		assert(ploidy>0) : ploidy;
-		final int homozygousLoc=homozygousPeak(peaks, ploidy);
+		final int homozygousLoc=homozygousPeak(peaks, ploidy, haploidPeakCenter);
 		final Peak homoPeak=peaks.get(homozygousLoc);
 		long sum=0;
 		final int lim=ploidy/2;
@@ -1120,5 +1203,6 @@ public class CallPeaks {
 	public static boolean weightByRelief=false;
 	public static final int BY_VOLUME=0, BY_HEIGHT=1;
 	public static int CALL_MODE=BY_VOLUME;
+	public static int ploidyLogic=2;
 	
 }
