@@ -19,6 +19,7 @@ import fileIO.ReadWrite;
 import fileIO.TextFile;
 import fileIO.TextStreamWriter;
 import shared.KillSwitch;
+import shared.LineParser1;
 import shared.Parse;
 import shared.Parser;
 import shared.PreParser;
@@ -100,6 +101,11 @@ public class CoveragePileup {
 				in1=b;
 			}else if(a.equals("in2")){
 				in2=b;
+			}else if(a.equals("covin") || a.equals("incov")){
+				covIn.clear();
+				if(b!=null) {
+					for(String cf : Tools.commaPattern.split(b)) {covIn.add(cf);}
+				}
 			}else if(a.equals("out") || a.equals("coveragestats") || a.equals("covstats") || a.equals("stats")){
 				covstats=b;
 				outset=true;
@@ -434,7 +440,12 @@ public class CoveragePileup {
 		processReference();
 //		System.err.println("E");
 		if(maxReads<0){maxReads=Long.MAX_VALUE;}
-		if(useStreamer && !FileFormat.isStdio(in1)){
+		
+		if(covIn!=null && !covIn.isEmpty()) {
+			for(String s : covIn) {
+				loadCoverageFile(s);
+			}
+		}else if(useStreamer && !FileFormat.isStdio(in1)){
 			tf.close();
 //			System.err.println("F");
 			processViaStreamer(tsw);
@@ -489,6 +500,43 @@ public class CoveragePileup {
 		if(tsw!=null){tsw.poison();}
 	}
 	
+	//Untested
+	//See also CoverageArray.loadDepth; that implementation is better (also untested)
+	private void loadCoverageFile(String fname) {
+		ByteFile bf=ByteFile.makeByteFile(fname, true);
+		
+		byte[] line=bf.nextLine();
+		for(; line!=null && Tools.startsWith(line, '#'); line=bf.nextLine()) {} //Skip header
+		if(line==null) {return;}
+		lp.set(line);
+		String name=lp.parseString(0);
+		Scaffold scaf=table.get(name);
+		
+		for(; line!=null; line=bf.nextLine()) {
+			assert(!Tools.startsWith(line, '#'));
+			lp.set(line);
+			int len=lp.length(0);
+			if(len>0 && !lp.termEquals(name, 0)) {
+				name=lp.parseString(0);
+				scaf=table.get(name);
+			}
+			int pos=lp.parseInt(1);
+			int cov=lp.parseInt(2);
+			if(cov>0) {
+				if(USE_COVERAGE_ARRAYS){
+					if(scaf.obj0==null){
+						scaf.obj0=(bits32 ? new CoverageArray3(table.size(), scaf.length) : new CoverageArray2(table.size(), scaf.length));
+					}
+					CoverageArray ca=(CoverageArray)scaf.obj0;
+					ca.increment(pos, cov);
+				}else if(USE_BITSETS){
+					if(scaf.obj0==null){scaf.obj0=new BitSet(scaf.length+1);}
+					BitSet bs=(BitSet)scaf.obj0;
+					bs.set(pos);
+				}
+			}
+		}
+	}
 	
 	/*--------------------------------------------------------------*/
 	/*----------------             Setup            ----------------*/
@@ -508,7 +556,8 @@ public class CoveragePileup {
 				final byte a=line[1], b=line[2];
 
 				if(a=='S' && b=='Q'){
-					Scaffold scaf=new Scaffold(line);
+					lp.set(line);
+					Scaffold scaf=new Scaffold(lp);
 					if(COUNT_GC){scaf.basecount=KillSwitch.allocLong1D(8);}
 					assert(!table.containsKey(scaf.name)) : "\nDuplicate scaffold name!\n"+scaf+"\n\n"+table.get(scaf.name);
 					table.put(scaf.name, scaf);
@@ -517,12 +566,18 @@ public class CoveragePileup {
 //					sc.obj=new CoverageArray2(table.size(), sc.length+1);
 //					outstream.println("Made scaffold "+sc.name+" of length "+sc.length);
 				}else if(a=='P' && b=='G'){
-					String[] split=new String(line).split("\t");
-					for(String s : split){
-						if(s.startsWith("PN:")){
-							if(program==null){program=Data.forceIntern(s.substring(3));}
-						}else if(s.startsWith("VN:")){
-							if(version==null){version=Data.forceIntern(s.substring(3));}
+					lp.set(line);
+					for(int i=1, terms=lp.terms(); i<terms; i++) {
+						if(lp.termStartsWith("PN:", i)){
+							if(program==null){
+								lp.incrementA(3);
+								program=lp.parseStringFromCurrentField();
+							}
+						}else if(lp.termStartsWith("VN:", i)){
+							if(version==null){
+								lp.incrementA(3);
+								version=lp.parseStringFromCurrentField();
+							}
 						}
 					}
 				}else if(a=='R' && b=='G'){
@@ -706,7 +761,7 @@ public class CoveragePileup {
 						}
 					}
 					
-					CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj2 : scaf.obj1); //TODO:  Strand logic here depends on stranding protocol.
+					CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj1 : scaf.obj0); //TODO:  Strand logic here depends on stranding protocol.
 					orf.readCoverageArray(ca);
 				}
 				
@@ -817,13 +872,13 @@ public class CoveragePileup {
 		scaf.basehits+=basehits;
 		
 		if(USE_COVERAGE_ARRAYS){
-			if(scaf.obj1==null){
-				scaf.obj1=(bits32 ? new CoverageArray3(table.size(), scaf.length+1) : new CoverageArray2(table.size(), scaf.length+1));
+			if(scaf.obj0==null){
+				scaf.obj0=(bits32 ? new CoverageArray3(table.size(), scaf.length) : new CoverageArray2(table.size(), scaf.length));
 				if(STRANDED){
-					scaf.obj2=(bits32 ? new CoverageArray3(table.size(), scaf.length+1) : new CoverageArray2(table.size(), scaf.length+1));
+					scaf.obj1=(bits32 ? new CoverageArray3(table.size(), scaf.length) : new CoverageArray2(table.size(), scaf.length));
 				}
 			}
-			CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj2 : scaf.obj1);
+			CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj1 : scaf.obj0);
 			if(START_ONLY){
 				ca.increment(start);
 			}else if(STOP_ONLY){
@@ -837,13 +892,13 @@ public class CoveragePileup {
 				//123
 			}
 		}else if(USE_BITSETS){
-			if(scaf.obj1==null){
-				scaf.obj1=new BitSet(scaf.length+1);
+			if(scaf.obj0==null){
+				scaf.obj0=new BitSet(scaf.length+1);
 				if(STRANDED){
-					scaf.obj2=new BitSet(scaf.length+1);
+					scaf.obj1=new BitSet(scaf.length+1);
 				}
 			}
-			BitSet bs=(BitSet)(STRANDED && strand==1 ? scaf.obj2 : scaf.obj1);
+			BitSet bs=(BitSet)(STRANDED && strand==1 ? scaf.obj1 : scaf.obj0);
 			if(START_ONLY){
 				bs.set(start);
 			}else if(STOP_ONLY){
@@ -866,13 +921,13 @@ public class CoveragePileup {
 		
 		int basehits=0;
 		if(USE_COVERAGE_ARRAYS){
-			if(scaf.obj1==null){
-				scaf.obj1=(bits32 ? new CoverageArray3(table.size(), scaf.length+1) : new CoverageArray2(table.size(), scaf.length+1));
+			if(scaf.obj0==null){
+				scaf.obj0=(bits32 ? new CoverageArray3(table.size(), scaf.length) : new CoverageArray2(table.size(), scaf.length));
 				if(STRANDED){
-					scaf.obj2=(bits32 ? new CoverageArray3(table.size(), scaf.length+1) : new CoverageArray2(table.size(), scaf.length+1));
+					scaf.obj1=(bits32 ? new CoverageArray3(table.size(), scaf.length) : new CoverageArray2(table.size(), scaf.length));
 				}
 			}
-			CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj2 : scaf.obj1);
+			CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj1 : scaf.obj0);
 			for(int rpos=start, mpos=0; mpos<match.length && rpos<=stop; mpos++){
 				byte m=match[mpos];
 				if(m=='m' || m=='S' || m=='N'){
@@ -888,13 +943,13 @@ public class CoveragePileup {
 				}
 			}
 		}else if(USE_BITSETS){
-			if(scaf.obj1==null){
-				scaf.obj1=new BitSet(scaf.length+1);
+			if(scaf.obj0==null){
+				scaf.obj0=new BitSet(scaf.length+1);
 				if(STRANDED){
-					scaf.obj2=new BitSet(scaf.length+1);
+					scaf.obj1=new BitSet(scaf.length+1);
 				}
 			}
-			BitSet bs=(BitSet)(STRANDED && strand==1 ? scaf.obj2 : scaf.obj1);
+			BitSet bs=(BitSet)(STRANDED && strand==1 ? scaf.obj1 : scaf.obj0);
 			for(int rpos=start, mpos=0; mpos<match.length && rpos<=stop; mpos++){
 				byte m=match[mpos];
 				if(m=='m' || m=='S' || m=='N'){
@@ -928,8 +983,10 @@ public class CoveragePileup {
 			}
 			
 			if(line[1]=='S' && line[2]=='Q'){
-				Scaffold scaf=new Scaffold(line);
-				if(!table.containsKey(scaf.name)){
+				lp.set(line);
+				String name=Scaffold.name(lp);
+				if(!table.containsKey(name)){
+					Scaffold scaf=new Scaffold(lp);
 					if(COUNT_GC){scaf.basecount=KillSwitch.allocLong1D(8);}
 					table.put(scaf.name, scaf);
 					list.add(scaf);
@@ -1329,7 +1386,7 @@ public class CoveragePileup {
 			int underWindowAverage=0;
 			final double stdev;
 			if(USE_COVERAGE_ARRAYS){
-				CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj2 : scaf.obj1);
+				CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj1 : scaf.obj0);
 				if(ca!=null){
 					for(int i=0; i<scaf.length; i++){
 						int x=ca.get(i);
@@ -1358,7 +1415,7 @@ public class CoveragePileup {
 				}
 			}else if(USE_BITSETS){
 //				sum+=scaf.basehits;
-				BitSet bs=(BitSet)(STRANDED && strand==1 ? scaf.obj2 : scaf.obj1);
+				BitSet bs=(BitSet)(STRANDED && strand==1 ? scaf.obj1 : scaf.obj0);
 				covered=(bs==null ? 0 : bs.cardinality());
 				stdev=-1;
 			}else{
@@ -1477,7 +1534,7 @@ public class CoveragePileup {
 		
 		for(Scaffold scaf : list){
 			int last=-1;
-			CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj2 : scaf.obj1);
+			CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj1 : scaf.obj0);
 			if(scaf.length>=minscaf){
 				for(int i=0, len=scaf.length; i<len; i++){
 					int x=(ca==null ? 0 : ca.get(i));
@@ -1521,7 +1578,7 @@ public class CoveragePileup {
 		for(Scaffold scaf : list){
 			tsw.print('#');
 			tsw.println(scaf.name);
-			CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj2 : scaf.obj1);
+			CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj1 : scaf.obj0);
 			if(scaf.length>=minscaf){
 				for(int i=0; i<scaf.length; i++){
 					int x=(ca==null ? 0 : ca.get(i));
@@ -1568,7 +1625,7 @@ public class CoveragePileup {
 		final float invbin=1f/binsize;
 		for(Scaffold scaf : list){
 			if(scaf.length>=binsize && scaf.length>=minscaf){
-				CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj2 : scaf.obj1);
+				CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj1 : scaf.obj0);
 				int lastPos=-1, nextPos=binsize-1;
 				long sum=0;
 				for(int i=0; i<scaf.length; i++){
@@ -1615,7 +1672,7 @@ public class CoveragePileup {
 		
 		long running=0;
 		for(Scaffold scaf : list){
-			CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj2 : scaf.obj1);
+			CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj1 : scaf.obj0);
 			int lastPos=-1, nextPos=binsize-1;
 			long sum=0;
 			final int lim=scaf.length-1;
@@ -1645,7 +1702,7 @@ public class CoveragePileup {
 		
 		LongList list=new LongList();
 		for(Scaffold scaf : scaffolds){
-			CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj2 : scaf.obj1);
+			CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj1 : scaf.obj0);
 			int lastPos=-1, nextPos=binsize-1;
 			long sum=0;
 			final int lim=scaf.length-1;
@@ -1678,7 +1735,7 @@ public class CoveragePileup {
 		
 		for(Scaffold scaf : scaffolds){
 			if(scaf.length>=minscaf){
-				CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj2 : scaf.obj1);
+				CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj1 : scaf.obj0);
 				int lastPos=-1, nextPos=binsize-1;
 				long tempSum=0;
 				final int lim=scaf.length-1;
@@ -1704,7 +1761,7 @@ public class CoveragePileup {
 		
 		for(Scaffold scaf : scaffolds){
 			if(scaf.length>=minscaf){
-				CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj2 : scaf.obj1);
+				CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj1 : scaf.obj0);
 				int lastPos=-1, nextPos=binsize-1;
 				long tempSum=0;
 				final int lim=scaf.length-1;
@@ -1729,15 +1786,18 @@ public class CoveragePileup {
 	}
 
 	public static double[] standardDeviation(ArrayList<Scaffold> scaffolds, int strand, int minscaf){
-		long totalSum=0, bins=0;
+		long totalSum=0, bins=0, scafsCounted=0;
 		for(Scaffold scaf : scaffolds){
 //			System.err.println(scaf.name+", "+scaf.length+"/"+minscaf);
 			if(scaf.length>=minscaf){
-				final CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj2 : scaf.obj1);
+				final CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj1 : scaf.obj0);
 				bins+=scaf.length;
-				for(int i=0; i<scaf.length; i++){
-					int depth=(ca==null ? 0 : ca.get(i));
-					totalSum+=depth;
+				scafsCounted++;
+				if(ca!=null) {
+					for(int i=0; i<scaf.length; i++){
+						int depth=(ca==null ? 0 : ca.get(i));
+						totalSum+=depth;
+					}
 				}
 			}
 		}
@@ -1745,22 +1805,26 @@ public class CoveragePileup {
 		if(bins<1){return new double[] {0, 0};}
 		final double mean=totalSum/(double)bins;
 		double sumdev2=0;
-//		System.err.println(totalSum+", "+bins+", "+mean);
+//		System.err.println("sum="+totalSum+", bins="+bins+", mean="+mean+", scaffolds="+scafsCounted);
 		for(Scaffold scaf : scaffolds){
 			if(scaf.length>=minscaf){
-				final CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj2 : scaf.obj1);
+				final CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj1 : scaf.obj0);
 				double sumTemp=0;
-				for(int i=0; i<scaf.length; i++){
-					int depth=(ca==null ? 0 : ca.get(i));
-					double dev=mean-depth;
-					sumTemp+=(dev*dev);
+				if(ca!=null) {
+					for(int i=0; i<scaf.length; i++){
+						int depth=(ca==null ? 0 : ca.get(i));
+						double dev=mean-depth;
+						sumTemp+=(dev*dev);
+					}
+				}else {
+					sumTemp+=(mean*mean*scaf.length);
 				}
 				sumdev2+=sumTemp;
 			}
 		}
 
 		final double stdev=Math.sqrt(sumdev2/bins);
-//		System.err.println(totalSum+", "+bins+", "+mean+", "+stdev);
+//		System.err.println("sumdev2="+sumdev2+", stdev="+stdev);
 		return new double[] {mean, stdev};
 	}
 	
@@ -1792,7 +1856,7 @@ public class CoveragePileup {
 			if(scaf.length>=binsize && scaf.length>=minscaf){
 				long max=-1;
 				
-				final CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj2 : scaf.obj1);
+				final CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj1 : scaf.obj0);
 				double lastPos=-1, nextPos=binsize-1;
 				long sum=0;
 
@@ -1869,7 +1933,7 @@ public class CoveragePileup {
 
 				if(scaf.readhits>0){
 					long max=-1;
-					final CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj2 : scaf.obj1);
+					final CoverageArray ca=(CoverageArray)(STRANDED && strand==1 ? scaf.obj1 : scaf.obj0);
 					double lastPos=-1, nextPos=binsize-1;
 					long sum=0;
 
@@ -2000,6 +2064,8 @@ public class CoveragePileup {
 	public String in1=null;
 	/** Secondary input file (typically for coverage directly from BBMap) */
 	public String in2=null;
+	
+	public ArrayList<String> covIn=new ArrayList<String>();
 	/** Optional, for calculating GC */
 	public String reference=null;
 	public String orffasta=null;
@@ -2076,6 +2142,8 @@ public class CoveragePileup {
 	
 	private boolean errorState=false;
 	
+	private final LineParser1 lp=new LineParser1('\t');
+	
 	/*--------------------------------------------------------------*/
 	/*----------------        Static Fields         ----------------*/
 	/*--------------------------------------------------------------*/
