@@ -42,7 +42,7 @@ public class DemuxByName {
 
 	public static void main(String[] args){
 		
-		final int oldCap=Shared.numBuffers(), oldZipThreads=ReadWrite.MAX_ZIP_THREADS, oldZl=ReadWrite.ZIPLEVEL;
+		final int oldCap=Shared.numBuffers(), oldZipThreads=ReadWrite.MAX_ZIP_THREADS(), oldZl=ReadWrite.ZIPLEVEL;
 		final boolean oldPigz=ReadWrite.USE_PIGZ, oldUnpigz=ReadWrite.USE_UNPIGZ;
 		
 		Timer t=new Timer();
@@ -53,7 +53,7 @@ public class DemuxByName {
 		ReadWrite.ZIPLEVEL=oldZl;
 		ReadWrite.USE_PIGZ=oldPigz;
 		ReadWrite.USE_UNPIGZ=oldUnpigz;
-		ReadWrite.MAX_ZIP_THREADS=oldZipThreads;
+		ReadWrite.setZipThreads(oldZipThreads);
 		
 		//Close the print stream if it was redirected
 		Shared.closeStream(x.outstream);
@@ -71,7 +71,7 @@ public class DemuxByName {
 		
 		Shared.capBuffers(4);
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		ReadWrite.ZIPLEVEL=1;
 		
 		
@@ -118,67 +118,7 @@ public class DemuxByName {
 			}else if(a.equals("perheader") || a.equals("persequence") || a.equals("pername")){
 				perheader=Parse.parseBoolean(b);
 			}else if(a.equals("delimiter")){
-				if(b==null){delimiter=null;}
-				
-				//Convenience characters
-				else if(b.equalsIgnoreCase("space")){
-					delimiter=" ";
-				}else if(b.equalsIgnoreCase("tab")){
-					delimiter="\t";
-				}else if(b.equalsIgnoreCase("whitespace")){
-					delimiter="\\s+";
-				}else if(b.equalsIgnoreCase("pound")){
-					delimiter="#";
-				}else if(b.equalsIgnoreCase("greaterthan")){
-					delimiter=">";
-				}else if(b.equalsIgnoreCase("lessthan")){
-					delimiter="<";
-				}else if(b.equalsIgnoreCase("equals")){
-					delimiter="=";
-				}else if(b.equalsIgnoreCase("colon")){
-					delimiter=":";
-				}else if(b.equalsIgnoreCase("semicolon")){
-					delimiter=";";
-				}else if(b.equalsIgnoreCase("bang")){
-					delimiter="!";
-				}else if(b.equalsIgnoreCase("and") || b.equalsIgnoreCase("ampersand")){
-					delimiter="&";
-				}else if(b.equalsIgnoreCase("quote") || b.equalsIgnoreCase("doublequote")){
-					delimiter="\"";
-				}else if(b.equalsIgnoreCase("singlequote") || b.equalsIgnoreCase("apostrophe")){
-					delimiter="'";
-				}
-				
-				//Java meta characters
-				else if(b.equalsIgnoreCase("backslash")){
-					delimiter="\\\\";
-				}else if(b.equalsIgnoreCase("hat") || b.equalsIgnoreCase("caret")){
-					delimiter="\\^";
-				}else if(b.equalsIgnoreCase("dollar")){
-					delimiter="\\$";
-				}else if(b.equalsIgnoreCase("dot")){
-					delimiter="\\.";
-				}else if(b.equalsIgnoreCase("pipe") || b.equalsIgnoreCase("or")){
-					delimiter="\\|";
-				}else if(b.equalsIgnoreCase("questionmark")){
-					delimiter="\\?";
-				}else if(b.equalsIgnoreCase("star") || b.equalsIgnoreCase("asterisk")){
-					delimiter="\\*";
-				}else if(b.equalsIgnoreCase("plus")){
-					delimiter="\\+";
-				}else if(b.equalsIgnoreCase("openparen")){
-					delimiter="\\(";
-				}else if(b.equalsIgnoreCase("closeparen")){
-					delimiter="\\)";
-				}else if(b.equalsIgnoreCase("opensquare")){
-					delimiter="\\[";
-				}else if(b.equalsIgnoreCase("opencurly")){
-					delimiter="\\{";
-				}
-				
-				else{
-					delimiter=b;
-				}
+				delimiter=Parse.parseSymbol(b);
 			}else if(parser.in1==null && i==0 && !arg.contains("=") && (arg.toLowerCase().startsWith("stdin") || new File(arg).exists())){
 				parser.in1=arg;
 			}else{
@@ -231,7 +171,7 @@ public class DemuxByName {
 			
 			assert((affixLengths.length>0 && affixLengths[0]>0) || delimiter!=null || perheader) : 
 				"Must include at least one non-zero-length affix (name), or a delimiter.";
-			ReadWrite.MAX_ZIP_THREADS=Tools.max(1, Tools.min(ReadWrite.MAX_ZIP_THREADS, (Shared.threads()*2-1)/Tools.max(1, names.size())));
+			ReadWrite.setZipThreads(Tools.max(1, Tools.min(ReadWrite.MAX_ZIP_THREADS(), (Shared.threads()*2-1)/Tools.max(1, names.size()))));
 			if(names.size()>8){ReadWrite.USE_PIGZ=false;}
 		}
 		
