@@ -5,18 +5,26 @@ import java.io.PrintStream;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.BitSet;
+import java.util.Collection;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.LinkedHashSet;
 import java.util.Locale;
 import java.util.Map.Entry;
 import java.util.regex.Pattern;
 
+import barcode.Barcode;
+import barcode.BarcodeCounter;
+import barcode.BarcodeStats;
+import barcode.PCRMatrix;
 import dna.AminoAcid;
 import fileIO.ByteFile;
 import fileIO.ByteStreamWriter;
 import fileIO.FileFormat;
 import fileIO.ReadWrite;
 import fileIO.TextFile;
+import hiseq.IlluminaHeaderParser1;
+import hiseq.IlluminaHeaderParser2;
 import shared.Parse;
 import shared.Parser;
 import shared.PreParser;
@@ -28,8 +36,6 @@ import stream.ConcurrentReadInputStream;
 import stream.ConcurrentReadOutputStream;
 import stream.FASTQ;
 import stream.FastaReadInputStream;
-import stream.MultiCros2;
-import stream.MultiCros3;
 import stream.Read;
 import stream.SamLine;
 import structures.ByteBuilder;
@@ -52,7 +58,7 @@ public class DemuxByName2 {
 	public static void main(String[] args){
 		
 		//Capture values of static variables that might be modified in case this is called by another class.
-		final int oldCap=Shared.numBuffers(), oldZipThreads=ReadWrite.MAX_ZIP_THREADS, oldZl=ReadWrite.ZIPLEVEL;
+		final int oldCap=Shared.numBuffers(), oldZipThreads=ReadWrite.MAX_ZIP_THREADS(), oldZl=ReadWrite.ZIPLEVEL;
 		
 		//External compressor selection
 		final boolean oldPigz=ReadWrite.USE_PIGZ, oldUnpigz=ReadWrite.USE_UNPIGZ;
@@ -77,7 +83,7 @@ public class DemuxByName2 {
 		ReadWrite.USE_BGZIP=oldBgzip;
 		ReadWrite.PREFER_BGZIP=oldPreferBgzip;
 		ReadWrite.USE_UNPIGZ=oldUnpigz;
-		ReadWrite.MAX_ZIP_THREADS=oldZipThreads;
+		ReadWrite.setZipThreads(oldZipThreads);
 		Read.CHANGE_QUALITY=oldCQ;
 		
 		//Close the print stream if it was redirected
@@ -104,7 +110,10 @@ public class DemuxByName2 {
 		Shared.capBuffers(4);
 		ReadWrite.USE_PIGZ=true;
 		ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
+		if(ReadWrite.MAX_ZIP_THREADS()>=8) {
+			ReadWrite.setZipThreadMult(8f/ReadWrite.MAX_ZIP_THREADS());
+		}
 		SamLine.SET_FROM_OK=true;
 		
 		//Reduce default compression level to increase speed
@@ -142,6 +151,8 @@ public class DemuxByName2 {
 			extin=parser.extin;
 			extout=parser.extout;
 		}
+
+		PCRMatrix.postParseStatic();
 		
 		//Ensure the input and output files are specified correctly
 		validate();
@@ -150,6 +161,12 @@ public class DemuxByName2 {
 		ffin1=FileFormat.testInput(in1, FileFormat.FASTQ, extin, true, true);
 		ffin2=FileFormat.testInput(in2, FileFormat.FASTQ, extin, true, true);
 
+		if(useMatrix || mode==BARCODE_MODE) {
+			barcodeDelimiter=ffin1.barcodeDelimiter();
+			barcodeLength1=ffin1.barcodeLength(1);
+			barcodeLength2=ffin1.barcodeLength(2);
+		}
+		
 		//If the input and output are in sam/bam format, print the sam header
 		if(ffin1!=null && out1!=null && ffin1.samOrBam()){
 			String ext=ReadWrite.rawExtension(out1);
@@ -158,6 +175,7 @@ public class DemuxByName2 {
 		
 		//Set up data structures
 		handleModeAndNames();
+		if(mapOut!=null) {BarcodeCounter.writeAssignmentMap(assignmentMap, mapOut, overwrite, append);}
 	}
 	
 	/** 
@@ -181,28 +199,35 @@ public class DemuxByName2 {
 			if(a.equals("verbose")){
 				verbose=Parse.parseBoolean(b);
 				BufferedMultiCros.verbose=verbose;
-			}else if(a.equals("names") || a.equals("name") || a.equals("affixes")){
+			}else if(a.equals("verbosemcros")){
+				BufferedMultiCros.verbose=Parse.parseBoolean(b);
+			}else if(a.equals("names") || a.equals("name") || a.equals("affixes") || a.equals("expected")){
 				if(b!=null){
 					String[] x=b.split(",");
 					for(String s : x){
-						names.put(s, s);
+						assignmentMap.put(s, s);
 					}
 				}
-			}else if(a.equals("threaded")){
-				threaded=Parse.parseBoolean(b);
 			}else if(a.equals("chrom") || a.equals("chrommode") || a.equals("scaffold")){
 				if(Parse.parseBoolean(b)){mode=CHROM_MODE;}
 			}else if(a.equals("substringmode") || a.equals("substring")){
 				if(Parse.parseBoolean(b)){mode=SUBSTRING_MODE;}
 			}else if(a.equals("barcode") || a.equals("barcodemode") || a.equals("index")){
 				if(Parse.parseBoolean(b)){mode=BARCODE_MODE;}
+			}else if(a.equals("tile")){
+				if(Parse.parseBoolean(b)){mode=TILE_MODE;}
 			}else if(a.equals("perheader") || a.equals("persequence") || a.equals("header") || a.equals("headermode")){
 				if(Parse.parseBoolean(b)){mode=HEADER_MODE;}
 			}else if(a.equals("affix") || a.equals("affixmode")){
 				if(Parse.parseBoolean(b)){mode=AFFIX_MODE;}
+			}else if(a.equals("pcr") || a.equals("pcrmode") || a.equals("probability") || a.equals("prob")){
+				useMatrix=Parse.parseBoolean(b);
+			}else if(a.equals("hdistsum") || a.equals("pairhdist") || a.equals("hdistpair") || a.equals("sumhdist")){
+				hdistSum=Parse.parseBoolean(b);
+				PCRMatrix.parseStatic(arg, a, b);
 			}else if(a.equals("delimiter")){
 				delimiter=Parse.parseSymbol(b);
-				if(delimiter!=null){mode=DELIMITER_MODE;}
+				if(delimiter!=null && mode==AFFIX_MODE){mode=DELIMITER_MODE;}
 			}else if(a.equals("prefixmode") || a.equals("prefix") || a.equals("pm")){
 				prefixMode=Parse.parseBoolean(b);
 			}else if(a.equals("suffixmode") || a.equals("suffix") || a.equals("sm")){
@@ -217,16 +242,12 @@ public class DemuxByName2 {
 				stats=b;
 			}else if(a.equalsIgnoreCase("minreadstodump") || a.equalsIgnoreCase("minreads")){
 				minReadsToDump=Parse.parseKMG(b);
-			}else if(a.equalsIgnoreCase("mcrostype")){
-				mcrosType=Integer.parseInt(b);
-			}else if(a.equalsIgnoreCase("streams") || a.equalsIgnoreCase("maxstreams")){
-				maxStreams=Integer.parseInt(b);
-			}else if(a.equalsIgnoreCase("readsperbuffer") || a.equalsIgnoreCase("rpb")){
-				readsPerBuffer=Parse.parseIntKMG(b);
-			}else if(a.equalsIgnoreCase("bytesPerBuffer") || a.equalsIgnoreCase("bpb")){
-				bytesPerBuffer=Parse.parseIntKMG(b);
-			}else if(a.equals("hdist") || a.equals("hamming") || a.equals("hammingdistance")){
+			}else if(a.equalsIgnoreCase("printRetireTime")){
+				printRetireTime=Parse.parseBoolean(b);
+			}else if(a.equals("hdist") || a.equals("hamming") || 
+					a.equals("hammingdistance") || a.equals("maxhdist")){
 				hdist=Integer.parseInt(b);
+				PCRMatrix.parseStatic(arg, a, b);
 			}else if(a.equals("outu") || a.equals("outu1")){
 				outu1=b;
 			}else if(a.equals("outu2")){
@@ -239,6 +260,28 @@ public class DemuxByName2 {
 				remap=Parse.parseRemap(b);
 			}
 			
+			else if(a.equals("countsin")){
+				countsIn=b;
+			}else if(a.equals("countsout")){
+				countsOut=b;
+			}else if(a.equals("mincount0")){
+				minCount0=Long.parseLong(b);
+			}else if(a.equals("mincounta")){
+				minCountA=Long.parseLong(b);
+			}else if(a.equals("mincountr")){
+				minCountR=Long.parseLong(b);
+			}else if(a.equals("mapin")){
+				mapIn=b;
+			}else if(a.equals("mapout")){
+				mapOut=b;
+			}
+			
+			else if(a.equals("rc1") || a.equals("rcindex1")){
+				rcIndex1=Parse.parseBoolean(b);
+			}else if(a.equals("rc2") || a.equals("rcindex2")){
+				rcIndex2=Parse.parseBoolean(b);
+			}
+			
 			//Old version, allowed more complicated replacements
 //			else if(a.equals("replace")){
 //				if(b==null){
@@ -254,7 +297,12 @@ public class DemuxByName2 {
 //					}
 //				}
 //			}
-			else if(parser.parse(arg, a, b)){
+			
+			else if(PCRMatrix.parseStatic(arg, a, b)){
+				//Flag was captured by PCRMatrix; do nothing
+			}else if(BufferedMultiCros.parseStatic(arg, a, b)){
+				//Flag was captured by BufferedMultiCros; do nothing
+			}else if(parser.parse(arg, a, b)){
 				//Flag was captured by the parser; do nothing
 			}else if(parser.in1==null && i==0 && !arg.contains("=") && (arg.toLowerCase().startsWith("stdin") || new File(arg).exists())){
 				parser.in1=arg;
@@ -347,26 +395,64 @@ public class DemuxByName2 {
 	 */
 	private void handleModeAndNames(){
 		
+		//For length-1 delimiters, use a char instead of a String
+		assert(column<0 || delimiter!=null) : "Column may not be set if there is no delimiter.";
+		if(delimiter!=null){
+			if(delimiter.length()==1){
+				delimiterChar=delimiter.charAt(0);
+			}else if(delimiter.length()==2 && delimiter.charAt(0)=='\\'){
+				delimiterChar=delimiter.charAt(1);
+			}else{
+				delimiterChar=0;
+			}
+			delimiterPattern=Pattern.compile(delimiter);
+		}
+		
+		int barcodeLength=0;
+		
 		//Handle names
-		if(names.isEmpty()){
-			names=null;
+		if(mapIn!=null) {
+			assignmentMap=BarcodeCounter.loadAssignmentMap(mapIn);
+		}else if(assignmentMap.isEmpty()){
+			assignmentMap=null;
 		}else{
 			{//Process names, because they can either be literals or filenames at this point
-				String[] keys=names.keySet().toArray(new String[names.size()]);
-				names.clear();
-				for(String key : keys){//Process either literal names or files of names
-					File f=new File(key);
-					if(f.exists() && f.isFile()){//Read the file
-						TextFile tf=new TextFile(key);
-						String[] lines=tf.toStringLines();
-						for(String line : lines){names.put(line, line);}//Add each line as a name
-					}else{names.put(key, key);}//Re-add each key as a literal name
-				}
+				LinkedHashSet<String> set=new LinkedHashSet<String>(assignmentMap.keySet());
+				assignmentMap.clear();
+				set=BarcodeStats.loadBarcodeSet(set, (byte)barcodeDelimiter, rcIndex1, rcIndex2);
+				for(String key : set){assignmentMap.put(key, key);}
 			}
 		}
 		
+		if(useMatrix) {
+			pcrMatrix=PCRMatrix.create(barcodeLength1, barcodeLength2, barcodeDelimiter);
+			assert(!assignmentMap.isEmpty());
+			assert(mapIn==null);
+			pcrMatrix.populateExpected(assignmentMap.keySet());
+			Timer t=new Timer();
+			final Collection<Barcode> counts;
+			
+			System.err.print("Counting Barcodes:\t");
+			if(countsIn!=null) {counts=BarcodeCounter.loadCounts(countsIn, minCount0);}
+			else {
+				HashMap<String, Barcode> codeMap=BarcodeCounter.countBarcodes(ffin1, maxReads, PCRMatrix.byTile);
+				counts=codeMap.values();
+				if(countsOut!=null) {BarcodeCounter.writeCounts(counts, minCount0, countsOut, true, overwrite, append);}
+			}
+			t.stop("");
+			t.start();
+			
+			pcrMatrix.populateSplitCodes();
+			pcrMatrix.initializeData();
+			pcrMatrix.refine(counts, minCountR);
+//			pcrMatrix.verbose=true;
+			assignmentMap=pcrMatrix.makeAssignmentMap(counts, minCountA);
+			pcrMatrix=null;
+			t.stop("Assignment Time:\t\t");
+		}
+		
 		//Handle affix lengths
-		if(mode==AFFIX_MODE && names!=null){
+		if(mode==AFFIX_MODE && assignmentMap!=null){
 			
 			{//Find the lengths of all names to fill the lengthArray array
 				BitSet bs=new BitSet();//A set of observed affix lengths
@@ -375,7 +461,7 @@ public class DemuxByName2 {
 				if(fixedLength>0){bs.set(fixedLength);}
 				
 				//Add the lengths of everything in the names set
-				for(String s : names.keySet()){bs.set(s.length());}
+				for(String s : assignmentMap.keySet()){bs.set(s.length());}
 				
 				//Put the set members into an array for faster access
 				lengthArray=new int[bs.cardinality()];
@@ -403,45 +489,44 @@ public class DemuxByName2 {
 			}
 		}
 		
-		//For length-1 delimiters, use a char instead of a String
-		assert(column<0 || delimiter!=null) : "Column may not be set if there is no delimiter.";
-		if(delimiter!=null){
-			if(delimiter.length()==1){
-				delimiterChar=delimiter.charAt(0);
-			}else if(delimiter.length()==2 && delimiter.charAt(0)=='\\'){
-				delimiterChar=delimiter.charAt(1);
-			}else{
-				delimiterChar=0;
-			}
-			delimiterPattern=Pattern.compile(delimiter);
-		}
-		
 		//Populate names table with mutants for hamming distance
-		if(hdist>0 && names.size()>0){
-			@SuppressWarnings("unused")
-			int numMutants=mutate(names, hdist, outstream);
+		if(hdist>0 && assignmentMap.size()>0 && !useMatrix){
+			boolean oldMode=false;
+			if(oldMode) {
+				@SuppressWarnings("unused")
+				int numMutants=mutate_old(assignmentMap, hdist, outstream);
+			}else if(hdistSum){
+				mutateUnified(assignmentMap, hdist, outstream);
+			}else {
+				HashMap<String, String>[] maps=mutateDual(assignmentMap, hdist, barcodeDelimiter, outstream);
+				if(maps!=null) {
+					leftNames=maps[0];
+					rightNames=maps[1];
+				}
+			}
 		}
 		
 		//Populate name list with keys from the name map
-		if(names==null){
+		if(assignmentMap==null && !useMatrix){
 			nameList=null;
 		}else{
-			for(Entry<String, String> e : names.entrySet()){
+			for(Entry<String, String> e : assignmentMap.entrySet()){
 				nameList.add(e.getKey());
 			}
 		}
 		
 		//Ensure there are names in substring mode
 		if(mode==SUBSTRING_MODE){
-			assert(nameList!=null && !nameList.isEmpty()) : "Empty names list is not allowed in substring mode.";
+			assert(nameList!=null && !nameList.isEmpty()) : 
+				"Empty names list is not allowed in substring mode.";
 		}
 		
 		//Perform symbol replacements in output filenames
-		if(remap!=null && names!=null && !names.isEmpty()){
+		if(remap!=null && assignmentMap!=null && !assignmentMap.isEmpty()){
 			for(String key : nameList){
-				String old=names.get(key);
+				String old=assignmentMap.get(key);
 				String value=Tools.remap(remap, old);
-				if(!value.equals(old)){names.put(key, value);}
+				if(!value.equals(old)){assignmentMap.put(key, value);}
 			}
 		}
 	}
@@ -475,6 +560,10 @@ public class DemuxByName2 {
 		
 		//Report statistics to file
 		if(stats!=null){printReport(mcros);}
+
+		if(printRetireTime) {
+			outstream.println("\n"+mcros.printRetireTime()+mcros.printCreateTime()+"\n");
+		}
 		
 		//Stop the timer
 		t.stop();
@@ -506,17 +595,9 @@ public class DemuxByName2 {
 		final BufferedMultiCros mcros;
 		if(out1!=null){
 			
-			if(mcrosType==2){//Slow, synchronous mcros type
-				mcros=new MultiCros2(out1, out2, overwrite, append, true, useSharedHeader, FileFormat.FASTQ, threaded);
-			}else if(mcrosType==3){//Faster, asynchronous type
-				mcros=new MultiCros3(out1, out2, overwrite, append, true, useSharedHeader, FileFormat.FASTQ, threaded, maxStreams);
-			}else{
-				throw new RuntimeException("Bad mcrosType: "+mcrosType);
-			}
+			mcros=BufferedMultiCros.make(out1, out2, overwrite, append, true, useSharedHeader, FileFormat.FASTQ);
 			
 			//Set mcros fields not available in constructor
-			mcros.readsPerBuffer=readsPerBuffer;
-			mcros.bytesPerBuffer=bytesPerBuffer;
 			mcros.minReadsToDump=minReadsToDump;
 			mcros.trackCardinality=trackCardinality;
 			
@@ -526,7 +607,7 @@ public class DemuxByName2 {
 			}
 			
 			//Start the mcros thread if necessary
-			if(threaded){mcros.start();}
+			if(mcros.threaded){mcros.start();}
 		}else{
 			mcros=null;
 		}
@@ -650,12 +731,13 @@ public class DemuxByName2 {
 		//Report statistics to screen
 		double rpnano=readsProcessed/(double)(t.elapsed);
 		double bpnano=basesProcessed/(double)(t.elapsed);
-
+		
 		outstream.println("Time:               "+t);
 		outstream.println("Reads Processed:    "+readsProcessed+" \t"+String.format(Locale.ROOT, "%.2fk reads/sec", rpnano*1000000));
 		outstream.println("Bases Processed:    "+basesProcessed+" \t"+String.format(Locale.ROOT, "%.2fm bases/sec", bpnano*1000));
 		outstream.println("Reads Out:          "+readsOut);
 		outstream.println("Bases Out:          "+basesOut);
+		outstream.println("Yield:              "+String.format("%.5f", readsOut*1.0/readsProcessed));
 	}
 	
 	/*--------------------------------------------------------------*/
@@ -669,7 +751,7 @@ public class DemuxByName2 {
 	 * @param outstream Report results here
 	 * @return Number of mutants added (net)
 	 */
-	private static int mutate(HashMap<String, String> names, int hdist, PrintStream outstream){
+	private static int mutate_old(HashMap<String, String> names, int hdist, PrintStream outstream){
 		
 		//Put the names in a list for iteration, to prevent concurrent modification of the table
 		ArrayList<String> list=new ArrayList<String>(names.size());
@@ -683,7 +765,7 @@ public class DemuxByName2 {
 		
 		//Generate the mutants
 		for(String name : list){
-			mutants+=addMutants(name.getBytes(), name, hdist, names, collisions);
+			mutants+=addMutants_old(name.getBytes(), name, hdist, names, collisions);
 		}
 		outstream.println("Added "+mutants+" mutants to the initial "+list.size()+" names.");
 		
@@ -712,7 +794,7 @@ public class DemuxByName2 {
 	 * @param collisions Set of keys mapping to multiple values
 	 * @return Number of mutants added
 	 */
-	private static int addMutants(final byte[] keyArray, final String value, final int hdist, final HashMap<String, String> names, final HashSet<String> collisions){
+	private static int addMutants_old(final byte[] keyArray, final String value, final int hdist, final HashMap<String, String> names, final HashSet<String> collisions){
 		assert(hdist>0);
 		int added=0;
 		
@@ -738,7 +820,7 @@ public class DemuxByName2 {
 						}
 						if(hdist>1){
 							//Recur on the mutant, with a reduced Hamming distance
-							added+=addMutants(keyArray, value, hdist-1, names, collisions);
+							added+=addMutants_old(keyArray, value, hdist-1, names, collisions);
 						}
 					}
 				}
@@ -749,6 +831,121 @@ public class DemuxByName2 {
 		return added;
 	}
 	
+	//Easy mode for single barcodes or shared hdist
+	private static void mutateUnified(final HashMap<String, String> input, final int maxHDist, PrintStream outstream) {
+		if(maxHDist<1) {return;}
+		for(int hdist=1; hdist<=maxHDist; hdist++) {
+			addMutants(input);
+		}
+	}
+	
+	//TODO: Seems to map mutants to themselves instead of to their targets
+	//Oh...  this is just wrong.  The key pairs need to stay associated.  Only one pair can be mutated at a time.
+	@SuppressWarnings("unchecked")
+	private static HashMap<String, String>[] mutateDual(final HashMap<String, String> input, final int maxHDist, final int delimiter, PrintStream outstream) {
+		assert(maxHDist>0);
+		
+		HashMap<String, String> left=new HashMap<String, String>(input.size()*2);
+		HashMap<String, String> right=new HashMap<String, String>(input.size()*2);
+		for(Entry<String, String> e : input.entrySet()) {
+			String key=e.getKey();
+			String value=e.getValue();
+			int pos=key.indexOf(delimiter);
+			assert(pos>0) : "Can't find delimiter '"+Character.toString(delimiter)+"' in "+key;
+			final String k1=key.substring(0, pos);
+			final String k2=key.substring(pos+1);
+			final String v1, v2;
+			if(key.equals(value)) {
+				v1=k1;
+				v2=k2;
+			}else {
+				v1=value.substring(0, pos);
+				v2=value.substring(pos+1);
+			}
+			//Here
+			left.put(k1, v1);
+			right.put(k2,  v2);
+		}
+		for(int hdist=1; hdist<=maxHDist; hdist++) {
+			addMutants(left);
+			addMutants(right);
+		}
+		long total=left.size()*(long)(right.size());
+		
+		assert(false) : left+"\n"+right;
+		
+		if(total>1000000) {//For large sets return the twin sets
+			return new HashMap[] {left, right};
+		}
+		
+		//For small sets use cross product
+		input.clear();
+		final ByteBuilder bbk=new ByteBuilder(), bbv=new ByteBuilder();
+		for(Entry<String, String> e1 : left.entrySet()) {
+			final String k1=e1.getKey(), v1=e1.getValue();
+			bbk.clear().append(k1).append((char)delimiter);
+			bbv.clear().append(v1).append((char)delimiter);
+			final int len=bbk.length();
+			for(Entry<String, String> e2 : right.entrySet()) {
+				final String k2=e2.getKey(), v2=e2.getValue();
+				final String key=bbk.setLength(len).append(k2).toString();
+				final String value=bbv.setLength(len).append(v2).toString();
+				String old=input.put(key, value);
+				assert(old==null);
+			}
+		}
+		return null;
+	}
+	
+	private static void addMutants(final HashMap<String, String> input) {
+		final HashMap<String, String> output=new HashMap<String, String>();
+		for(Entry<String, String> e : input.entrySet()) {
+			String k=e.getKey();
+			String v=e.getValue();
+			addMutants2(k, v, output);
+		}
+		for(Entry<String, String> e : output.entrySet()) {
+			String k=e.getKey();
+			String v=e.getValue();
+			if(v.length()>0 && !input.containsKey(k)) {
+				input.put(k, v);
+			}
+		}
+	}
+	
+	private static int addMutants2(final String k0, final String v0, final HashMap<String, String> map) {
+		byte[] bases=k0.getBytes();
+		int added=0, removed=0, collisions=0;
+		//For each position in the array, change it to all possible values
+		for(int i=0; i<bases.length; i++){
+			final byte old=bases[i];
+			if(AminoAcid.isACGTN(old)){//Only mutate bases
+				for(byte b : symbols){
+					if(b!=old){
+						bases[i]=b;
+						String key=new String(bases);
+						String oldValue=map.get(key);
+						if(oldValue==null){//Add to map
+							map.put(key, v0);
+							added++;
+						}else if(oldValue==empty) {
+							collisions++;
+						}else if(oldValue.equals(v0)) {
+							//do nothing
+						}else {//Remove value
+							map.put(key, empty);
+							removed++;
+							collisions++;
+						}
+					}
+				}
+				//Restore the original value before moving to the next position
+				bases[i]=old;
+			}
+		}
+		return added;
+	}
+	
 	/** Returns the value this read maps to; meaning, the variable part of the output filename */
 	private String getValue(Read r){
 		final String key=getKey(r);
@@ -756,9 +953,19 @@ public class DemuxByName2 {
 		if(key==null){return null;}
 		
 		//Return the value this key maps to
-		if(names!=null){
-			if(verbose){System.err.println("Got value "+names.get(key));}
-			return names.get(key);
+		if(assignmentMap!=null){
+			String name=assignmentMap.get(key);
+			if(name==null && leftNames!=null) {//This branch does not occur in matrix mode
+				int idx=key.indexOf(barcodeDelimiter);
+				assert(idx>=0);
+				String k1=key.substring(0, idx), k2=key.substring(idx+1);
+				String v1=leftNames.get(k1), v2=rightNames.get(k2);
+				if(v1!=null && v2!=null) {
+					name=v1+Character.toString(barcodeDelimiter)+v2;
+				}
+			}
+			if(verbose){System.err.println("Got value "+assignmentMap.get(key));}
+			return name;
 		}
 		
 		//Return the value generated by remapping the key's symbols, if needed
@@ -767,6 +974,34 @@ public class DemuxByName2 {
 		return value;
 	}
 	
+
+	private int hdist(String q, String r) {
+		return hdistSum ? Barcode.hdist(q, r) : Tools.max(Barcode.hdistL(q, r), Barcode.hdistR(q, r));
+	}
+	
+	//TODO: Populate expectedList; currently this is unused.
+	//TODO: Decide to populate based on number of expected mutants.
+	public String findClosest(String q, int maxHDist, int clearzone) {
+		assert(!expectedList.isEmpty());
+		
+		int hdist=q.length();
+		int hdist2=hdist;
+		String best=null;
+		for(String b : expectedList) {
+			final int d=hdist(q, b);
+			if(d<hdist2) {
+				hdist2=d;
+				if(d<hdist) {
+					hdist2=hdist;
+					best=b;
+					hdist=d;
+				}
+			}
+		}
+		if(hdist>maxHDist || hdist+clearzone>hdist2) {return null;}
+		return best;
+	}
+	
 	/** Generates a key from the read header */
 	private String getKey(Read r){
 		final String id=r.id;
@@ -777,7 +1012,17 @@ public class DemuxByName2 {
 		if(mode==HEADER_MODE){return id;}
 		
 		//Parse the barcode
-		if(mode==BARCODE_MODE){return r.barcode(true);}
+		if(mode==BARCODE_MODE){
+			ihp.parse(id);
+			String bc=ihp.barcode();
+			return (useMatrix && PCRMatrix.byTile) ? bc+ihp.tile() : bc;
+		}
+		
+		//Parse the tile
+		if(mode==TILE_MODE){
+			ihp.parse(id);
+			return Integer.toString(ihp.tile());//This could be faster with an ArrayList<String> cache
+		}
 		
 		//Use mapping information
 		if(mode==CHROM_MODE){return r.rnameS();}
@@ -790,7 +1035,7 @@ public class DemuxByName2 {
 			}else{//Make a substring of each possible length and look it up in the hashmap
 				for(int affixLen : lengthArray){
 					final String sub=idlen>=affixLen ? prefixMode ? id.substring(0, affixLen) : id.substring(idlen-affixLen) : id;
-					if(names.containsKey(sub)){return sub;}
+					if(assignmentMap.containsKey(sub)){return sub;}
 				}
 				return null;
 			}
@@ -864,7 +1109,7 @@ public class DemuxByName2 {
 			bb.append("#Name\tReads\tBases"+(trackCardinality ? "\tCardinality" : "")+"\n");
 			
 			//Print results of unmatched reads, which is not captured by the mcros
-			if(names!=null && !names.isEmpty()){
+			if(assignmentMap!=null && !assignmentMap.isEmpty()){
 				bb.append("Unmatched\t").append(readsUnmatched).tab().append(basesUnmatched).nl();
 			}
 			bsw.print(bb);
@@ -913,6 +1158,23 @@ public class DemuxByName2 {
 	/** File extension override for output files */
 	private String extout=null;
 	
+	/** Pre-counted barcodes */
+	private String countsIn=null;
+	/** Output for barcode counts */
+	private String countsOut=null;
+	
+	/** 
+	 * Ignore barcodes occuring fewer times than this.
+	 * Saves memory and speeds processing. */
+	private long minCount0=0;
+	private long minCountR=4;
+	private long minCountA=4;
+	
+	/** Premade assignment map */
+	private String mapIn=null;
+	/** Assignment map output */
+	private String mapOut=null;
+	
 	/*--------------------------------------------------------------*/
 	
 	/** Primary input file */
@@ -981,12 +1243,8 @@ public class DemuxByName2 {
 	
 	/** Prevents issuing warnings multiple times */
 	private boolean warned=false;
-
-	/** Mcros buffers will be dumped after accumulating this many reads */
-	private int readsPerBuffer=1600;
-
-	/** Mcros buffers will be dumped after accumulating this many reads */
-	private int bytesPerBuffer=3000000;
+	
+	private boolean printRetireTime=false;
 	
 	/** Hamming distance for read indexes */
 	private int hdist=0;
@@ -1010,29 +1268,35 @@ public class DemuxByName2 {
 	 * The keys and values will be identical unless a Hamming distance is used.
 	 * This is only filled if explicit names are provided.
 	 */
-	private HashMap<String, String> names=new HashMap<String, String>();
+	private HashMap<String, String> assignmentMap=new HashMap<String, String>();
+
+	private ArrayList<String> expectedList;
+	
+	/** These are for dual barcodes and normally null unless
+	 * the mutant table would grow too big. */
+	private HashMap<String, String> leftNames=null, rightNames=null;
 	
 	/** Names in list form, for substring matching */
 	private ArrayList<String> nameList=new ArrayList<String>();
 	
-	/** 
-	 * Select MultiCros2 or MultiCros3.
-	 * MultiCros2 only opens 1 stream at a time and does not do concurrent writes.
-	 * MultiCros3 maintains multiple streams and does concurrent writes even with only 1 stream.
-	 */
-	private int mcrosType=3;
-	
-	/** Max concurrent streams for MultiCros3.  Does not seem to speed things up. */
-	private int maxStreams=BufferedMultiCros.DEFAULT_MAX_STREAMS;
-	
-	/** 
-	 * This puts the mcros handling in a second thread.
-	 * Does not seem to speed things up.
-	 */
-	private boolean threaded=false;
-	
 	/** Whether input file interleaving was explicitly set */
 	private boolean setInterleaved=false;
+
+//	private IlluminaHeaderParser1 ihp1=new IlluminaHeaderParser1();
+	private IlluminaHeaderParser2 ihp=new IlluminaHeaderParser2();
+	
+	/*--------------------------------------------------------------*/
+	
+	private int barcodeDelimiter=0;
+	private int barcodeLength1=0;
+	private int barcodeLength2=0;
+	private boolean rcIndex1=false;
+	private boolean rcIndex2=false;
+	
+	private PCRMatrix pcrMatrix;
+	private boolean useMatrix=false;
+	private boolean hdistSum=true;
+	
 	
 	/*--------------------------------------------------------------*/
 	
@@ -1059,7 +1323,8 @@ public class DemuxByName2 {
 	/*--------------------------------------------------------------*/
 	
 	/** Operation mode for selection of key */
-	public static final int AFFIX_MODE=1, DELIMITER_MODE=2, BARCODE_MODE=3, SUBSTRING_MODE=4, HEADER_MODE=5, CHROM_MODE=6;
+	public static final int AFFIX_MODE=1, DELIMITER_MODE=2, BARCODE_MODE=3,
+			SUBSTRING_MODE=4, HEADER_MODE=5, CHROM_MODE=6, TILE_MODE=7;
 	
 	/** 
 	 * Symbols allowed to substitute for Hamming distance.
@@ -1067,6 +1332,8 @@ public class DemuxByName2 {
 	 */
 	private static final byte[] symbols={'A', 'C', 'G', 'T', 'N'};
 	
+	private static final String empty="";
+	
 	/** Verbose messages for debugging */
 	public static boolean verbose=false;
 	
