@@ -57,7 +57,7 @@ public class FilterByCoverage {
 		Shared.capBufferLen(20);
 		Shared.capBuffers(4);
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		
 		
 		Parser parser=new Parser();
