@@ -73,7 +73,7 @@ public class FilterBySequence {
 		
 		//Set shared static variables
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 //		FASTQ.FORCE_INTERLEAVED=false;
 //		FASTQ.TEST_INTERLEAVED=false;
 		
