@@ -4,6 +4,8 @@ import java.io.File;
 import java.util.ArrayList;
 import java.util.Locale;
 import java.util.concurrent.atomic.AtomicIntegerArray;
+import java.util.concurrent.locks.ReadWriteLock;
+import java.util.concurrent.locks.ReentrantReadWriteLock;
 
 import aligner.Aligner;
 import aligner.MultiStateAligner9PacBioAdapter2;
@@ -621,6 +623,10 @@ public class FindPrimers implements Accumulator<FindPrimers.ProcessThread> {
 	private ArrayList<Read> queries;
 	private final int maxqlen;
 	
+	@Override
+	public final ReadWriteLock rwlock() {return rwlock;}
+	private final ReadWriteLock rwlock=new ReentrantReadWriteLock();
+	
 	/*--------------------------------------------------------------*/
 	
 	protected long readsProcessed=0;
