@@ -72,7 +72,7 @@ public class FungalRelease {
 		// Set shared static variables
 		Shared.capBuffers(4);
 		ReadWrite.USE_PIGZ = ReadWrite.USE_UNPIGZ = true;
-		ReadWrite.MAX_ZIP_THREADS = Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 
 		Read.TO_UPPER_CASE = true;
 
