@@ -65,7 +65,7 @@ public class GatherKapaStats {
 		
 		//Set shared static variables prior to parsing
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		
 		{//Parse the arguments
 			final Parser parser=parse(args);
@@ -701,7 +701,7 @@ public class GatherKapaStats {
 	private String in1=null;
 	private String out1=null;
 	
-	private String addressPrefix="https://rqc.jgi-psf.org/api/plate_ui/page/";
+	private String addressPrefix="https://rqc.jgi.doe.gov/api/plate_ui/page/";
 	private String addressSuffix="/kapa spikein";//"/kapa spikein";
 	
 	private boolean printRaw=false;
