@@ -70,7 +70,7 @@ public class KExpand {
 		
 		//Set shared static variables prior to parsing
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		Shared.capBuffers(4); //Only for singlethreaded programs
 		
 		{//Parse the arguments
