@@ -328,13 +328,14 @@ public class KmerCoverage {
 		KCountArray prefilterArray=null;
 		outstream.println();
 		ReadCounter rc=new ReadCounter(k, true, false, false, Shared.AMINO_IN);
+		rc.detectStepsize=buildStepsize;
 		if(prefilter){
 			prefilterArray=rc.makeKca(in1, in2, extra, 2, precells, prehashes, minq,
-					tablereads, 1, buildStepsize, 1, 1, null, 0);
+					tablereads, 1, 1, 1, null, 0);
 			outstream.println("Made prefilter:   \t"+prefilterArray.toShortString(prehashes));
 		}
 		kca=rc.makeKca(in1, in2, extra, cbits, cells, hashes, minq,
-				tablereads, buildpasses, buildStepsize, 2, 2, prefilterArray, (prefilterArray==null ? 0 : prefilterArray.maxValue));
+				tablereads, buildpasses, 2, 2, prefilterArray, (prefilterArray==null ? 0 : prefilterArray.maxValue));
 		ht.stop();
 		
 		outstream.println("Made hash table:  \t"+kca.toShortString(hashes));
