@@ -65,7 +65,7 @@ public class PartitionReads {
 		Shared.setBufferLen(Tools.max(400, Shared.bufferLen()));
 		Shared.capBuffers(2);
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		SamLine.SET_FROM_OK=true;
 		
 		//Create a parser object
