@@ -45,7 +45,7 @@ public class RandomGenome {
 		}
 		
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		
 		Parser parser=new Parser();
 		for(int i=0; i<args.length; i++){
