@@ -20,7 +20,7 @@ import fileIO.FileFormat;
 import fileIO.ReadWrite;
 import shared.KillSwitch;
 import shared.LineParser2;
-import shared.LineParser2S;
+import shared.LineParserS2;
 import shared.MetadataWriter;
 import shared.Parse;
 import shared.Parser;
@@ -73,7 +73,7 @@ public class ReformatReads {
 		
 		Shared.capBuffers(4);
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		
 		
 		SamLine.SET_FROM_OK=true;
@@ -926,12 +926,12 @@ public class ReformatReads {
 						Quantizer.quantize(quals2);
 					}
 					
-					if(forceTrimLeft>0 || forceTrimRight>0 || forceTrimModulo>0 || forceTrimRight2>0){
+					if(forceTrimLeft>0 || forceTrimRight>=0 || forceTrimModulo>0 || forceTrimRight2>0){
 						if(r1!=null && !r1.discarded()){
 							final int len=r1.length();
 							final int a=forceTrimLeft>0 ? forceTrimLeft : 0;
 							final int b0=forceTrimModulo>0 ? len-1-len%forceTrimModulo : len;
-							final int b1=forceTrimRight>0 ? forceTrimRight : len;
+							final int b1=forceTrimRight>=0 ? forceTrimRight : len;
 							final int b2=forceTrimRight2>0 ? len-1-forceTrimRight2 : len;
 							final int b=Tools.min(b0, b1, b2);
 							final int x=TrimRead.trimToPosition(r1, a, b, 1);
