@@ -62,11 +62,12 @@ public class Shred {
 		Shared.capBufferLen(100);
 		Shared.capBuffers(4);
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		
 		long seed=-1;
 		Parser parser=new Parser();
 		parser.minReadLength=parser.maxReadLength=-1;
+		int increment_=0;
 		boolean even=false;
 		for(int i=0; i<args.length; i++){
 			String arg=args[i];
@@ -78,6 +79,8 @@ public class Shred {
 				shredLength=Parse.parseIntKMG(b);
 			}else if(a.equals("overlap")){
 				overlap=Parse.parseIntKMG(b);
+			}else if(a.equals("increment")){
+				increment_=Parse.parseIntKMG(b);
 			}else if(a.equals("verbose")){
 				verbose=Parse.parseBoolean(b);
 			}else if(a.equals("even") || a.equals("equal")){
@@ -92,6 +95,7 @@ public class Shred {
 				//Set a variable here
 			}else if(parser.parse(arg, a, b)){
 				//do nothing
+				
 			}else{
 				outstream.println("Unknown parameter "+args[i]);
 				assert(false) : "Unknown parameter "+args[i];
@@ -117,14 +121,21 @@ public class Shred {
 
 			minLength=parser.minReadLength;
 			maxLength=parser.maxReadLength;
+			maxNs=parser.maxNs;
 		}
 		
 		minLength=Tools.mid(1, minLength, shredLength);
-		assert(shredLength>0);
-		assert(shredLength>overlap);
-		increment=shredLength-overlap;
-		incMult=1.0/increment;
+		if(increment_>0) {
+			assert(shredLength>0);
+			increment=increment_;
+			overlap=(maxLength<0 ? shredLength : minLength)-increment;
+		} else {
+			assert(shredLength>0);
+			assert(shredLength>overlap);
+			increment=shredLength-overlap;
+		}
 		assert(increment>0);
+		incMult=1.0/increment;
 		
 		assert(FastaReadInputStream.settingsOK());
 		
@@ -306,9 +317,13 @@ public class Shred {
 			final byte[] bases2=KillSwitch.copyOfRange(bases, a, b);
 			final byte[] quals2=(quals==null ? null : KillSwitch.copyOfRange(quals, a, b));
 			Read shred=new Read(bases2, quals2, name+"_"+a+"-"+(b-1), readsOut);
-			readsOut++;
-			basesOut+=shred.length();
-			list.add(shred);
+			boolean pass=true;
+			if(pass && maxNs>=0 && shred.countUndefined()>maxNs) {pass=false;}
+			if(pass) {
+				readsOut++;
+				basesOut+=shred.length();
+				list.add(shred);
+			}
 		}
 	}
 	
@@ -323,9 +338,13 @@ public class Shred {
 			final byte[] bases2=KillSwitch.copyOfRange(bases, i, limit);
 			final byte[] quals2=(quals==null ? null : KillSwitch.copyOfRange(quals, i, limit));
 			Read shred=new Read(bases2, quals2, name+"_"+i+"-"+(limit-1), readsOut, r1.flags);
-			readsOut++;
-			basesOut+=shred.length();
-			list.add(shred);
+			boolean pass=true;
+			if(pass && maxNs>=0 && shred.countUndefined()>maxNs) {pass=false;}
+			if(pass) {
+				readsOut++;
+				basesOut+=shred.length();
+				list.add(shred);
+			}
 			if(limit==bases.length){return;}
 			assert(limit<bases.length);
 		}
@@ -379,6 +398,7 @@ public class Shred {
 	private int shredLength=500;
 	private int minLength=-1;
 	private int maxLength=-1;
+	private int maxNs=-1;
 	private final int range;
 	private int overlap=0;
 	private final int increment;
