@@ -181,12 +181,12 @@ public abstract class AbstractKmerTableSet {
 		if(onePass){
 			assert(filter==null || filter.length==1) : "Multiple filtering passes are not allowed in onepass mode.\n"+filter.length+","+prepasses+", "+onePass+", "+prefilter;
 			filter[0]=rc.makeKca(null, null, null, cbits, precells, prehashes, minq,
-					maxReads, 1, 1, 1, 1, null, 0);
+					maxReads, 1, 1, 1, null, 0);
 		}else{
 			if(ht==null){ht=new Timer();}
 			ht.start();
 			filter[0]=rc.makeKca_als(in1, in2, extra, cbits, precells, prehashes, minq,
-					maxReads, 1, 1, 1, 1, filter[0], filterMax);
+					maxReads, 1, 1, 1, filter[0], filterMax);
 			assert(filterMax<filter[0].maxValue || (currentPass>0 && currentPass==prepasses-1));
 			outstream.println("Made prefilter:   \t"+filter[0].toShortString(prehashes));
 			double uf=filter[0].usedFraction();
