@@ -1,6 +1,5 @@
 package ml;
 
-import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Random;
 
@@ -13,60 +12,45 @@ public class Cell extends Source {
 	/*----------------        Initialization        ----------------*/
 	/*--------------------------------------------------------------*/
 	
-	public Cell(int id_, int activationType, int lpos_, int layer_, int pWid, int wid, int nWid,
+	public Cell(int id_, int activationType, int lpos_, int layer_, int maxLayer_, 
+			int prevLayerStart_, int nextLayerStart_, int wid,
 			float[] values_, float[] eOverNetArray_) {
 		id=id_;
 //		type=activationType;
 		function=Function.getFunction(activationType);
 		lpos=lpos_;
 		layer=layer_;
-		inputs=(pWid<1 || CellNet.FAST ? null : new ArrayList<Edge>(pWid));
-		outputs=(nWid<1 || CellNet.FAST ? null : new ArrayList<Edge>(nWid));
+		maxLayer=maxLayer_;
 		values=values_;
 		eOverNetArray=eOverNetArray_;
 		assert(values.length==wid);
-		weights=(pWid<1 ? null : new float[pWid]);
-//		weights2=(pWid<1 ? null : new double[pWid]);
-		deltas=(pWid<1 ? null : new float[pWid]);
+		prevLayerStart=prevLayerStart_;
+		nextLayerStart=nextLayerStart_;
+		
+		//Initialize later
+//		inputs=in;
+//		outputs=out;
+//		weights=(inputs==null || inputs.length<1 ? null : new float[inputs.length]);
+//		deltas=(inputs==null || inputs.length<1 ? null : new float[inputs.length]);
+		
 		assert(lpos>=0 & lpos<wid);
 	}
 	
 	/*--------------------------------------------------------------*/
 	/*----------------         Non-Mutators         ----------------*/
 	/*--------------------------------------------------------------*/
-
-	public double summateSlow() {
-		sum=bias;
-//		CellNet.println("sum="+sum);
-		for(Edge e : inputs) {
-			sum+=e.source.value()*e.weight();
-//			CellNet.println("sum="+sum);
-		}
-//		CellNet.println("sig="+activation(sum));
-//		CellNet.println("sig -1="+activation(-1));
-//		CellNet.println("sig 1="+activation(1));
-//		assert(bias==0) : bias+", "+sum+", "+this;
-		return sum;
-	}
 	
-//	public void summateFast(float[] valuesIn) {
-//		if(CellNet.SIMD && valuesIn.length>=16) {
-//			summateFast_vec(valuesIn);
-//			return;
-//		}
-//		sum=bias;
-//		assert(valuesIn.length==weights.length) : valuesIn.length+", "+weights.length;
-//		for(int i=0; i<valuesIn.length; i++) {//TODO: Should this run from 1 to length?
-//			sum+=valuesIn[i]*weights[i];
-//		}
-//		final float v=(float)activation(sum);
-//		setValue(v);
-//	}
-	
-	public void summateFast(float[] valuesIn) {
+	public void summateDense(float[] valuesIn) {
 		sum=bias;
 		assert(valuesIn.length==weights.length) : valuesIn.length+", "+weights.length;
-		sum+=Vector.fma(valuesIn, weights);
+		sum+=Vector.fma(weights, valuesIn);
+		final float v=(float)activation(sum);
+		setValue(v);
+	}
+	
+	public void summateSparse(float[] valuesIn, int edgeBlockSize) {
+		sum=bias;
+		sum+=Vector.fma(weights, valuesIn, inputs, edgeBlockSize, Vector.SIMD_FMA_SPARSE);
 		final float v=(float)activation(sum);
 		setValue(v);
 	}
@@ -77,7 +61,13 @@ public class Cell extends Source {
 	}
 	
 	synchronized public boolean check() {
-		assert(false);
+//		assert(false);
+		
+		if(!CellNet.DENSE) {
+			assert(layer==maxLayer || outputs!=null) :
+				layer+", "+maxLayer+", "+lpos+", "+id();
+		}
+		
 		if(value!=values[lpos] && (value!=-1 && values[lpos]!=0)) {
 			assert(false) : id+", "+layer+", "+lpos+": "+value+", "+values[lpos]+", "+Arrays.toString(values);
 			return false;
@@ -88,25 +78,9 @@ public class Cell extends Source {
 		}
 		if(layer>0) {
 			assert(weights!=null);
-			assert(inputs!=null);
-			assert(weights.length==inputs.size()) : weights.length+", "+inputs.size();
+			assert(CellNet.DENSE==(inputs==null));
 //			assert(weights2.length==weights.length);
-			assert(deltas.length==weights.length);
-			for(int i=0; i<weights.length; i++) {
-				Edge e=inputs.get(i);
-				if(e.weight()!=weights[i]) {
-					assert(false) : id+", "+layer+", "+lpos+": "+e.weight()+", "+weights[i];
-					return false;
-				}
-//				if(e.weight2!=weights2[i]) {
-//					assert(false) : id+", "+layer+", "+lpos+": "+e.weight2+", "+weights2[i];
-//					return false;
-//				}
-				if(e.delta!=deltas[i]) {
-					assert(false) : id+", "+layer+", "+lpos+": "+e.delta+", "+deltas[i];
-					return false;
-				}
-			}
+			assert(deltas==null || deltas.length==weights.length);
 //			for(int i=0; i<weights2.length; i++) {
 //				Edge e=outputs.get(i);
 //				if(e.weight2!=weights2[i]) {
@@ -116,7 +90,7 @@ public class Cell extends Source {
 //			}
 		}else {
 //			assert(inputs==null || outputs.isEmpty())  : id+", "+layer+", "+lpos+": "+weights2+": "+outputs+"\n"+toString()+"\n";
-			assert(inputs==null || outputs.isEmpty())  : id+", "+layer+", "+lpos+": "+deltas+": "+outputs+"\n"+toString()+"\n";
+			assert(inputs==null || outputs==null)  : id+", "+layer+", "+lpos+": "+deltas+": "+outputs+"\n"+toString()+"\n";
 		}
 		return true;
 	}
@@ -125,45 +99,26 @@ public class Cell extends Source {
 	/*----------------           Mutators           ----------------*/
 	/*--------------------------------------------------------------*/
 	
-	void applyUpdatesSlow(float invSamples, float alpha) {
-		if(layer<1) {return;}
-		adjustBias(invSamples, alpha);
-		for(int i=0; i<weights.length; i++) {
-			Edge e=inputs.get(i);
-			assert(e.weight()==weights[i]);
-//			assert(e.weight2==weights2[i]);
-			assert(e.delta==deltas[i]);
-//			weights[i]=e.adjustWeight(invSamples);
-//			final float w3=(float)(weights2[i]*invSamples);
-			final float d3=(float)(deltas[i]*invSamples*alpha);
-			final float w4=weights[i]+d3;
-//			weights2[i]=e.weight2=0;
-			e.delta=0;
-			deltas[i]=0;
-			weights[i]=w4;
-			e.setWeight(w4);
-		}
-//		for(Edge e : inputs) {
-//			e.adjustWeight(invSamples);
-//		}
-//		loadFast();
-	}
-	
-	void applyUpdatesFast(float invSamples, float alpha) {//only called once per epoch
+	void applyUpdates(float invSamples, float alpha) {//only called once per epoch
 		if(layer<1) {return;}
 		adjustBias(invSamples, alpha);
 		for(int i=0; i<weights.length; i++) {
 			final float w=weights[i];
 			
-//			final float w3=(float)(weights2[i]*invSamples);
-//			weights2[i]=0;
-			
 			final float d3=(float)(deltas[i]*invSamples*alpha);
-			final float w4=w+d3;
-//			assert(Tools.absdif(w3, w4)<0.00001) : "w3="+w3+", w4="+w4+", d3="+d3+", w="+weights[i]+", d"+deltas[i]+", w2="+weights2[i];
+			float w4=w+d3;
+			{
+				final float absW4=Math.abs(w4);
+				if(absW4>Math.abs(w) && absW4>edgeAmplitudeIncreaseThresh) {
+					//Very sensitive; strong values break convergence.
+					//0.98 seems OK though.  Goal is to keep edges low.
+					//.94 is too extreme.  .96 seems OK.
+					w4=w+edgeAmplitudeIncreaseMult*d3;//Slow down increase in edge magnitude
+				}
+			}
 			deltas[i]=0;
 
-			weights[i]=w4;
+			if((w!=0 || !CellNet.DENSE) && Math.abs(w4)>Float.MIN_NORMAL) {weights[i]=w4;}
 		}
 	}
 	
@@ -176,25 +131,13 @@ public class Cell extends Source {
 		error=0;
 	}
 	
-	public void clearTempSlow() {
-//		bias2=0;
-		biasDelta=0;
-		error=0;
-		if(layer<1) {return;}
-		for(int i=0; i<weights.length; i++) {
-			Edge e=inputs.get(i);
-			e.clearTemp();
-		}
-//		Arrays.fill(weights2, 0);
-		Arrays.fill(deltas, 0);
-	}
-	
-	public void clearTempFast() {
+	public void clearTemp() {
 //		bias2=0;
 		biasDelta=0;
 		error=0;
 		if(layer<1) {return;}
 //		Arrays.fill(weights2, 0);
+		assert(deltas!=null) : id()+", "+layer;
 		Arrays.fill(deltas, 0);
 	}
 	
@@ -246,42 +189,8 @@ public class Cell extends Source {
 	
 	/*--------------------------------------------------------------*/
 	
-	void updateEdgesFinalLayerSlow(float alpha, float target, float weightMult) {
-		assert(alpha>0) : alpha;
-		//assert(check());
-		final float v=value();
-		eTotalOverOut=calcETotalOverOut(v, target, weightMult);
-		outOverNet=(float)derivativeXFX(sum, v);
-		eOverNet=eOverNetArray[lpos]=eTotalOverOut*outOverNet;
-		
-		for(int i=0; i<weights.length; i++) {
-			final Edge e=inputs.get(i);
-//			assert(e.source.check());
-			float netOverWeight=e.source.value();
-			float eTotalOverWeight=eOverNet*netOverWeight;
-			
-//			double incr=e.weight()-alpha*eTotalOverWeight;
-//			e.incrementWeight2(incr);
-//			weights2[i]+=incr;
-//			assert(weights2[i]==e.weight2);
-			
-			float dincr=-eTotalOverWeight;
-			e.incrementDelta(dincr);
-			deltas[i]+=dincr;
-			assert(deltas[i]==e.delta);
-		}
-		{
-//			float product=(float) (eTotalOverOut*outOverNet);
-//			bias2+=bias-alpha*eOverNet*biasAlphaMult;
-			biasDelta-=eOverNet;
-//			CellNet.println("C"+id()+": bias="+bias+", bias2="+bias2);
-		}
-		//assert(check());
-	}
-	
 	//TODO: Vectorize?  Final layer is usually small though
-	void updateEdgesFinalLayerFast(float alpha, float target, float[] valuesIn, float weightMult) {
-		assert(alpha>0) : alpha;
+	void updateEdgesFinalLayerDense(float target, float[] valuesIn, float weightMult) {
 		//assert(check());
 		final float v=value();
 		eTotalOverOut=calcETotalOverOut(v, target, weightMult);
@@ -313,96 +222,7 @@ public class Cell extends Source {
 	
 	/*--------------------------------------------------------------*/
 	
-	void updateEdgesHiddenLayerSlow(float alpha) {
-		assert(alpha>0) : alpha;
-		//assert(check());
-		eTotalOverOut=0;
-		outOverNet=value()*(1-value());
-		
-		for(int i=0; i<outputs.size(); i++) {
-			final Edge o=outputs.get(i);
-//			assert(o.dest.check());
-//			double eOverNetDest=o.dest.eTotalOverOut*o.dest.outOverNet;
-//			assert(eOverNetDest==o.dest.eOverNet);
-			float eOverNetDest=o.dest.eOverNet;
-			float netOverOut=o.weight();
-			float eOverOut=eOverNetDest*netOverOut;
-			eTotalOverOut+=eOverOut;
-		}
-		eOverNet=eOverNetArray[lpos]=eTotalOverOut*outOverNet;
-		
-		for(int i=0; i<inputs.size(); i++) {
-			final Edge e=inputs.get(i);
-//			assert(e.source.check());
-			float netOverWeight=e.source.value();
-			float eTotalOverWeight=(eTotalOverOut*outOverNet*netOverWeight);
-//			e.incrementWeight2(e.weight()-alpha*eTotalOverWeight);
-			
-//			final double w=e.weight();
-//			final double incr=w-alpha*eTotalOverWeight;
-//			weights2[i]+=incr;
-//			e.incrementWeight2(incr);
-//			assert(weights2[i]==e.weight2);
-			
-			e.incrementDelta(-eTotalOverWeight);
-			assert(deltas[i]==e.delta);
-		}
-		if(layer>0){
-			float product=(float) (eTotalOverOut*outOverNet);
-//			bias2+=bias-alpha*product*biasAlphaMult;//Bias adjusts slower than edges
-			biasDelta-=product;
-		}
-		//assert(check());
-	}
-	
-	void updateEdgesHiddenLayerFast_double(float alpha, float[] valuesPrev, double[] eOverNetNext, float[] weightsOut) {
-		assert(alpha>0) : alpha;
-		//assert(check());
-		eTotalOverOut=0;
-		final float v=value();
-		assert(v==values[lpos]) : v+", "+values[lpos]+//Also fires on NaN, but that shouldn't happen...
-			"\n"+Arrays.toString(values)+
-			"\n"+Arrays.toString(weights)+"\n";
-		outOverNet=(float)derivativeXFX(sum, v);
-		
-		for(int dest=0; dest<eOverNetNext.length; dest++){
-//			Edge o=outputs.get(dest);
-//			final float netOverOut=o.weight();//Random access
-			
-			final float netOverOut=weightsOut[dest];
-			
-//			assert(o.dest.eTotalOverOut==eTotalOverOutNext[dest]);
-//			assert(o.dest.outOverNet==outOverNetNext[dest]);
-//			final double eOverNetDest=eTotalOverOutNext[dest]*outOverNetNext[dest];//TODO: Should be already calculated by dest
-//			assert(eOverNetDest==o.dest.eOverNet);
-			final double eOverNetDest=eOverNetNext[dest];
-//			assert(eOverNetDest==o.dest.eOverNet);
-			final double eOverOut=eOverNetDest*netOverOut;
-			eTotalOverOut+=eOverOut;
-		}
-		eOverNet=eOverNetArray[lpos]=eTotalOverOut*outOverNet;
-		
-//		final double eTotalOverOut_X_outOverNet=eTotalOverOut*outOverNet;
-		for(int source=0; source<valuesPrev.length; source++) {
-			final float netOverWeight=valuesPrev[source];
-			final double eTotalOverWeight=eOverNet*netOverWeight;
-			
-//			final double incr=weights[source]-alpha*eTotalOverWeight;
-//			weights2[source]+=incr;
-
-			deltas[source]-=eTotalOverWeight;
-		}
-//		if(layer>0){
-		{
-			assert(layer>0) : layer;
-//			bias2+=bias-alpha*eOverNet*biasAlphaMult;//Bias adjusts slower than edges
-			biasDelta-=eOverNet;
-		}
-		//assert(check());
-	}
-	
-	public void updateEdgesHiddenLayerFast(float alpha, float[] valuesPrev, float[] eOverNetNext, float[] weightsOut) {
-		assert(alpha>0) : alpha;
+	public void updateEdgesHiddenLayerDense(float[] valuesIn, float[] eOverNetNext, float[] weightsOut) {
 		//assert(check());
 //		eTotalOverOut=0;
 		final float v=value();
@@ -411,7 +231,10 @@ public class Cell extends Source {
 			"\n"+Arrays.toString(weights)+"\n";
 		outOverNet=(float)derivativeXFX(sum, v);
 
-		if(!CellNet.SPECIAL_FMA) {eTotalOverOut=Vector.fma(weightsOut, eOverNetNext);}
+		assert(CellNet.DENSE);
+		if(!CellNet.SPECIAL_FMA) {
+			eTotalOverOut=Vector.fma(weightsOut, eOverNetNext);
+		}
 		
 //		if(CellNet.SIMD && eOverNetNext.length>=16) {
 //			eTotalOverOut=shared.Vector.fma(weightsOut, eOverNetNext);
@@ -436,7 +259,54 @@ public class Cell extends Source {
 //
 //			deltas[source]-=eTotalOverWeight;
 //		}
-		Vector.addProduct(deltas, valuesPrev, -eOverNet);
+		Vector.addProduct(deltas, valuesIn, -eOverNet);
+		
+//		if(layer>0){
+		{
+			assert(layer>0) : layer;
+//			bias2+=bias-alpha*eOverNet*biasAlphaMult;//Bias adjusts slower than edges
+			biasDelta-=eOverNet;
+		}
+		//assert(check());
+	}
+	
+	public void updateEdgesHiddenLayerSparse(float[] valuesIn, float[] eOverNetNext,
+			float[] weightsOut, int edgeBlockSize) {
+		//assert(check());
+//		eTotalOverOut=0;
+		final float v=value();
+		assert(v==values[lpos]) : v+", "+values[lpos]+//Also fires on NaN, but that shouldn't happen...
+			"\n"+Arrays.toString(values)+
+			"\n"+Arrays.toString(weights)+"\n";
+		outOverNet=(float)derivativeXFX(sum, v);
+
+		assert(!CellNet.DENSE);
+		eTotalOverOut=Vector.fma(weightsOut, eOverNetNext, outputs, 1, false);
+		
+//		if(CellNet.SIMD && eOverNetNext.length>=16) {
+//			eTotalOverOut=shared.Vector.fma(weightsOut, eOverNetNext);
+//		}else {
+//			for(int dest=0; dest<eOverNetNext.length; dest++){
+//				final float netOverOut=weightsOut[dest];
+//				final float eOverNetDest=eOverNetNext[dest];
+//				final float eOverOut=eOverNetDest*netOverOut;
+//				
+//				eTotalOverOut+=eOverOut;
+//			}
+//		}
+		
+		eOverNetArray[lpos]=eOverNet=eTotalOverOut*outOverNet;
+		
+//		for(int source=0; source<valuesPrev.length; source++) {
+//			final float netOverWeight=valuesPrev[source];
+//			final float eTotalOverWeight=eOverNet*netOverWeight;
+//			
+////			final double incr=weights[source]-alpha*eTotalOverWeight;
+////			weights2[source]+=incr;
+//
+//			deltas[source]-=eTotalOverWeight;
+//		}
+		Vector.addProduct(deltas, valuesIn, inputs, -eOverNet, edgeBlockSize);
 		
 //		if(layer>0){
 		{
@@ -449,44 +319,7 @@ public class Cell extends Source {
 	
 	/*--------------------------------------------------------------*/
 	
-	public void accumulateSlow(Cell c2) {
-		//assert(check());
-		//assert(c2.check());
-//		System.err.println(">>>"+c2);
-//		System.err.println(">>>"+this);
-		error+=c2.error;
-//		bias2+=c2.bias2;
-		biasDelta+=c2.biasDelta;
-//		if(layer>0) {
-//			final double[] c2w2=c2.weights2;
-//			for(int i=0; i<weights2.length; i++) {
-//				Edge e=inputs.get(i);
-//				Edge e2=c2.inputs.get(i);
-//				assert(e.weight2==weights2[i]);
-//				assert(e.weight()==weights[i]);
-//				assert(e2.weight2==c2.weights2[i]) : e2.weight2+", "+c2.weights2[i];
-//				assert(e2.weight()==c2.weights[i]);
-//				e.accumulate(e2);
-//				weights2[i]+=c2w2[i];
-//			}
-//		}
-		if(layer>0) {
-			final float[] c2w2=c2.deltas;
-			for(int i=0; i<deltas.length; i++) {
-				Edge e=inputs.get(i);
-				Edge e2=c2.inputs.get(i);
-				assert(e.delta==deltas[i]);
-				assert(e.weight()==weights[i]);
-				assert(e2.delta==c2.deltas[i]) : e2.delta+", "+c2.deltas[i];
-				assert(e2.weight()==c2.weights[i]);
-				e.accumulate(e2);
-				deltas[i]+=c2w2[i];
-			}
-		}
-//		System.err.println(">>>"+this);
-	}
-	
-	public void accumulateFast(Cell c2) {
+	public void accumulate(Cell c2) {
 		//assert(check());
 		//assert(c2.check());
 		error+=c2.error;
@@ -509,32 +342,7 @@ public class Cell extends Source {
 	
 	/*--------------------------------------------------------------*/
 	
-	public void annealSlow(float strength, Random randy) {
-		if(layer<1){return;}
-		if(annealBias) {
-			final float abs=Math.abs(bias);
-			float xb=strength*(randy.nextFloat()-randy.nextFloat())*biasAnnealMult;
-			if(abs<0.2f) {xb=xb*(Tools.max(abs*5, 0.2f));} //Weaker anneal for weaker bias //TODO: Make lower limit even weaker
-			setBias(bias+xb, true);
-		}
-		for(int i=0; i<weights.length; i++) {
-			final float w=weights[i];
-			if(w!=0) {
-				final Edge e=inputs.get(i);
-				assert(w==e.weight());
-				final float abs=Math.abs(w);
-				float xe=strength*(randy.nextFloat()-randy.nextFloat());//*Tools.max(e.weight(), 0.01f);
-//				if(abs<0.2f) {xe=xe*(Tools.max(abs*5, 0.2f));} //Weaker anneal for weaker weight
-				if(abs<lowWeightAnnealCutoff) {xe=xe*(Tools.max(abs*lowWeightAnnealMult, lowWeightAnnealCutoff));} //Weaker anneal for weaker weight
-				final float w2=w+xe;
-				e.setWeight(w2);
-				weights[i]=w2;
-				assert(w2==e.weight());
-			}
-		}
-	}
-	
-	public void annealFast(float strength, Random randy) {
+	public void anneal(float strength, Random randy) {
 		if(layer<1){return;}
 		if(annealBias) {
 			final float abs=Math.abs(bias);
@@ -548,12 +356,12 @@ public class Cell extends Source {
 			if(w!=0) {
 				//			assert(w==e.weight());
 				final float abs=Math.abs(w);
-				float xe=strength*(randy.nextFloat()-randy.nextFloat());//*Tools.max(e.weight(), 0.01f);
-				//			if(abs<0.2f) {xe=xe*(Tools.max(abs*5, 0.2f));} //Weaker anneal for weaker weight
+				float xe=strength*(randy.nextFloat()-randy.nextFloat());
 				if(abs<lowWeightAnnealCutoff) {xe=xe*(Tools.max(abs*lowWeightAnnealMult, lowWeightAnnealCutoff));} //Weaker anneal for weaker weight
+				if(Math.abs(w+xe)>abs) {xe*=edgeAmplitudeIncreaseMult;} //Weaker anneal when it increases absolute magnitude of weight
 				final float w2=w+xe;
 				//			e.setWeight(w2);
-				weights[i]=w2;
+				if(Math.abs(w2)>Float.MIN_NORMAL) {weights[i]=w2;}
 				//			assert(w2==e.weight());
 			}
 		}
@@ -568,43 +376,39 @@ public class Cell extends Source {
 		value=c.value;
 		sum=c.sum;
 		error=c.error;
-		if(copyDelta) {
-//			bias2=c.bias2;
-			biasDelta=c.biasDelta;
-		}
-		if(layer==0){return;}
-		if(CellNet.FAST) {
+//		if(layer==0){return;}
+		if(weights==null) {
+			weights=(c.weights==null ? null : Arrays.copyOf(c.weights, c.weights.length));
+			assert(inputs==null);
+			inputs=(c.inputs==null ? null : Arrays.copyOf(c.inputs, c.inputs.length));
+		}else{
 			Vector.copy(weights, c.weights);
-//			for(int i=0; i<weights.length; i++) {
-//				weights[i]=c.weights[i];
-//			}
-			if(copyDelta) {
-//				for(int i=0; i<weights2.length; i++) {
-//					weights2[i]=c.weights2[i];
-//				}
-				Vector.copy(deltas, c.deltas);
-//				for(int i=0; i<deltas.length; i++) {
-//					deltas[i]=c.deltas[i];
-//				}
-			}
-		}else {
-			if(copyDelta) {
-//				for(int i=0; i<weights2.length; i++) {
-//					weights2[i]=c.weights2[i];
-//				}
-				for(int i=0; i<deltas.length; i++) {
-					deltas[i]=c.deltas[i];
-				}
-			}
-			for(int i=0; i<weights.length; i++) {
-				weights[i]=c.weights[i];
-				inputs.get(i).setFrom(c.inputs.get(i), copyDelta);
-			}
+			if(!CellNet.DENSE) {Vector.copy(inputs, c.inputs);}
 		}
+		
+		if(outputs==null) {
+			outputs=(c.outputs==null ? null : Arrays.copyOf(c.outputs, c.outputs.length));
+		}else{
+			Vector.copy(outputs, c.outputs);
+		}
+
+		biasDelta=0;
+		if(copyDelta) {
+			biasDelta=c.biasDelta;
+			deltas=(c.deltas==null ? null : Arrays.copyOf(c.deltas, c.deltas.length));
+		}else if(deltas==null){
+			deltas=(weights==null ? null : new float[weights.length]);//(c.deltas==null ? null : new float[c.deltas.length]);
+		}else {
+			Arrays.fill(deltas, 0);
+		}
+		assert(Tools.equals(inputs, c.inputs));
+		assert(Tools.equals(outputs, c.outputs));
+		assert(Tools.equals(weights, c.weights));
+//		assert(false) : outputs+", "+c.outputs;
+		
 		//assert(check());
 		//assert(c.check());
-		assert(CellNet.FAST || inputs.size()==0 || inputs.size()==c.inputs.size()) : "\n"+inputs+"\n"+c.inputs;
-		assert(c.weights.length==weights.length) : "\n"+inputs+"\n"+c.inputs;
+//		assert(c.weights.length==weights.length) : "\n"+inputs+"\n"+c.inputs;
 	}
 	
 	/*--------------------------------------------------------------*/
@@ -618,16 +422,17 @@ public class Cell extends Source {
 		sb.append("C"+id()+": v="+String.format("%.4f, b=%.4f, b2=%.4f, e=%.5f", value(), bias, biasDelta, error));
 		if(!terminal()) {
 			sb.append(", Edges: {");
-			if(CellNet.FAST) {
+			if(CellNet.DENSE) {
 				int prevBase=id-lpos-weights.length;
 				for(int i=0; i<weights.length; i++) {
 					//TODO: Could use source array or source value array here
 //					return String.format("C"+(prevBase+i)+"->C"+id+",%.3f,w?=%.4f,w2=%.4f,d=%.4f; ", -1, weights[i], weights2[i], deltas[i]);
-					return String.format("C"+(prevBase+i)+"->C"+id+",%.3f,w?=%.4f,d=%.4f; ", -1, weights[i], deltas[i]);
+					sb.append(String.format("C"+(prevBase+i)+"->C"+id+",%.3f,w?=%.4f,d=%.4f; ", -1, weights[i], deltas[i]));
 				}
 			}else {
-				for(Edge e : inputs) {
-					sb.append(e.toString());
+				for(int i=0; i<weights.length; i++) {
+					final int prev=inputs[i]+prevLayerStart;
+					sb.append(String.format("C"+(prev)+"->C"+id+",%.3f,w?=%.4f,d=%.4f; ", -1, weights[i], deltas[i]));
 				}
 			}
 			sb.setLength(sb.length()-2);
@@ -735,19 +540,6 @@ public class Cell extends Source {
 				mult=negativeErrorMult;
 			}
 		}
-//		if(v>=target) {
-//			if(v>cutoffForTraining-spread && target<cutoffForTraining+spread) {
-//				mult=falsePositiveErrorMult;
-//			}else{
-//				mult=positiveErrorMult;
-//			}
-//		}else{//Negative error
-//			if(v<cutoffForTraining+spread && target>=cutoffForTraining-spread) {
-//				mult=falseNegativeErrorMult;
-//			}else{
-//				mult=negativeErrorMult;
-//			}
-//		}
 		return ((mult-1)*multFraction)+1; //multFraction=0.5, for example, returns halfway between 1.0 and mult
 	}
 	
@@ -755,24 +547,6 @@ public class Cell extends Source {
 		float eTotalOverOut=v-target; //Aka out-target
 		final float ret=toWeightedError(eTotalOverOut, v, target, weightMult);
 		return ret;
-//		final float mult=toErrorMult(v, target);
-//		final float ret2=(float)(eTotalOverOut*mult);
-//		
-//		if(v>target) {
-//			eTotalOverOut*=positiveErrorMult;
-//			if(v>lowThresh && target<lowThresh) {
-//				eTotalOverOut*=falsePositiveErrorMult;
-//			}
-//		}else{
-//			eTotalOverOut*=negativeErrorMult;
-//			if(v<highThresh && target>=highThresh) {
-//				eTotalOverOut*=falseNegativeErrorMult;
-//			}
-//		}
-//		assert(Tools.absdif(ret, ret2)<0.00001) : ret+", "+ret2;
-//		assert(Tools.absdif(eTotalOverOut, ret2)<0.00001) : eTotalOverOut+", "+ret2;
-//		
-//		return (float)eTotalOverOut;
 	}
 	
 	public static void setLowWeightAnnealCutoff(float c) {
@@ -790,7 +564,7 @@ public class Cell extends Source {
 	/*--------------------------------------------------------------*/
 	
 	//Fake legacy method
-	public void updateEdgesHiddenLayerFast(float alpha, float[] valuesIn, double[] eOverNetNext, float[] weightsOut) {
+	public void updateEdgesHiddenLayerDense(float alpha, float[] valuesIn, double[] eOverNetNext, float[] weightsOut) {
 		throw new RuntimeException("Wrong method, for legacy double[] version in class CellNetDouble");
 	}
 
@@ -816,22 +590,25 @@ public class Cell extends Source {
 	private final int id;
 	final int lpos;//position within layer
 	final int layer;
+	final int maxLayer;
 	int nextWeight=0; //For use when loading networks
 	
 //	int type;//TODO: Change to Function.
 	Function function;
 	
 	/*--------------------------------------------------------------*/
+
+	final int prevLayerStart;
+	final int nextLayerStart;
 	
-	//Edges are not used in fast mode.
-	public final ArrayList<Edge> inputs;
-	public final ArrayList<Edge> outputs;
+	//Lpos (layer position) of inputs
+	public int[] inputs;
+	public int[] outputs;
+	public float[] weights;
+	float[] deltas;
 	
 	/*--------------------------------------------------------------*/
 	
-	public final float[] weights;
-//	private final double[] weights2;
-	private final float[] deltas;
 	private final float[] values; 
 	public final float[] eOverNetArray;
 	
@@ -877,5 +654,7 @@ public class Cell extends Source {
 	static float fnErrorIncr=0.01f;
 	static float fpErrorIncr=0.00f;
 	static float spread=0.050f;
+	static float edgeAmplitudeIncreaseMult=0.98f;
+	static float edgeAmplitudeIncreaseThresh=0.1f;
 	
 }
