@@ -4,10 +4,11 @@ import java.io.File;
 import java.io.PrintStream;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.concurrent.locks.ReadWriteLock;
+import java.util.concurrent.locks.ReentrantReadWriteLock;
 
 import dna.AminoAcid;
 import fileIO.ByteFile;
-import fileIO.ByteStreamWriter;
 import fileIO.FileFormat;
 import fileIO.ReadWrite;
 import shared.Parse;
@@ -21,7 +22,6 @@ import stream.ConcurrentReadOutputStream;
 import stream.FASTQ;
 import stream.FastaReadInputStream;
 import stream.Read;
-import structures.ByteBuilder;
 import structures.ListNum;
 import structures.LongList;
 import template.Accumulator;
@@ -29,13 +29,10 @@ import template.ThreadWaiter;
 import tracker.ReadStats;
 
 /**
- * This class does nothing.
- * It is designed to be easily modified into a program
- * that processes reads in multiple threads, by
- * filling in the processReadPair method.
+ * Filters sequences based on matching a neural network.
  * 
  * @author Brian Bushnell
- * @date November 19, 2015
+ * @date November 1, 2023
  *
  */
 public class NetFilter implements Accumulator<NetFilter.ProcessThread> {
@@ -76,7 +73,7 @@ public class NetFilter implements Accumulator<NetFilter.ProcessThread> {
 		
 		//Set shared static variables prior to parsing
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		
 		{//Parse the arguments
 			final Parser parser=parse(args);
@@ -195,6 +192,8 @@ public class NetFilter implements Accumulator<NetFilter.ProcessThread> {
 				outu2=b;
 			}else if(a.equals("annotate") || a.equals("rename")){
 				annotate=Parse.parseBoolean(b);
+			}else if(a.equals("k")){
+				k=Integer.parseInt(b);
 			}else if(a.equals("parse_flag_goes_here")){
 				long fake_variable=Parse.parseKMG(b);
 				//Set a variable here
@@ -586,7 +585,7 @@ public class NetFilter implements Accumulator<NetFilter.ProcessThread> {
 		private float processRead(Read r, float[] vec) {
 			if(r==null) {return -1;}
 			float result=(parseHeader ? Parse.parseFloat(r.id, "result=", '\t') : 0);
-			float score=score(r.bases, vec, net, rcomp);
+			float score=score(r.bases, vec, k, net, rcomp);
 
 			boolean pass=(!filter ? true : (score>=cutoff)==highpass);
 			boolean positive=(result>=0.5f);
@@ -668,33 +667,33 @@ public class NetFilter implements Accumulator<NetFilter.ProcessThread> {
 	
 	/*--------------------------------------------------------------*/
 	
-	float score(byte[] bases, float[] vec, CellNet net, boolean rcomp) {
-		final float f=score(bases, vec, net), r;
+	float score(byte[] bases, float[] vec, int k, CellNet net, boolean rcomp) {
+		final float f=score(bases, vec, k, net), r;
 		if(rcomp) {
 			AminoAcid.reverseComplementBasesInPlace(bases);
-			r=score(bases, vec, net);
+			r=score(bases, vec, k, net);
 			AminoAcid.reverseComplementBasesInPlace(bases);
 		}else {r=f;}
 		return Tools.max(r, f);
 	}
 	
-	float score(byte[] bases, float[] vec, CellNet net) {
+	float score(byte[] bases, float[] vec, int k, CellNet net) {
 		if(SCORE_MODE==SINGLE) {
-			return scoreSingle(bases, vec, net);
+			return scoreSingle(bases, vec, k, net);
 		}else {
-			return scoreFrames(bases, vec, net, SCORE_MODE, stepsize, width);
+			return scoreFrames(bases, vec, k, net, SCORE_MODE, stepsize, width);
 		}
 	}
 	
-	public static float scoreSingle(byte[] bases, float[] vec, CellNet net) {
+	public static float scoreSingle(byte[] bases, float[] vec, int k, CellNet net) {
 		assert(vec!=null);
-		SequenceToVector.fillVector(bases, vec);
+		SequenceToVector.fillVector(bases, vec, k);
 		net.applyInput(vec);
-		final float f=net.feedForwardFast();
+		final float f=net.feedForward();
 		return f;
 	}
 	
-	public static float scoreFrames(byte[] bases, float[] vec, CellNet net, int SCORE_MODE, int stepsize, int width) {
+	public static float scoreFrames(byte[] bases, float[] vec, int k, CellNet net, int SCORE_MODE, int stepsize, int width) {
 		assert(vec!=null);
 		assert(SCORE_MODE>=0 && SCORE_MODE!=SINGLE) : SCORE_MODE;
 		float min=9999, max=-9999;
@@ -702,9 +701,9 @@ public class NetFilter implements Accumulator<NetFilter.ProcessThread> {
 		long frames=1;
 		int start=0, stop=width;
 		do {
-			SequenceToVector.fillVector(bases, vec, start, stop);
+			SequenceToVector.fillVector(bases, vec, k, start, stop);
 			net.applyInput(vec);
-			final float f=net.feedForwardFast();
+			final float f=net.feedForward();
 			frames++;
 			sum+=f;
 			min=Tools.min(min, f);
@@ -715,10 +714,10 @@ public class NetFilter implements Accumulator<NetFilter.ProcessThread> {
 		return SCORE_MODE==AVERAGE ? (float)(sum/frames) : SCORE_MODE==MIN ? min : max;
 	}
 	
-	public static float score(byte[] bases, float[] vec, CellNet net, int from, int to) {
-		SequenceToVector.fillVector(bases, vec, from, to);
+	public static float score(byte[] bases, float[] vec, int k, CellNet net, int from, int to) {
+		SequenceToVector.fillVector(bases, vec, k, from, to);
 		net.applyInput(vec);
-		final float f=net.feedForwardFast();
+		final float f=net.feedForward();
 		return f;
 	}
 	
@@ -767,12 +766,19 @@ public class NetFilter implements Accumulator<NetFilter.ProcessThread> {
 	private LongList mhist=new LongList(101);
 	
 	/*--------------------------------------------------------------*/
+	
 
-	private boolean rcomp=false;
 	private boolean parseHeader=false;
-	private int width=-1;
 	private final CellNet net0;
-
+	
+	/** For raw sequence classification */
+	private boolean rcomp=false;
+	private int width=-1;
+	
+	/** For kmer spectrum classification */
+	private int k=0;
+	private int dims; //asdf//TODO
+	
 	private boolean filter=true;
 	private boolean highpass=true;
 	private boolean annotate=false;
@@ -835,6 +841,10 @@ public class NetFilter implements Accumulator<NetFilter.ProcessThread> {
 	/** Secondary unmatched output file */
 	private final FileFormat ffoutu2;
 	
+	@Override
+	public final ReadWriteLock rwlock() {return rwlock;}
+	private final ReadWriteLock rwlock=new ReentrantReadWriteLock();
+	
 	/*--------------------------------------------------------------*/
 	/*----------------        Common Fields         ----------------*/
 	/*--------------------------------------------------------------*/
