@@ -78,7 +78,7 @@ public class ScannerThread extends Thread {
 
 			for(int i=0; i<seedsToEvaluate; i++) {
 				final long seed=(i==0 ? netSeed0 : randy.nextLong()&Long.MAX_VALUE);
-				net0=parent.randomNetwork(seed, 0);
+				net0=parent.randomNetwork(seed);
 				data.reset();
 				runEpochs();
 				//			if(heap.size()<seedsToReturn){
@@ -101,7 +101,7 @@ public class ScannerThread extends Thread {
 				CellNet net=heap.poll();
 //				System.err.println(net.header());
 				
-				list.add(new Seed(net.seed, net.annealSeed, net.pivot()));
+				list.add(new Seed(net.seed, /*net.annealSeed,*/ net.pivot()));
 			}
 
 			try {
@@ -240,7 +240,7 @@ public class ScannerThread extends Thread {
 		@Override
 		public void run(){
 			for(JobData job=getJob(); job!=JobData.POISON; job=getJob()) {
-				launchJobsInner(job.immutableNet, job.set, job.maxSamples, job.epoch, job.alpha, 
+				launchJobsInner(job.immutableNet, job.set, job.maxSamples, job.epoch, job.alpha,  
 						job.backprop, job.weightMult, job.sort);
 			}
 		}
@@ -548,7 +548,7 @@ public class ScannerThread extends Thread {
 		net.alpha=(float) alpha;
 		net.annealStrength=0;
 		net.epoch=currentEpoch;
-		net.cutoff=(float) lastCutoffForTarget;
+		net.setCutoff((float) lastCutoffForTarget);
 	}
 	
 	/*--------------------------------------------------------------*/
