@@ -7,6 +7,8 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.Random;
 import java.util.concurrent.ArrayBlockingQueue;
+import java.util.concurrent.locks.ReadWriteLock;
+import java.util.concurrent.locks.ReentrantReadWriteLock;
 
 import fileIO.ByteFile;
 import fileIO.ByteStreamWriter;
@@ -74,7 +76,7 @@ public class Trainer implements Accumulator<WorkerThread> {
 		
 		//Set shared static variables prior to parsing
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		
 		{//Parse the arguments
 			final Parser parser=parse(args);
@@ -95,7 +97,7 @@ public class Trainer implements Accumulator<WorkerThread> {
 		checkStatics(); //Adjust file-related static fields as needed for this program
 
 		ffDataOut=FileFormat.testOutput(dataOut, FileFormat.TXT, null, true, overwrite, false, false);
-		ffNetOutFinal=FileFormat.testOutput(netOutFinal, FileFormat.TXT, null, true, overwrite, false, false);
+		ffNetOutFinal=FileFormat.testOutput(netOutFinal, FileFormat.TXT, null, true, true, false, false);
 		ffNetOutBest=FileFormat.testOutput(netOutBest, FileFormat.TXT, null, true, true, false, false);
 		if(ffNetOutBest!=null && ffNetOutBest.exists() && !overwrite) {
 			throw new RuntimeException(ffNetOutBest.name()+" exists and overwrite = "+overwrite);
@@ -113,7 +115,7 @@ public class Trainer implements Accumulator<WorkerThread> {
 		alphaDropoff=1.0/Math.pow(alphaMult2*alphaMult, 1.0/alphaEpochs);
 
 		randyNetSeed=(netSeed0<0 ? new Random() : new Random(netSeed0));
-		randyAnnealSeed=(annealSeed0<0 ? new Random() : new Random(netSeed0));
+//		randyAnnealSeed=(annealSeed0<0 ? new Random() : new Random(netSeed0));
 		
 		//Can be 0.9, for example
 		maxAnnealEpoch=(maxAnnealEpoch<Integer.MAX_VALUE ? maxAnnealEpoch : (int)(maxAnnealEpochMult*maxEpochs));
@@ -205,10 +207,18 @@ public class Trainer implements Accumulator<WorkerThread> {
 				}
 			}else if(a.equals("concise")){
 				CellNet.CONCISE=Parse.parseBoolean(b);
-			}else if(a.equals("fast")){
-				CellNet.FAST=Parse.parseBoolean(b);
-			}else if(a.equals("slow")){
-				CellNet.FAST=!Parse.parseBoolean(b);
+			}else if(a.equals("dense")){
+				CellNet.DENSE=Parse.parseBoolean(b);
+			}else if(a.equals("sparse")){
+				CellNet.DENSE=!Parse.parseBoolean(b);
+			}else if(a.equals("outsparse")){
+				CellNet.OUT_SPARSE=Parse.parseBoolean(b);
+				if(CellNet.OUT_SPARSE) {CellNet.OUT_DENSE=false;}
+			}else if(a.equals("outdense")){
+				CellNet.OUT_DENSE=Parse.parseBoolean(b);
+				if(CellNet.OUT_DENSE) {CellNet.OUT_SPARSE=false;}
+			}else if(a.equals("outhex")){
+				CellNet.OUT_HEX=Parse.parseBoolean(b);
 			}else if(a.equals("print") || a.equals("printstatus")){
 				printStatus=Parse.parseBoolean(b);
 			}else if(a.equals("machineout")){
@@ -320,22 +330,37 @@ public class Trainer implements Accumulator<WorkerThread> {
 				netSeed0=Long.parseLong(b);
 				setNetSeed=true;
 			}else if(a.equals("annealseed")){
-				annealSeed0=Long.parseLong(b);
-				setAnnealSeed=true;
+				assert(false) : "annealseed is no longer supported.";
+//				annealSeed0=Long.parseLong(b);
+//				setAnnealSeed=true;
 			}else if(a.equals("shuffleseed")){
 				SampleSet.shuffleSeed=Long.parseLong(b);
 			}else if(a.equals("seed")){
-				netSeed0=annealSeed0=Long.parseLong(b);
-				setNetSeed=setAnnealSeed=true;
-			}else if(a.equals("density")){
-				density=Float.parseFloat(b);
-				assert(density>0 && density<=1) : "Density must be between 0 and 1: "+density;
-			}else if(a.equals("dims") || a.equals("dimensions")){
-				dims0=Parse.parseIntArray(b, ",");
+				netSeed0=Long.parseLong(b);
+				setNetSeed=true;
+//				netSeed0=annealSeed0=Long.parseLong(b);
+//				setNetSeed=setAnnealSeed=true;
+			}else if(a.equals("density") || a.equals("maxdensity")){
+				maxDensity=Float.parseFloat(b);
+				assert(maxDensity>0 && maxDensity<=1) : "Density must be between 0 and 1: "+maxDensity;
+//				maxEdges=Long.MAX_VALUE;
+			}else if(a.equals("density1")){
+				maxDensity1=Float.parseFloat(b);
+			}else if(a.equals("densityarray")){
+				densityArray=Parse.parseFloatArray(b, ",", "x");
+			}else if(a.equals("edges") || a.equals("maxedges")){
+				maxEdges=Parse.parseKMG(b);
+//				maxDensity=0;
+			}else if(a.equals("edgeblocksize") || a.equals("blocksize") || a.equals("quant") || a.equals("ebs")){
+				edgeBlockSize=Parse.parseIntKMG(b);
+			}
+			
+			else if(a.equals("dims") || a.equals("dimensions")){
+				dims0=Parse.parseIntArray(b, ",", "x");
 			}else if(a.equals("mindims") || a.equals("mindimensions")){
-				minDims=Parse.parseIntArray(b, ",");
+				minDims=Parse.parseIntArray(b, ",", "x");
 			}else if(a.equals("maxdims") || a.equals("maxdimensions")){
-				maxDims=Parse.parseIntArray(b, ",");
+				maxDims=Parse.parseIntArray(b, ",", "x");
 			}else if(a.equals("in") || a.equals("data") || a.equals("din")){
 				dataIn=b;
 			}else if(a.equals("validateset") || a.equals("validate") || a.equals("vin") || a.equals("vset")){
@@ -418,6 +443,8 @@ public class Trainer implements Accumulator<WorkerThread> {
 				allowMTSort=Parse.parseBoolean(b);
 			}else if(a.equals("sortinthread")){
 				sortInThread=Parse.parseBoolean(b);
+			}else if(a.equals("pivotsort")){
+				PIVOT_SORT=Parse.parseBoolean(b);
 			}else if(a.equals("launchinthread")){
 				launchInThread=Parse.parseBoolean(b);
 			}else if(a.equals("setlock") || a.equals("usesetlock")){
@@ -477,6 +504,20 @@ public class Trainer implements Accumulator<WorkerThread> {
 				Cell.spread=Float.parseFloat(b);
 			}else if(a.equals("excesspivotmult") || a.equals("pivotmult") || a.equals("epm")){
 				Sample.excessPivotMult=Float.parseFloat(b);
+			}else if(a.equalsIgnoreCase("edgeAmplitudeIncreaseMult") || a.equals("eaim")){
+				Cell.edgeAmplitudeIncreaseMult=Float.parseFloat(b);
+			}else if(a.equalsIgnoreCase("edgeAmplitudeIncreaseThresh") || a.equals("eait")){
+				Cell.edgeAmplitudeIncreaseThresh=Float.parseFloat(b);
+			}
+			
+			else if(a.equalsIgnoreCase("probFlat") || a.equals("pflat")){
+				CellNet.PROB_FLAT=Float.parseFloat(b);
+			}else if(a.equalsIgnoreCase("probExp") || a.equals("pexp")){
+				CellNet.PROB_EXP=Float.parseFloat(b);
+			}else if(a.equalsIgnoreCase("explamda") || a.equals("lamda") || a.equals("lambda")){
+				CellNet.EXP_LAMDA=Float.parseFloat(b);
+			}else if(a.equalsIgnoreCase("randweightcap") || a.equalsIgnoreCase("weightcap") || a.equals("expcap")){
+				CellNet.RAND_WEIGHT_CAP=Float.parseFloat(b);
 			}
 			
 			else if(a.equals("convertto01") || a.equals("converttoboolean")){
@@ -685,7 +726,9 @@ public class Trainer implements Accumulator<WorkerThread> {
 //		outstream.println(Tools.timeLinesBytesProcessed(t, linesProcessed, bytesProcessed, 8));
 		outstream.println(Tools.timeLinesProcessed(t, linesProcessed, 8));
 //		outstream.println(Tools.thingsProcessed(t.elapsed, linesProcessed*(net0.list.size()-1), 8, "Cells"));
-		outstream.println(Tools.thingsProcessed(t.elapsed, linesProcessed*(finalNet.countEdges()), 8, "Edges"));
+		
+		//TODO: This is not correct for variable-width networks
+		outstream.println(Tools.thingsProcessed(t.elapsed, linesProcessed*(edgesIn/networksIn), 8, "Edges"));
 		
 		outstream.println();
 		outstream.println("Lines In:          \t"+(DataLoader.lastValidLines+DataLoader.lastInvalidLines));
@@ -700,7 +743,7 @@ public class Trainer implements Accumulator<WorkerThread> {
 		}
 		}
 		
-		if(machineOut){
+		if(machineOut && bestNet!=null && bestNet.dims!=null){
 			String sh=toMachineHeader(bestNet.dims);
 			String ss=toMachineOut(t);
 			outstream.println(sh);
@@ -820,7 +863,7 @@ public class Trainer implements Accumulator<WorkerThread> {
 			workerQueue.add(JobData.POISON);
 		}
 		
-		boolean success=ThreadWaiter.waitForThreads(alwt, this);
+		boolean success=ThreadWaiter.waitForThreadsToFinish(alwt, this);
 		mprof.printTimes();
 		wprof.printTimes();
 		errorState&=!success;
@@ -881,8 +924,12 @@ public class Trainer implements Accumulator<WorkerThread> {
 				}
 			}
 			
-			outstream.println("\nBest Seed:\t"+bestNet.seed+", "+bestNet.annealSeed);
+			outstream.println("\nBest Seed:\t"+bestNet.seed);
 		}else{
+			if(bestNet!=null) {
+				outstream.println("Best:\t\t"+genPrintString(bestNet, -1, false, 
+						printFPR, printFNR, printTPR, printTNR, printCutoff, false, false));
+			}
 			outstream.println("Final:\t\t"+genPrintString(finalNet, -1, false, 
 					printFPR, printFNR, printTPR, printTNR, printCutoff, false, false));
 			if(!quiet) {outstream.println();}
@@ -895,6 +942,8 @@ public class Trainer implements Accumulator<WorkerThread> {
 		assert(networksPerCycle==nets.length);
 		ArrayList<TrainerThread> altt=new ArrayList<TrainerThread>(networksPerCycle);
 		for(int i=0; i<nets.length; i++) {
+			networksIn++;
+			edgesIn+=nets[i].countEdges();
 			altt.add(new TrainerThread(this, nets[i]));
 		}
 		for(TrainerThread tt : altt) {
@@ -956,6 +1005,7 @@ public class Trainer implements Accumulator<WorkerThread> {
 		synchronized(tt) {
 			mprof.accumulate(tt.mprof);
 			networksTested++;
+//			edgesTested+=tt.net0.countEdges();
 		}
 	}
 	
@@ -1070,24 +1120,39 @@ public class Trainer implements Accumulator<WorkerThread> {
 	private CellNet randomNetwork() {
 		final long seed=(seedList!=null && netsMade<seedList.size() ? seedList.get(netsMade) : 
 			(netsMade>0 || netSeed0<0 ? randyNetSeed.nextLong()&Long.MAX_VALUE : netSeed0));
-		final long aseed=(netsMade>0 || annealSeed0<0 ? 
-				randyAnnealSeed.nextLong()&Long.MAX_VALUE : annealSeed0);
+//		final long aseed=(netsMade>0 || annealSeed0<0 ? 
+//				randyAnnealSeed.nextLong()&Long.MAX_VALUE : annealSeed0);
 		netsMade++;
-		return randomNetwork(seed, aseed);
+		return randomNetwork(seed);
 	}
 	
-	public final CellNet randomNetwork(long seed, long aseed) {
+	public final CellNet randomNetwork(long seed) {
 		final int[] dims=(dims0!=null ? dims0 : makeDims(minDims, maxDims, seed));
-		return randomNetwork(dims, seed, aseed);
+		return randomNetwork(dims, seed, maxDensity, maxDensity1, maxEdges, edgeBlockSize, commands);
 	}
 	
-	public final CellNet randomNetwork(int[] dims, long seed, long aseed) {
-		final CellNet net=new CellNet(dims, seed, commands);
-		net.annealSeed=aseed;
-		net.randomize(density);
+	public static final CellNet randomNetwork(int[] dims, long seed, float maxDensity,
+			float maxDensity1, long maxEdges, int edgeBlockSize, ArrayList<String> commands) {
+		final long fullyConnectedEdges=fullyConnectedEdges(dims);
+		float density=maxEdges<1 ? maxDensity : 
+			Tools.min(maxDensity, (float)(maxEdges/(double)fullyConnectedEdges));
+		float density1=(maxDensity1<=0 ? density : maxDensity1);
+		assert(density>0f && density<=1f) : density+", "+maxEdges+", "+fullyConnectedEdges;
+		final CellNet net=new CellNet(dims, seed, density, density1, edgeBlockSize, commands);
+//		net.annealSeed=aseed;
+		net.randomize();
 		return net;
 	}
 	
+	public static long fullyConnectedEdges(int[] dims) {
+		long sum=0;
+		for(int i=1; i<dims.length; i++) {
+			long edges=dims[i-1]*dims[i];
+			sum+=edges;
+		}
+		return sum;
+	}
+	
 	public static final int[] makeDims(int[] minDims, int[] maxDims, long seed) {
 		Random randy=Shared.threadLocalRandom(seed);
 		assert(minDims.length==maxDims.length);
@@ -1114,6 +1179,7 @@ public class Trainer implements Accumulator<WorkerThread> {
 	
 	private CellNet loadNetwork(String path){
 		final CellNet net=CellNetParser.load(path);
+		if(setCutoffForEvaluation) {net.setCutoff(cutoffForEvaluation);}
 //		dims=net.dims.clone();
 		assert(numInputs<0 || net.numInputs()==numInputs);
 		assert(numOutputs<0 || net.numOutputs()==numOutputs);
@@ -1121,7 +1187,7 @@ public class Trainer implements Accumulator<WorkerThread> {
 		numInputs=net.numInputs();
 		numOutputs=net.numOutputs();
 		net.commands.add(commandLine);
-		if(net.annealSeed<0) {net.annealSeed=annealSeed0;}
+//		if(net.annealSeed<0) {net.annealSeed=annealSeed0;}
 		return net;
 	}
 	
@@ -1253,7 +1319,7 @@ public class Trainer implements Accumulator<WorkerThread> {
 	private CellNet makeAvg(ArrayList<CellNet> list0) {
 		ArrayList<CellNet> list=concentrateBest(list0);
 		final CellNet zero=list.get(0);
-		CellNet base=new CellNet(new int[] {1,1}, 1, null);
+		CellNet base=new CellNet(new int[] {1,1}, 1, 1.0f, 0f, 1, null);
 		base.epoch=zero.epoch;
 		base.alpha=zero.alpha;
 		base.annealStrength=zero.annealStrength;
@@ -1263,7 +1329,7 @@ public class Trainer implements Accumulator<WorkerThread> {
 		base.fnRate=0;
 		base.tpRate=0;
 		base.tnRate=0;
-		base.cutoff=0;
+		float cutoff=0;
 		for(CellNet net : list) {
 			base.errorRate+=net.errorRate;
 			base.weightedErrorRate+=net.weightedErrorRate;
@@ -1271,7 +1337,7 @@ public class Trainer implements Accumulator<WorkerThread> {
 			base.fnRate+=net.fnRate;
 			base.tpRate+=net.tpRate;
 			base.tnRate+=net.tnRate;
-			base.cutoff+=net.cutoff;
+			cutoff+=net.cutoff;
 		}
 		final float mult=1.0f/list.size();
 		base.errorRate*=mult;
@@ -1280,7 +1346,7 @@ public class Trainer implements Accumulator<WorkerThread> {
 		base.fnRate*=mult;
 		base.tpRate*=mult;
 		base.tnRate*=mult;
-		base.cutoff*=mult;
+		base.setCutoff(cutoff*mult);
 		return base;
 	}
 	
@@ -1459,11 +1525,16 @@ public class Trainer implements Accumulator<WorkerThread> {
 	boolean exclusive=true;
 
 	private int netsMade=0;
-	private float density=1.0f;
+	private float maxDensity=1.0f;
+	//For layer 1 input only (between layers 0 and 1).
+	private float maxDensity1=0f;
+	private int edgeBlockSize=1;
+	private long maxEdges=-1;
+	private float[] densityArray=null;
 	private long netSeed0=-1;
-	private long annealSeed0=7;
+//	private long annealSeed0=7;
 	private boolean setNetSeed=false;
-	private boolean setAnnealSeed=false;
+//	private boolean setAnnealSeed=false;
 	
 	int threads=1;
 	
@@ -1499,6 +1570,7 @@ public class Trainer implements Accumulator<WorkerThread> {
 	boolean launchInThread=false;
 	boolean allowMTSort=true;
 	
+	static boolean PIVOT_SORT=false;
 	static int SHUFFLEMOD=4;//subcycle to shuffle on. 4 seems better than 6.
 	
 	//setlock=t setnet=t copynet=f was 2x slower,
@@ -1615,6 +1687,9 @@ public class Trainer implements Accumulator<WorkerThread> {
 	private long bytesProcessed=0;
 	private long bytesOut=0;
 
+	private long edgesIn=0;
+	private long networksIn=0;
+	
 	private int maxLines=Shared.MAX_ARRAY_LEN;
 	int maxLinesV=-1;
 	private boolean shuffleRaw=false;
@@ -1640,9 +1715,13 @@ public class Trainer implements Accumulator<WorkerThread> {
 //	/** Optional Output File for Junk */
 //	private final FileFormat ffoutInvalid;
 	
-	final Random randyAnnealSeed;
+//	final Random randyAnnealSeed;
 	final Random randyNetSeed;
 	
+	@Override
+	public final ReadWriteLock rwlock() {return rwlock;}
+	private final ReadWriteLock rwlock=new ReentrantReadWriteLock();
+	
 	/*--------------------------------------------------------------*/
 	/*----------------        Common Fields         ----------------*/
 	/*--------------------------------------------------------------*/
