@@ -8,6 +8,8 @@ import java.util.Comparator;
 import java.util.HashMap;
 import java.util.Map.Entry;
 import java.util.concurrent.ConcurrentLinkedQueue;
+import java.util.concurrent.locks.ReadWriteLock;
+import java.util.concurrent.locks.ReentrantReadWriteLock;
 
 import aligner.SingleStateAlignerFlat2;
 import consensus.BaseGraph;
@@ -77,7 +79,7 @@ public class MergeRibo implements Accumulator<MergeRibo.ProcessThread> {
 		
 		//Set shared static variables prior to parsing
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 //		Shared.capBufferLen(40);//This does not help; the slowness comes from unevenness in list length during pickBest.
 		//To fix it, long lists should be sorted to be first.
 		
@@ -712,6 +714,10 @@ public class MergeRibo implements Accumulator<MergeRibo.ProcessThread> {
 	
 	final int threads;
 	
+	@Override
+	public final ReadWriteLock rwlock() {return rwlock;}
+	private final ReadWriteLock rwlock=new ReentrantReadWriteLock();
+	
 	/*--------------------------------------------------------------*/
 	/*----------------        Common Fields         ----------------*/
 	/*--------------------------------------------------------------*/
