@@ -3,6 +3,8 @@ package prok;
 import java.io.PrintStream;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.concurrent.locks.ReadWriteLock;
+import java.util.concurrent.locks.ReentrantReadWriteLock;
 
 import aligner.SingleStateAlignerFlat2;
 import fileIO.ByteFile;
@@ -68,7 +70,7 @@ public class SplitRibo implements Accumulator<SplitRibo.ProcessThread> {
 		
 		//Set shared static variables prior to parsing
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		Shared.capBufferLen(50);
 		ReadWrite.ZIPLEVEL=9;
 		
@@ -602,6 +604,10 @@ public class SplitRibo implements Accumulator<SplitRibo.ProcessThread> {
 	/** Primary input file */
 	private final FileFormat ffin1;
 	
+	@Override
+	public final ReadWriteLock rwlock() {return rwlock;}
+	private final ReadWriteLock rwlock=new ReentrantReadWriteLock();
+	
 	/*--------------------------------------------------------------*/
 	/*----------------         Static Fields        ----------------*/
 	/*--------------------------------------------------------------*/
