@@ -3,6 +3,8 @@ package repeat;
 import java.io.PrintStream;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.concurrent.locks.ReadWriteLock;
+import java.util.concurrent.locks.ReentrantReadWriteLock;
 
 import dna.AminoAcid;
 import fileIO.ByteFile;
@@ -80,7 +82,7 @@ public class RepeatFinder implements Accumulator<RepeatFinder.ProcessThread> {
 		
 		//Set shared static variables prior to parsing
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		FASTQ.TEST_INTERLEAVED=FASTQ.FORCE_INTERLEAVED=false;
 		
 		{//Parse the arguments
@@ -1044,6 +1046,10 @@ public class RepeatFinder implements Accumulator<RepeatFinder.ProcessThread> {
 	
 	private final byte[] baseMaskArray;
 	
+	@Override
+	public final ReadWriteLock rwlock() {return rwlock;}
+	private final ReadWriteLock rwlock=new ReentrantReadWriteLock();
+	
 	/*--------------------------------------------------------------*/
 	/*-----------        Symbol-Specific Constants        ----------*/
 	/*--------------------------------------------------------------*/
