@@ -19,8 +19,10 @@ import com.sun.net.httpserver.Headers;
 import com.sun.net.httpserver.HttpExchange;
 
 import fileIO.ReadWrite;
+import shared.Timer;
 import shared.Tools;
 import structures.ByteBuilder;
+import structures.ListNum;
 import structures.StringNum;
 
 public class ServerTools {
@@ -162,6 +164,93 @@ public class ServerTools {
 	}
 	
 	
+	/** Send a message to a remote URL, and return the response.
+	 * Set message to null if there is no message. */
+	public static StringNum sendAndReceive(ArrayList<byte[]> messages, String address, 
+			boolean verbose){
+    	address=PercentEncoding.commonSymbolToCode(address);
+		URL url=null;
+		InputStream is=null;
+		HttpURLConnection connection=null;
+		OutputStream os=null;
+		try {
+			url=new URL(address);
+			connection=(HttpURLConnection) url.openConnection();
+			connection.setDoOutput(true);
+			connection.setConnectTimeout(1600000); //For testing
+			os=connection.getOutputStream();
+		} catch (IOException e1) {
+//			System.err.println("A:\t"+address+" -> "+url+" -> "+connection+" -> "+os);
+			// TODO Auto-generated catch block
+			if(!suppressErrors){e1.printStackTrace();}
+		}
+		
+		long sent=0, received=0;
+		Timer t=new Timer();
+		if(os!=null){
+			try {
+				//TODO: It may be useful to set a timeout here.
+				for(byte[] message : messages) {
+					if(message!=null){
+						os.write(message);
+						sent+=message.length;
+					}
+				}
+			} catch (IOException e) {
+//				System.err.println("B:\t"+connection+" -> "+os);
+				// TODO Auto-generated catch block
+				if(!suppressErrors){e.printStackTrace();}
+			}
+			try {
+				os.close();
+			} catch (IOException e) {
+				// TODO Auto-generated catch block
+				e.printStackTrace();
+			}
+		}
+		if(verbose) {t.stopAndStart("Sent "+sent+" bytes in ");}
+
+		String result=null;
+		int responseCode=1;
+		if(connection!=null){
+			IOException noInputStream=null;
+			try {
+				is=connection.getInputStream();
+			} catch (IOException e) {
+				noInputStream=e;
+			}
+			if(is==null){is=connection.getErrorStream();}
+			
+			try {
+				responseCode=connection.getResponseCode();
+				if(!suppressErrors && (responseCode<200 || responseCode>299)){
+					System.err.println("Error: Server returned response code "+responseCode);
+				}
+			} catch (IOException e) {
+				if(!suppressErrors) {
+					e.printStackTrace();
+				}
+			}
+			
+			if(is!=null){
+				result=readStream(is);
+				received+=(result==null ? 0 : result.length());
+				try {
+//					System.err.println("C:\t"+connection+" -> "+os+" -> "+is+" -> "+(result!=null));
+					is.close();
+				} catch (IOException e) {
+					e.printStackTrace();
+				}
+			}else if(noInputStream!=null && !suppressErrors){
+				noInputStream.printStackTrace();
+			}
+		}
+		if(verbose) {t.stop("Recieved "+received+" bytes in ");}
+		
+		return new StringNum(result, responseCode);
+	}
+	
+	
 	/** Send a message to a remote URL, and return the response.
 	 * Set message to null if there is no message. */
 	public static StringNum sendAndReceiveFTP(byte[] message, String address){
@@ -267,6 +356,38 @@ public class ServerTools {
 		return null;
 	}
 	
+	/** Completely read an InputStream into a String */
+	public static ArrayList<byte[]> readStreamToList(InputStream is){
+		if(is==null){return null;}
+		ArrayList<byte[]> list=new ArrayList<byte[]>();
+		ByteBuilder bb=new ByteBuilder();
+		long next=0;
+		final byte[] buffer=new byte[4096];
+		try {
+			int count=is.read(buffer);
+			while(count>-1){
+				next+=count;
+				bb.append(buffer, count);
+				if(bb.length()>8000000) {
+					list.add(bb.toBytes());
+					bb.clear();
+				}
+				count=is.read(buffer);
+			}
+			if(bb.length()>0) {
+				list.add(bb.toBytes());
+				bb.clear();
+			}
+			System.err.println("Read "+next+" bytes into "+list.size()+" chunks.");
+			is.close();
+			return list;
+		} catch (IOException e) {
+			// TODO Auto-generated catch block
+			e.printStackTrace();
+		}
+		return null;
+	}
+	
 	/** Write to the body of an incoming HTTP session */
 	public static boolean reply(String response, String type, HttpExchange t, boolean verbose, int code, boolean close){
 		if(verbose){System.err.println("Sending: "+response);}
@@ -359,7 +480,8 @@ public class ServerTools {
 		return clientAddress;
 	}
 	
-	public static boolean isInternalQuery(HttpExchange t, String prefix, boolean allowLocalHost, boolean printIP, boolean printHeaders){
+	public static boolean isInternalQuery(HttpExchange t, 
+			String prefix, boolean allowLocalHost, boolean printIP, boolean printHeaders){
 		
 		InetSocketAddress client=t.getRemoteAddress();
 		InetSocketAddress server=t.getLocalAddress();
