@@ -736,6 +736,12 @@ public class Parser {
 		
 		else if(a.equalsIgnoreCase("simd")){
 			Shared.SIMD=Parse.parseBoolean(b);
+		}else if(a.equalsIgnoreCase("simdsparse")){
+			Vector.SIMD_MULT_SPARSE=Vector.SIMD_FMA_SPARSE=Parse.parseBoolean(b);
+		}else if(a.equalsIgnoreCase("simdmultsparse")){
+			Vector.SIMD_MULT_SPARSE=Parse.parseBoolean(b);
+		}else if(a.equalsIgnoreCase("simdfmasparse")){
+			Vector.SIMD_FMA_SPARSE=Parse.parseBoolean(b);
 		}else if(a.equalsIgnoreCase("simdcopy")){
 			Vector.SIMDCOPY=Parse.parseBoolean(b);
 		}
@@ -985,7 +991,7 @@ public class Parser {
 				if(zt<1){ReadWrite.USE_BGZIP=false;}
 				else{
 					ReadWrite.USE_BGZIP=true;
-					ReadWrite.MAX_ZIP_THREADS=zt;
+					ReadWrite.setZipThreads(zt);
 				}
 			}else{ReadWrite.USE_BGZIP=Parse.parseBoolean(b);}
 			if(ReadWrite.USE_BGZIP){ReadWrite.PREFER_BGZIP=true;}
@@ -998,14 +1004,14 @@ public class Parser {
 		}else if(a.equals("preferbgzip")){
 			ReadWrite.PREFER_BGZIP=Parse.parseBoolean(b);
 		}else if(a.equals("zipthreads")){
-			ReadWrite.MAX_ZIP_THREADS=Integer.parseInt(b);
+			ReadWrite.setZipThreads(Integer.parseInt(b));
 		}else if(a.equals("usepigz") || a.equals("pigz")){
 			if(b!=null && Tools.isDigit(b.charAt(0))){
 				int zt=Integer.parseInt(b);
 				if(zt<1){ReadWrite.USE_PIGZ=false;}
 				else{
 					ReadWrite.USE_PIGZ=true;
-					ReadWrite.MAX_ZIP_THREADS=zt;
+					ReadWrite.setZipThreads(zt);
 				}
 			}else{ReadWrite.USE_PIGZ=Parse.parseBoolean(b);}
 		}else if(a.equals("zipthreaddivisor") || a.equals("ztd")){
@@ -1263,7 +1269,7 @@ public class Parser {
 	}
 	
 	public static void printHelp(int exitCode){
-		System.err.println("BBMap version "+Shared.BBMAP_VERSION_STRING);
+		System.err.println("BBTools version "+Shared.BBTOOLS_VERSION_STRING);
 		System.err.println("For help, please run the shellscript with no parameters, or look in /docs/.");
 		System.exit(exitCode);
 	}
