@@ -48,6 +48,7 @@ public class Shared {
 	private static String riboSketchServerNersc="https://ribo-sketch.jgi.doe.gov/";
 	private static String proteinSketchServerNersc="https://protein-sketch.jgi.doe.gov/";
 	private static String refseqSketchServerNersc="https://refseq-sketch.jgi.doe.gov/";
+	private static String demuxServer="https://demux.jgi.doe.gov/";//port 4096 on taxvm2
 
 	private static String taxServerAws="http://bbtaxonomy.org:3068/";
 	private static String ntSketchServerAws="http://nt-sketch.org:3071/";
@@ -60,6 +61,7 @@ public class Shared {
 	public static String riboSketchServer(){return awsServers ? riboSketchServerAws : riboSketchServerNersc;}
 	public static String proteinSketchServer(){return awsServers ? proteinSketchServerAws : proteinSketchServerNersc;}
 	public static String refseqSketchServer(){return awsServers ? refseqSketchServerAws : refseqSketchServerNersc;}
+	public static String demuxServer(){return demuxServer;}
 	
 	public static boolean awsServers=false;
 	
@@ -127,8 +129,8 @@ public class Shared {
 	public static final int GAPCOST=Tools.max(1, GAPLEN/64);
 	public static final byte GAPC='-';
 	
-	public static String BBMAP_VERSION_STRING="39.06";
-	public static String BBMAP_VERSION_NAME="Tag Parse";
+	public static String BBTOOLS_VERSION_STRING="39.08";
+	public static String BBMAP_VERSION_NAME="Demux Server";
 	
 	public static boolean TRIM_READ_COMMENTS=false;
 	public static boolean TRIM_RNAME=false; //For mapped sam reads
