@@ -60,7 +60,7 @@ public class AddSSU {
 		
 		//Set shared static variables prior to parsing
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		
 		{//Parse the arguments
 			final Parser parser=parse(args);
