@@ -87,7 +87,7 @@ public class BlacklistMaker extends SketchObject {
 		if(Shared.threads()>=32){Shared.capThreads(Shared.threads()/2);}//Deals with hyperthreading, which causes excessive memory usage
 		Shared.capBuffers(Tools.max(4, Shared.threads()+1));
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		KmerCountAbstract.SKETCH_MODE=true;
 		KmerCountAbstract.STORE_HASHED=true;
 		KmerCountAbstract.KEEP_DUPLICATE_KMERS=true;
@@ -531,7 +531,7 @@ public class BlacklistMaker extends SketchObject {
 
 			ArrayList<String> extra=null;
 			ReadCounter rc=new ReadCounter(k, rcomp, false, false, amino);
-			filter[0]=rc.makeKca(in1, in2, extra, cbits, precells, prehashes, minq, maxReads, 1, 1, 1, 1, filter[0], filterMax);
+			filter[0]=rc.makeKca(in1, in2, extra, cbits, precells, prehashes, minq, maxReads, 1, 1, 1, filter[0], filterMax);
 			assert(filterMax<=filter[0].maxValue || (currentPass==0)) : filterMax+", "+currentPass+", "+filter[0].maxValue;
 			outstream.println("Made prefilter:   \t"+filter[0].toShortString(prehashes));
 			double uf=filter[0].usedFraction();
