@@ -79,7 +79,7 @@ public class BlacklistMaker2 extends SketchObject {
 		if(Shared.threads()>=32){Shared.capThreads(Shared.threads()/2);}//Deals with hyperthreading, which causes excessive memory usage
 		Shared.capBuffers(Tools.max(4, Shared.threads()+1));
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		KmerCountAbstract.SKETCH_MODE=true;
 		KmerCountAbstract.STORE_HASHED=true;
 		KmerCountAbstract.KEEP_DUPLICATE_KMERS=true;
