@@ -81,7 +81,7 @@ public class CompareSketch extends SketchObject {
 		
 		//Set shared static variables
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		KILL_OK=true;
 		TaxFilter.REQUIRE_PRESENT=false;
 		defaultParams.mode=PER_FILE;
