@@ -72,7 +72,7 @@ public class KmerLimit2 extends SketchObject {
 		
 		//Set shared static variables
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		SketchObject.setKeyFraction(0.1);
 		defaultParams.minEntropy=0;
 		defaultParams.minProb=0.2f;
