@@ -74,7 +74,7 @@ public class Shuffle2 {
 		//Set shared static variables
 		Shared.capBuffers(4);
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		FASTQ.TEST_INTERLEAVED=true; //TEST_INTERLEAVED must explicitly be disabled with int=f to sort corrupt files.
 		FASTQ.FORCE_INTERLEAVED=false;
 		
@@ -544,9 +544,9 @@ public class Shuffle2 {
 	
 	private static void mergeAndDump(final ArrayList<CrisContainer> q, final ConcurrentReadOutputStream ros, PrintStream outstream) {
 		
-		for(CrisContainer cc : q){
-			assert(!cc.cris().paired()) : FASTQ.TEST_INTERLEAVED+", "+FASTQ.FORCE_INTERLEAVED;
-		}
+//		for(CrisContainer cc : q){
+//			assert(!cc.cris().paired()) : FASTQ.TEST_INTERLEAVED+", "+FASTQ.FORCE_INTERLEAVED;
+//		}
 		
 		final int limit=20000;
 		ArrayList<Read> buffer=new ArrayList<Read>(2*limit);
