@@ -205,7 +205,11 @@ public class ConcurrentGenericReadInputStream extends ConcurrentReadInputStream
 			rt2==null ? new Thread[] {Thread.currentThread(), rt1} :
 				new Thread[] {Thread.currentThread(), rt1, rt2});
 
-		readLists();
+		try {
+			readLists();
+		} catch (OutOfMemoryError e) {
+			KillSwitch.memKill(e);
+		}
 //		readSingles();
 
 		addPoison();
