@@ -11,6 +11,8 @@ import dna.Data;
 import dna.Gene;
 import dna.ScafLoc;
 import shared.KillSwitch;
+import shared.LineParser1;
+import shared.LineParser2;
 import shared.Parse;
 import shared.Shared;
 import shared.Tools;
@@ -49,7 +51,7 @@ public class SamLine implements Serializable {
 	 */
 	private static final long serialVersionUID = -4180486051387471116L;
 
-	public SamLine(String s){
+	public SamLine(String s){//TODO: Change to LineParser version
 		this(s.split("\t"));
 	}
 
@@ -57,18 +59,18 @@ public class SamLine implements Serializable {
 		setFrom(sl);
 	}
 	
-	/** Prevents references to original string, in case of e.g. very long MD tags. */
-	public SamLine toSamLine(String s){
-		String[] split=s.split("\t");
-		split[0]=new String(split[0]);
-		split[5]=new String(split[5]);
-		split[9]=new String(split[9]);
-		split[10]=new String(split[10]);
-		for(int i=11; i<split.length; i++){
-			split[i]=new String(split[i]);
-		}
-		return new SamLine(split);
-	}
+//	/** Prevents references to original string, in case of e.g. very long MD tags. */
+//	public SamLine toSamLine(String s){
+//		String[] split=s.split("\t");
+//		split[0]=new String(split[0]);
+//		split[5]=new String(split[5]);
+//		split[9]=new String(split[9]);
+//		split[10]=new String(split[10]);
+//		for(int i=11; i<split.length; i++){
+//			split[i]=new String(split[i]);
+//		}
+//		return new SamLine(split);
+//	}
 	
 	private void setFrom(SamLine sl){
 		qname=sl.qname;
@@ -463,6 +465,7 @@ public class SamLine implements Serializable {
 			optional=new ArrayList<String>(1);
 			for(int i=11; i<s.length; i++){
 				if(s[i].startsWith("MD:")){
+					mdTag=s[i].substring(3).getBytes();
 					optional.add(s[i]);
 					break;
 				}
@@ -470,6 +473,9 @@ public class SamLine implements Serializable {
 		}else{
 			optional=new ArrayList<String>(s.length-11);
 			for(int i=11; i<s.length; i++){
+				if(s[i].startsWith("MD:")){
+					mdTag=s[i].substring(3).getBytes();
+				}
 				optional.add(s[i]);
 			}
 		}
@@ -477,6 +483,63 @@ public class SamLine implements Serializable {
 		trimNames();
 	}
 	
+	public SamLine(LineParser1 lp){
+		assert(!lp.startsWith('@')) : "Tried to make a SamLine from a header: "+lp.toString();
+		
+		if(PARSE_0){qname=lp.parseString(0);}
+		flag=lp.parseInt(1);
+		if(PARSE_2){
+			if(RNAME_AS_BYTES){
+				rname=(lp.termEquals('*', 2)) ? null : lp.parseByteArray(2);
+			}else {
+				rnameS=(lp.termEquals('*', 2)) ? null : lp.parseString(2);
+			}
+		}
+		pos=lp.parseInt(3);
+		mapq=lp.parseInt(4);
+		if(PARSE_5){cigar=lp.parseString(5);}
+		if(PARSE_6){rnext=lp.parseByteArray(6);}
+		if(PARSE_7){pnext=lp.parseInt(7);}
+		if(PARSE_8){tlen=lp.parseInt(8);}
+		seq=lp.parseByteArray(9);
+		if(PARSE_10){qual=lp.parseByteArray(10);}
+
+		assert((seq==bytestar)==(Tools.equals(seq, bytestar)));
+		assert((qual==bytestar)==(Tools.equals(qual, bytestar)));
+		
+		if(mapped() && strand()==Shared.MINUS && FLIP_ON_LOAD){
+			if(seq!=bytestar){AminoAcid.reverseComplementBasesInPlace(seq);}
+			if(qual!=bytestar){Tools.reverseInPlace(qual);}
+		}
+		
+		if(qual!=null && qual!=bytestar){
+			for(int i=0; i<qual.length; i++){qual[i]-=33;}
+		}
+		
+		if(PARSE_OPTIONAL && lp.terms()>11) {
+			if(PARSE_OPTIONAL_MD_ONLY){
+				optional=new ArrayList<String>(1);
+				for(int i=11, terms=lp.terms(); i<terms; i++) {
+					if(lp.termStartsWith("MD:", i)){
+						String s=lp.parseString(i);
+						optional.add(s);
+						lp.incrementA(3);
+						mdTag=lp.parseByteArrayFromCurrentField();
+					}
+				}
+			}else{
+				optional=new ArrayList<String>(lp.terms()-11);
+				for(int i=11, terms=lp.terms(); i<terms; i++) {
+					String s=lp.parseString(i);
+					optional.add(s);
+				}
+			}
+		}
+		
+		trimNames();
+	}
+	
+	//TODO: Should be replaced by LineParser
 	public SamLine(byte[] s){
 		assert(s[0]!='@') : "Tried to make a SamLine from a header: "+new String(s);
 		
@@ -567,37 +630,37 @@ public class SamLine implements Serializable {
 		if(qual!=null && qual!=bytestar){
 			for(int i=0; i<qual.length; i++){qual[i]-=33;}
 		}
-		
-		if(!PARSE_OPTIONAL || b>=s.length){return;}
-		
-		if(PARSE_OPTIONAL_MD_ONLY){
-			optional=new ArrayList<String>(1);
-			while(b<s.length){
-				while(b<s.length && s[b]!='\t'){b++;}
-				if(b>a){
-					if(b>=a+3 && s[a]=='M' && s[a+1]=='D' && s[a+2]==':'){
+
+		if(PARSE_OPTIONAL && b<s.length) {
+			if(PARSE_OPTIONAL_MD_ONLY) {
+				optional=new ArrayList<String>(1);
+				while(b<s.length){
+					while(b<s.length && s[b]!='\t'){b++;}
+					if(b>a){
+						if(b>=a+3 && s[a]=='M' && s[a+1]=='D' && s[a+2]==':'){
+							String x=new String(s, a, b-a);
+							optional.add(x);
+							return;
+						}
+					}else{
+						//Empty field
+					}
+					b++;
+					a=b;
+				}
+			}else{
+				optional=new ArrayList<String>(4);
+				while(b<s.length){
+					while(b<s.length && s[b]!='\t'){b++;}
+					if(b>a){
 						String x=new String(s, a, b-a);
 						optional.add(x);
-						return;
+					}else{
+						//Empty field
 					}
-				}else{
-					//Empty field
+					b++;
+					a=b;
 				}
-				b++;
-				a=b;
-			}
-		}else{
-			optional=new ArrayList<String>(4);
-			while(b<s.length){
-				while(b<s.length && s[b]!='\t'){b++;}
-				if(b>a){
-					String x=new String(s, a, b-a);
-					optional.add(x);
-				}else{
-					//Empty field
-				}
-				b++;
-				a=b;
 			}
 		}
 		
@@ -2357,6 +2420,19 @@ public class SamLine implements Serializable {
 		return flag;
 	}
 	
+	public boolean hasCigar() {
+		return cigar!=null && cigar.length()>0 && cigar.charAt(0)!='*';
+	}
+	
+	/** contains a cigar with X or = symbols */
+	public boolean hasCigarXE() {
+		if(cigar==null || cigar.length()<1 || cigar.charAt(0)=='*') {return false;}
+		for(int i=0; i<cigar.length(); i++) {
+			char c=cigar.charAt(i);
+			if(c=='=' || c=='X') {return true;}
+		}
+		return false;
+	}
 	
 	public boolean hasMate(){
 		return (flag&0x1)==0x1;
@@ -2584,7 +2660,7 @@ public class SamLine implements Serializable {
 	/*----------------           Fields             ----------------*/
 	/*--------------------------------------------------------------*/
 	
-	private String mdTag(){
+	public String mdTag(){
 		if(optional==null){return null;}
 		for(String s : optional){
 			if(s.startsWith("MD:Z:")){return s;}
@@ -2624,6 +2700,7 @@ public class SamLine implements Serializable {
 	public byte[] seq;
 	public byte[] qual;
 	public ArrayList<String> optional;
+	public byte[] mdTag;
 	
 	public Object obj;
 	public int scafnum=-1;
