@@ -44,7 +44,7 @@ public class SamStreamerWrapper {
 		
 		Shared.capBuffers(4);
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		ReadStreamByteWriter.USE_ATTACHED_SAMLINE=true;
 		
 		filter=new SamFilter();
