@@ -187,6 +187,27 @@ public final class FloatList{
 		array=KillSwitch.copyOf(array, size3);
 	}
 	
+	public int maxIdx() {
+		if(size<1) {return -1;}
+		float max=array[0];
+		int maxIdx=0;
+		for(int i=1; i<size; i++) {
+			if(array[i]>max) {
+				max=array[i];
+				maxIdx=i;
+			}
+			max=max(max, array[i]);
+		}
+		return maxIdx;
+	}
+	
+	public float max() {
+		if(size<1) {return 0;}
+		float max=array[0];
+		for(int i=1; i<size; i++) {max=max(max, array[i]);}
+		return max;
+	}
+	
 	public final float stdev(){
 		if(size<2){return 0;}
 		double sum=sum();
@@ -407,6 +428,7 @@ public final class FloatList{
 	
 	private static final int min(int x, int y){return x<y ? x : y;}
 	private static final int max(int x, int y){return x>y ? x : y;}
+	private static final float max(float x, float y){return x>y ? x : y;}
 	
 	public float[] array;
 	public int size=0;
