@@ -8,6 +8,8 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.Locale;
 import java.util.Map.Entry;
+import java.util.concurrent.locks.ReadWriteLock;
+import java.util.concurrent.locks.ReentrantReadWriteLock;
 
 import fileIO.ByteFile;
 import fileIO.ByteFile1;
@@ -62,7 +64,7 @@ public class AnalyzeAccession implements Accumulator<AnalyzeAccession.ProcessThr
 		}
 		
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		
 		Parser parser=new Parser();
 		for(int i=0; i<args.length; i++){
@@ -198,7 +200,7 @@ public class AnalyzeAccession implements Accumulator<AnalyzeAccession.ProcessThr
 			ThreadWaiter.startThreads(alpt);
 		}
 		for(ArrayList<ProcessThread> alpt : perFileList){
-			boolean success=ThreadWaiter.waitForThreads(alpt, this);
+			boolean success=ThreadWaiter.waitForThreadsToFinish(alpt, this);
 			errorState|=!success;
 		}
 	}
@@ -423,6 +425,10 @@ public class AnalyzeAccession implements Accumulator<AnalyzeAccession.ProcessThr
 	private final FileFormat[] ffina;
 	private final FileFormat ffout;
 	
+	@Override
+	public final ReadWriteLock rwlock() {return rwlock;}
+	private final ReadWriteLock rwlock=new ReentrantReadWriteLock();
+	
 	private static final byte[] remap=makeRemap();
 	
 	private static byte[] makeRemap(){
