@@ -64,7 +64,7 @@ public class PrintTaxonomy {
 		
 		//Set shared static variables
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		
 		//Create a parser object
 		Parser parser=new Parser();
