@@ -67,7 +67,7 @@ public class SplitByTaxa {
 		ReadWrite.USE_UNPIGZ=true;
 		ReadWrite.USE_PIGZ=false;
 		ReadWrite.USE_GZIP=false;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 
 		boolean setInterleaved=false; //Whether interleaved was explicitly set.
 		String giTableFile=null;
