@@ -930,6 +930,24 @@ public class ReadStats {
 		errorState|=tsw.errorState;
 	}
 	
+	public int q30(int pairnum) {
+		return lengthAboveAverageQscore(30, pairnum);
+	}
+	
+	public int lengthAboveAverageQscore(float limit, int pairnum) {
+		final long[] qs=qualSum[pairnum];
+		long[] ql=qualLength[pairnum];
+		if(ql!=null) {ql=ql.clone();}//Prevents modification of originals
+		
+		for(int i=MAXLEN-2; i>=0; i--){ql[i]+=ql[i+1];}
+		for(int i=0; i<MAXLEN; i++) {
+			if(ql[i]<0) {return i;}
+			float avg=qs[i]/(float)ql[i];
+			if(avg<limit) {return i;}
+		}
+		return MAXLEN;
+	}
+	
 	public void writeQualityToFile(String fname, boolean writePaired){
 		StringBuilder sb=new StringBuilder();
 		final boolean measure=(matchSum!=null);
@@ -943,7 +961,10 @@ public class ReadStats {
 			sb.append("#BaseNum\tRead1_linear\tRead1_log"+(writePaired ? "\tRead2_linear\tRead2_log" : "")+"\n");
 		}
 		
-		final long[] qs1=qualSum[0], qs2=qualSum[1], ql1=qualLength[0], ql2=qualLength[1];
+		final long[] qs1=qualSum[0], qs2=qualSum[1];
+		long[] ql1=qualLength[0], ql2=qualLength[1];
+		if(ql1!=null) {ql1=ql1.clone();}//Prevents modification of originals
+		if(ql2!=null) {ql2=ql2.clone();}
 		final double[] qsd1=qualSumDouble[0], qsd2=qualSumDouble[1];
 		
 		for(int i=MAXLEN-2; i>=0; i--){
