@@ -84,7 +84,7 @@ public class CallVariants2 {
 		
 		//Set shared static variables
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		ReadWrite.USE_BGZIP=true;
 		
 		//Create a parser object
