@@ -46,7 +46,7 @@ public class CompareVCF {
 		}
 		
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		
 		int mode_=DIFFERENCE;
 		
