@@ -68,7 +68,7 @@ public class FilterSam {
 		
 		//Set shared static variables
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		SamLine.SET_FROM_OK=true;
 		Var.CALL_INS=false;
 		Var.CALL_DEL=false;
