@@ -45,7 +45,7 @@ public class FilterVCF {
 		}
 		
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		
 		varFilter.clear();
 
