@@ -71,7 +71,7 @@ public class Realign {
 		SamLine.SET_FROM_OK=true;
 		ReadStreamWriter.USE_ATTACHED_SAMLINE=true;
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
-		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
+		ReadWrite.setZipThreads(Shared.threads());
 		Shared.TRIM_READ_COMMENTS=Shared.TRIM_RNAME=true;
 		
 		//Create a parser object
