@@ -3,7 +3,7 @@
 usage(){
 echo "
 Written by Brian Bushnell
-Last modified September 28, 2023
+Last modified April 2, 2024
 
 Description:  Renames reads to <prefix>_<number> where you specify the prefix
 and the numbers are ordered.  There are other renaming modes too.
@@ -42,6 +42,14 @@ fixsra=f            Fixes headers of SRA reads renamed from Illumina.
                     ...into this:
                     HWI-ST79:17:D091UACXX:4:1101:210:824 1:
 
+Trimming:
+trimleft=0          Trim this many characters from the header start.
+trimright=0         Trim this many characters from the header end.
+trimbeforesymbol=0  Trim this many characters before the last instance of
+                    a specified symbol.
+symbol=             Trim before this symbol.  This can be a literal like ':'
+                    or a word like tab or lessthan for reserved symbols.
+
 Sampling parameters:
 reads=-1            Set to a positive number to only process this many INPUT reads (or pairs), then quit.
 
